// (c) 2016, SAS Institute Inc.

jQuery.sap.declare("sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder");


jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayout");
jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayoutArea");
jQuery.sap.require("sas.hc.ui.core.Item");
jQuery.sap.require("sas.hc.ui.layout.VerticalLayout");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.hc.ui.table.Table");

jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.CompositeInput");
jQuery.sap.require("sas.hc.m.Input");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.Select");
jQuery.sap.require("sas.hc.m.SelectDialog");
jQuery.sap.require("sas.hc.m.TextArea");
jQuery.sap.require("sas.hc.m.Toolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");

jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.msa.RiskSourceItem");
jQuery.sap.require("sas.risk.common.msa.Scenario");
jQuery.sap.require("sas.risk.common.msa.WorkGroup");
jQuery.sap.require("sas.risk.common.utils.CommonUtil");
jQuery.sap.require("sas.risk.common.widgets.HelpLabel");
jQuery.sap.require("sas.risk.common.widgets.RiskColumnText");
jQuery.sap.require("sas.risk.common.widgets.RiskObjectPanel");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");


/**
 * Defines a control containing the necessary inputs to build Model Sensitivity Analysis
 * configuration.
 *
 * @name sas.risk.common.widgets.RiskModelSensitivityAnalysisBuilder
 */
sas.hc.ui.commons.layout.BorderLayout.extend("sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder", {
    metadata: {
        properties: {
            allowOutputDataSetName: {type: "boolean", defaultValue: true},

            /**
             * Optional. If set, the workGroup will be displayed alongside the scenario in the builder.
             */
            workGroup: {type: "sas.risk.common.msa.WorkGroup"},

            /**
             * Custom validation function for output data set name. This function
             * is expected to return an object containing two properties:
             * valueState - sap.ui.core.ValueState
             * valueStateText - String
             */
            outputDataSetNameValidator: {type: "function"},

            //
            // help properties
            //

            // consuming applications need to specify their library name, since they provide the help content
            applicationLibraryName: {type: "String", defaultValue: null},

            // help keys
            sourceTypeHelpKey: {type: "String", defaultValue: null},
            shockTypeHelpKey: {type: "String", defaultValue: null},
            shockValuesHelpKey: {type: "String", defaultValue: null},
            floorHelpKey: {type: "String", defaultValue: null},
            capHelpKey: {type: "String", defaultValue: null},
            fadeInHelpKey: {type: "String", defaultValue: null},
            fadeOutHelpKey: {type: "String", defaultValue: null},
        },

        aggregations: {
            scenarios : {type: "sas.risk.common.msa.Scenario", multiple: true, singularName: "scenario"},
            riskFactors: {type: "sas.risk.common.msa.RiskSourceItem", multiple: true, singularName: "riskFactor"},
            riskFactorArrays: {type: "sas.risk.common.msa.RiskSourceItem", multiple: true, singularName: "riskFactorArray"},
            riskFactorGroups: {type: "sas.risk.common.msa.RiskSourceItem", multiple: true, singularName: "riskFactorGroup"},
            riskFactorDrivers: {type: "sas.risk.common.msa.RiskSourceItem", multiple: true, singularName: "riskFactorDriver"},
            instrumentVariables: {type: "sas.risk.common.msa.RiskSourceItem", multiple: true, singularName: "instrumentVariable"}
        },

        events: {
            /**
             * Fired when any input or selection control is changed. Typically this would be
             * used for the consuming application to manage its dirty state.
             */
            inputChange: {
                parameters: {}
            },

            /**
             * Fired when analysis data changes. This includes changes to any required input,
             * such as output data set name (if configured) and shocks.
             */
            analysisChange: {
                parameters: {
                    isValid: {type: "boolean"}  // true if all required analysis data is valid, false otherwise
                }
            }
        }
    },

    renderer: {},

    rb: sap.ui.getCore().getLibraryResourceBundle("sas.risk.common"),

    sourceTypeKeyMap : {
        201: "riskFactor",
        202: "riskFactorArray",
        203: "riskFactorGroup",
        204: "driver",
        301: "instrument",
    },

    shockTypeKeyMap : {
        ABS: "absolute",
        REL: "relative"
    },

    shockValueDelimiter: "\n"
});


// lifecycle functions

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.applySettings = function(mSettings, oScope){
    var self = this;

    // apply settings
    sas.hc.ui.commons.layout.BorderLayout.prototype.applySettings.apply(this, arguments);

    // initialize the model
    self.setModel(new sas.risk.common.models.RiskJSONModel({
        shockInputData: {},
        shocks: []
    }));
    self.getModel().attachPropertyChange($.proxy(self._handleInputChange, self));

    // create selector data
    self._createRiskSourceTypes();
    self._createShockTypes();

    // build and position the controls
    this.setBegin(new sas.hc.ui.commons.layout.BorderLayoutArea({size: "20rem", content: this._createShockInputPanel()}).addStyleClass("riskMSAInputArea"));
    this.setCenter(new sas.hc.ui.commons.layout.BorderLayoutArea({content: this._createConfigurationEntryLayout()}).addStyleClass("riskMSAConfigurationArea"));
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.setScenarios = function(scenarios) {
    var self = this;

    self.removeAllScenarios();
    _.each(scenarios, function(scenario) {
        self.addScenario(scenario);
    });
    this.getModel().setProperty("/scenarios", self.getScenarios());

    this._updateBaselineContent();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.setRiskFactors = function(riskFactors) {
    var self = this;

    self.removeAllRiskFactors();
    _.each(riskFactors, function(riskFactor) {
        self.addRiskFactor(riskFactor);
    });

    self.sourceTypes.riskFactor.sources = self.getRiskFactors();
    self._updateRiskSourceTypes();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.setRiskFactorArrays = function(riskFactorArrays) {
    var self = this;

    self.removeAllRiskFactorArrays();
    _.each(riskFactorArrays, function(riskFactorArray) {
        self.addRiskFactorArray(riskFactorArray);
    });

    self.sourceTypes.riskFactorArray.sources = self.getRiskFactorArrays();
    self._updateRiskSourceTypes();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.setRiskFactorGroups = function(riskFactorGroups) {
    var self = this;

    self.removeAllRiskFactorGroups();
    _.each(riskFactorGroups, function(riskFactorGroup) {
        self.addRiskFactorGroup(riskFactorGroup);
    });

    self.sourceTypes.riskFactorGroup.sources = self.getRiskFactorGroups();
    self._updateRiskSourceTypes();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.setRiskFactorDrivers = function(riskFactorDrivers) {
    var self = this;

    self.removeAllRiskFactorDrivers();
    _.each(riskFactorsDrivers, function(riskFactorDriver) {
        self.addRiskFactorDriver(riskFactorDriver);
    });

    self.sourceTypes.driver.sources = self.getRiskFactorDrivers();
    self._updateRiskSourceTypes();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.setInstrumentVariables = function(instrumentVariables) {
    var self = this;

    self.removeAllInstrumentVariables();
    _.each(instrumentVariables, function(instrumentVariable) {
        self.addInstrumentVariable(instrumentVariable);
    });

    self.sourceTypes.instrument.sources = self.getInstrumentVariables();
    self._updateRiskSourceTypes();
};

///**
// * Destroys the control
// * @private
// */
//sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.exit = function () {
//    sas.hc.ui.commons.layout.BorderLayout.prototype.exit.apply(this, arguments);
//};


// public functions

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.getMSAPackageObject = function() {
    var self = this;
    var model = self.getModel();

    var modelSensitivityAnalysisPackage = {};

    modelSensitivityAnalysisPackage.packageName = model.getProperty("/outputDataSetName");
    modelSensitivityAnalysisPackage.baselineName = model.getProperty("/baselineName");

    modelSensitivityAnalysisPackage.items = [];
    _.each(model.getProperty("/shocks"), function(shock) {
        var packageEntry = {};
        packageEntry.scenarioName = shock.scenarioName;
        packageEntry.sourceType = self.sourceTypes[shock.sourceTypeKey].serverKey;
        packageEntry.sourceName = shock.sourceName;
        packageEntry.perturbType = self.shockTypes[shock.shockTypeKey].serverKey;
        packageEntry.shockValue = Number(shock.shockValue);
        if (shock.cap) {
            packageEntry.cap = Number(shock.cap);
        }
        if (shock.floor) {
            packageEntry.floor = Number(shock.floor);
        }
        if (shock.fadeIn) {
            packageEntry.fadeIn = Number(shock.fadeIn);
        }
        if (shock.fadeOut) {
            packageEntry.fadeOut = Number(shock.fadeOut);
        }
        modelSensitivityAnalysisPackage.items.push(packageEntry);
    });

    return modelSensitivityAnalysisPackage;
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.setMSAPackageObject = function(msaPackage) {
    var self = this;
    var model = self.getModel();

    model.setProperty("/outputDataSetName", msaPackage.packageName);
    model.setProperty("/baselineName", msaPackage.baselineName);

    var shocks = [];
    _.each(msaPackage.items, function(item) {
        var sourceTypeKey = self.sourceTypeKeyMap[item.sourceType];
        var shockTypeKey = self.shockTypeKeyMap[item.perturbType];
        var sourceType = self.sourceTypes[self.sourceTypeKeyMap[item.sourceType]].displayName;
        var shockType = self.shockTypes[self.shockTypeKeyMap[item.perturbType]].displayName;
        shocks.push({
            id: self._getNextShockId(),
            scenarioName: item.scenarioName,
            sourceTypeKey: self.sourceTypeKeyMap[item.sourceType],
            sourceType: self.sourceTypes[self.sourceTypeKeyMap[item.sourceType]].displayName,
            sourceName: item.sourceName,
            shockTypeKey: self.shockTypeKeyMap[item.perturbType],
            shockType: self.shockTypes[self.shockTypeKeyMap[item.perturbType]].displayName,
            shockValue: item.shockValue,
            cap: item.cap,
            floor: item.floor,
            fadeIn: item.fadeIn,
            fadeOut: item.fadeOut
        });
    });
    model.setProperty("/shocks", shocks);

    self._validateShocks();
    self._fireAnalysisChange();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype.clearMSAPackageSelections = function() {
    this.getModel().setProperty("/shockInputData", {});
    this.getModel().setProperty("/outputDataSetName", null);
    this.getModel().setProperty("/baselineName", null);
    this.getModel().setProperty("/shocks", []);

    this._fireAnalysisChange();
};


// content creation

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createShockInputPanel = function() {
    var self = this;

    var layout = new sas.hc.ui.commons.layout.BorderLayout({height: "100%", width: "100%"});

    var centerArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
        content: [
            new sas.hc.ui.layout.VerticalLayout({
                content: [
                    self._createSourceTypeLayout(),
                    self._createSourceNameLayout(),
                    self._createShockTypeLayout(),
                    self._createShockValuesLayout(),
                    self._createOptionalInputsLayout()
                ]
            })
        ]
    });
    layout.setCenter(centerArea);

    self.applyButton = new sas.hc.m.Button({
        text: self.rb.getText("modelSensitivityAnalysisPanel.apply.button.txt"),
        design: sas.hc.m.Button.Design.SmallNeutral,
        enabled: {
            parts: [
                {path: "/shockInputData/sourceTypeKey"},
                {path: "/shockInputData/sourceName"},
                {path: "/shockInputData/shockTypeKey"},
                {path: "/shockInputData/shockValues"},
                {path: "/shockInputData/cap"},
                {path: "/shockInputData/floor"},
                {path: "/shockInputData/fadeIn"},
                {path: "/shockInputData/fadeOut"}
            ],
            formatter: function(sourceTypeKey, sourceName, shockTypeKey, shockValues, cap, floor, fadeIn, fadeOut) {
                if (self.shockValuesTextArea.getValueState() === sap.ui.core.ValueState.None &&
                    self.capInput.getValueState() === sap.ui.core.ValueState.None &&
                    self.floorInput.getValueState() === sap.ui.core.ValueState.None &&
                    self.fadeInInput.getValueState() === sap.ui.core.ValueState.None &&
                    self.fadeOutInput.getValueState() === sap.ui.core.ValueState.None)
                {
                    return !!(sourceTypeKey) && !!(sourceName) && !!(shockTypeKey) && !!(shockValues);
                }
                return false;
            }
        },
        press: function() {
            // create the entries and add them to the table
            self._createShocks();

            // clear the inputs
            self.getModel().setProperty("/shockInputData/sourceTypeKey", "");
            self.getModel().setProperty("/shockInputData/sourceName", null);
            self.getModel().setProperty("/shockInputData/shockTypeKey", "");
            self.getModel().setProperty("/shockInputData/shockValues", null);
            self.getModel().setProperty("/shockInputData/cap", null);
            self.getModel().setProperty("/shockInputData/floor", null);
            self.getModel().setProperty("/shockInputData/fadeIn", null);
            self.getModel().setProperty("/shockInputData/fadeOut", null);
        }
    });
    self.applyButton.addStyleClass("sasPrimaryButton");

    var bottomArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
        size: "2.5rem",
        content: new sas.hc.m.Toolbar({
            content: [
                new sas.hc.m.ToolbarSpacer(),
                self.applyButton,
                new sas.hc.m.ToolbarSpacer({width: "2rem"})
            ]
        }).addStyleClass("riskMSAInputToolbar")
    });
    layout.setBottom(bottomArea);

    return layout;
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createSourceTypeLayout = function() {
    var self = this;

    var sourceTypeLabel;
    if (self.getApplicationLibraryName() && self.getSourceTypeHelpKey()) {
        sourceTypeLabel = new sas.risk.common.widgets.HelpLabel({
            text: self.rb.getText("modelSensitivityAnalysisPanel.sourceType.label.txt"),
            library: self.getApplicationLibraryName(),
            key: self.getSourceTypeHelpKey(),
            required: true
        });
    }
    else {
        sourceTypeLabel = new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.sourceType.label.txt"),
            required: true
        });
    }
    sourceTypeLabel.addStyleClass("riskInputLabel");

    var sourceTypeSelect = new sas.hc.m.Select({
        width: "18rem",
        items: {
            path: "/sourceTypeItems",
            template: new sas.hc.ui.core.Item({
                key: "{key}",
                text: "{displayName}"
            })
        },
        selectedKey: "{/shockInputData/sourceTypeKey}",
        change: function(evt) {
            // clear the variable when the type is changed
            self.getModel().setProperty("/shockInputData/sourceName", null);
        }
    });

    return new sas.hc.ui.layout.VerticalLayout({content: [sourceTypeLabel, sourceTypeSelect]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createSourceNameLayout = function() {
    var self = this;

    var sourceNameLabel = new sas.hc.m.Label({
        text: this.rb.getText("modelSensitivityAnalysisPanel.sourceName.label.txt"),
        required: true
    });
    sourceNameLabel.addStyleClass("riskInputLabel");

    var sourceNameInput = new sas.hc.m.CompositeInput({
        width: "18rem",
        enabled: {
            parts: [{path: "/shockInputData/sourceTypeKey"}],
            formatter: function (sourceTypeKey) {
                return !!(sourceTypeKey);
            }
        },
        inputProperties: {
            type: sap.m.InputType.text,
            editable: false,
            value: "{/shockInputData/sourceName}"
        },
        buttonProperties: {
            icon: sas.icons.HC.OPEN,
            tooltip: this.rb.getText("modelSensitivityAnalysisPanel.sourceName.tooltip"),
            press: function() {
                var riskSourceKey = self.getModel().getProperty("/shockInputData/sourceTypeKey");
                var riskSourceItems = self.sourceTypes[riskSourceKey].sources;
                var selectionItems = [];
                _.each(riskSourceItems, function(riskSourceItem) {
                    selectionItems.push({name: riskSourceItem.getName()});
                });

                var selectionDialog = new sas.hc.m.SelectDialog({
                    title: self.sourceTypes[riskSourceKey].selectDialogTitle,
                    items: {
                        path: "/selectionItems",
                        template: new sas.hc.m.StandardListItem({
                            title: "{name}"
                        })
                    },
                    liveChange: function(evt) {
                        var sValue = evt.getParameter("value");
                        var oFilter = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, sValue);
                        var oBinding = evt.getSource().getBinding("items");
                        oBinding.filter([oFilter]);
                    },
                    confirm: function(itemSelectionEvent) {
                        var selectedItem = itemSelectionEvent.getParameter("selectedItem");
                        self.getModel().setProperty("/shockInputData/sourceName", selectedItem.getTitle(), true);
                        self._handleInputChange();
                        selectionDialog.destroy();
                    },
                    cancel: function(evt) {
                        selectionDialog.destroy();
                    }
                });
                selectionDialog.setModel(new sas.risk.common.models.RiskJSONModel({selectionItems: selectionItems}));
                selectionDialog.open();
            }
        }
    });

    return new sas.hc.ui.layout.VerticalLayout({content: [sourceNameLabel, sourceNameInput]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createShockTypeLayout = function() {
    var shockTypeLabel;
    if (this.getApplicationLibraryName() && this.getShockTypeHelpKey()) {
        shockTypeLabel = new sas.risk.common.widgets.HelpLabel({
            text: this.rb.getText("modelSensitivityAnalysisPanel.shockType.label.txt"),
            library: this.getApplicationLibraryName(),
            key: this.getShockTypeHelpKey(),
            required: true
        });
    }
    else {
        shockTypeLabel = new sas.hc.m.Label({
            text: this.rb.getText("modelSensitivityAnalysisPanel.shockType.label.txt"),
            required: true
        });
    }
    shockTypeLabel.addStyleClass("riskInputLabel");

    var shockTypeSelect = new sas.hc.m.Select({
        width: "18rem",
        items: this._getShockTypeItems(),
        selectedKey: "{/shockInputData/shockTypeKey}",
    });

    return new sas.hc.ui.layout.VerticalLayout({content: [shockTypeLabel, shockTypeSelect]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createShockValuesLayout = function() {
    var self = this;

    var shockValuesLabel;
    if (self.getApplicationLibraryName() && self.getShockValuesHelpKey()) {
        shockValuesLabel = new sas.risk.common.widgets.HelpLabel({
            text: self.rb.getText("modelSensitivityAnalysisPanel.shockValues.label.txt"),
            library: self.getApplicationLibraryName(),
            key: self.getShockValuesHelpKey(),
            required: true
        });
    }
    else {
        shockValuesLabel = new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.shockValues.label.txt"),
            required: true
        });
        shockValuesLabel.addStyleClass("riskInputLabel");
    }

    var _validateShockValues = function(shockValuesString) {
        if (!self._isValidShockValues(shockValuesString)) {
            self.shockValuesTextArea.setValueState(sap.ui.core.ValueState.Error);
            self.shockValuesTextArea.setValueStateText(self.rb.getText("modelSensitivityAnalysisPanel.shockValues.valueStateMessage.txt"));
        }
        else {
            self.shockValuesTextArea.setValueState(sap.ui.core.ValueState.None);
            self.shockValuesTextArea.setValueStateText(null);
        }
    };

    self.shockValuesTextArea = new sas.hc.m.TextArea({
        width: "18rem",
        rows: 5,
        maxLength: 1024,
        value: "{/shockInputData/shockValues}",
        liveChange: function(event) {
            _validateShockValues(self.shockValuesTextArea.getValue());
        },
        change: function(event) {
            _validateShockValues(self.shockValuesTextArea.getValue());
        }
    });

    // disable internal validation as it clears the value state we set even though the
    // shock values are not valid
    // TODO (ryalbe) open a defect for this? Similar issue as with inputs
    self.shockValuesTextArea.detachLiveChange(self.shockValuesTextArea._handleLiveChange);

    // Override this so we can make sure the shock values are validated. This is necessary
    // because the liveChange doesn't validate unless the user types a trailing space,
    // meaning the user may have a previously invalid value that they removed and added
    // back in, which would not register as a change and not fire when the user leaves
    // the textarea. This is a clone of the TextArea onChange method with the else clause
    // added to force validation.
    self.shockValuesTextArea.onChange = function(oEvent) {

        // check the control is editable or not
        if (!this.getEditable() || !this.getEnabled()) {
            return;
        }

        // get the dom value respect to max length
        var sValue = this._getInputValue();

        // compare with the old known value
        if (sValue !== this._lastValue) {

            // save the value on change
            this.setProperty("value", sValue, true);

            // get the value back maybe formatted
            sValue = this.getValue();

            // remember the last value on change
            this._lastValue = sValue;

            // fire change event
            this.fireChangeEvent(sValue);

            // inform change detection
            return true;
        }
        else {
            _validateShockValues(self.shockValuesTextArea.getValue());
        }
    };

    return new sas.hc.ui.layout.VerticalLayout({content: [shockValuesLabel, self.shockValuesTextArea]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createOptionalInputsLayout = function() {
    var optionalInputsLayout = new sap.ui.commons.layout.MatrixLayout({
        layoutFixed: true,
        columns: 2,
        width: "18rem",
        widths: ["50%", "50%"]
    });

    var floorInputCell = new sas.hc.ui.commons.layout.MatrixLayoutCell({
        backgroundDesign: sap.ui.commons.layout.BackgroundDesign.Transparent,
        hAlign: sap.ui.commons.layout.HAlign.Begin,
        padding: sap.ui.commons.layout.Padding.None,
        vAlign: sap.ui.commons.layout.VAlign.Middle,
        content: [this._createFloorInputLayout()]
    });
    var capInputCell = new sas.hc.ui.commons.layout.MatrixLayoutCell({
        backgroundDesign: sap.ui.commons.layout.BackgroundDesign.Transparent,
        hAlign: sap.ui.commons.layout.HAlign.Begin,
        padding: sap.ui.commons.layout.Padding.None,
        vAlign: sap.ui.commons.layout.VAlign.Middle,
        content: [this._createCapInputLayout()]
    });
    optionalInputsLayout.createRow(floorInputCell, capInputCell);

    var fadeInInputCell = new sas.hc.ui.commons.layout.MatrixLayoutCell({
        backgroundDesign: sap.ui.commons.layout.BackgroundDesign.Transparent,
        hAlign: sap.ui.commons.layout.HAlign.Begin,
        padding: sap.ui.commons.layout.Padding.None,
        vAlign: sap.ui.commons.layout.VAlign.Middle,
        content: [this._createFadeInInputLayout()]
    });
    var fadeOutInputCell = new sas.hc.ui.commons.layout.MatrixLayoutCell({
        backgroundDesign: sap.ui.commons.layout.BackgroundDesign.Transparent,
        hAlign: sap.ui.commons.layout.HAlign.Begin,
        padding: sap.ui.commons.layout.Padding.None,
        vAlign: sap.ui.commons.layout.VAlign.Middle,
        content: [this._createFadeOutInputLayout()]
    });
    optionalInputsLayout.createRow(fadeInInputCell, fadeOutInputCell);

    return optionalInputsLayout;
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createCapInputLayout = function() {
    var self = this;

    var capLabel;
    if (self.getApplicationLibraryName() && self.getCapHelpKey()) {
        capLabel = new sas.risk.common.widgets.HelpLabel({
            text: self.rb.getText("modelSensitivityAnalysisPanel.cap.label.txt"),
            library: self.getApplicationLibraryName(),
            key: self.getCapHelpKey()
        });
    }
    else {
        capLabel = new sas.hc.m.Label({
            text: this.rb.getText("modelSensitivityAnalysisPanel.cap.label.txt")
        });
    }
    capLabel.addStyleClass("riskInputLabel");

    self.capInput = new sas.hc.m.Input({
        width: "8rem",
        type: sap.m.InputType.Number,
        maxLength: 17,  // 15 significant digit limit plus decimal and negative
        enableGroupingSeparator: false,
        value: "{/shockInputData/cap}",
        valueLiveUpdate: true,
        valueState: {
            parts: [
                {path: "/shockInputData/cap"},
                {path: "/shockInputData/floor"},
            ],
            formatter: function(capValue, floorValue) {
                if (!self._isValidCapOrFloorValue(capValue) || !self._isValidCapAndFloorRange(capValue, floorValue)) {
                    return sap.ui.core.ValueState.Error;
                }
                else {
                    return sap.ui.core.ValueState.None;
                }
            }
        },
        valueStateText: {
            parts: [
                {path: "/shockInputData/cap"},
                {path: "/shockInputData/floor"},
            ],
            formatter: function(capValue, floorValue) {
                if (!self._isValidCapOrFloorValue(capValue)) {
                    return self.rb.getText("modelSensitivityAnalysisPanel.capFloor.valueStateMessage.txt");
                }
                else if (!self._isValidCapAndFloorRange(capValue, floorValue)) {
                    return self.rb.getText("modelSensitivityAnalysisPanel.capRangeError.valueStateMessage.txt");
                }
                else {
                    return null;
                }
            }
        },
        selectTextOnPress: false
    });
    /*
     * FIXME this is a hack for HC 2.x
     * The commons input component calls an internal function _performValidation when
     * focus is removed (see sas.hc.m.Input.prototype.onfocusout). This replaces the
     * value state and value state text with whatever the component validation resolves
     * to. The component validation is by field type, in our case number, so as long as
     * the input value is a valid number, when focus is removed the fields is restored
     * to a valid state. Recommend HC provide a way to specify custom validation logic
     * to run along with or in place of the current internal validation scheme.
     *
     * Defect: S1195414
     */
    self.capInput._performValidation = function(){};    // disable commons validation that removes our validation
    self.capInput._performLiveValidation = function(){};    // disable commons validation that removes our validation

    return new sas.hc.ui.layout.VerticalLayout({content: [capLabel, self.capInput]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createFloorInputLayout = function() {
    var self = this;

    var floorLabel;
    if (self.getApplicationLibraryName() && self.getFloorHelpKey()) {
        floorLabel = new sas.risk.common.widgets.HelpLabel({
            text: self.rb.getText("modelSensitivityAnalysisPanel.floor.label.txt"),
            library: self.getApplicationLibraryName(),
            key: self.getFloorHelpKey()
        });
    }
    else {
        floorLabel = new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.floor.label.txt")
        });
    }
    floorLabel.addStyleClass("riskInputLabel");

    self.floorInput = new sas.hc.m.Input({
        width: "8rem",
        type: sap.m.InputType.Number,
        maxLength: 17,  // 15 significant digit limit plus decimal and negative
        enableGroupingSeparator: false,
        value: "{/shockInputData/floor}",
        valueLiveUpdate: true,
        valueState: {
            parts: [
                {path: "/shockInputData/cap"},
                {path: "/shockInputData/floor"},
            ],
            formatter: function(capValue, floorValue) {
                if (!self._isValidCapOrFloorValue(floorValue) || !self._isValidCapAndFloorRange(capValue, floorValue)) {
                    return sap.ui.core.ValueState.Error;
                }
                else {
                    return sap.ui.core.ValueState.None;
                }
            }
        },
        valueStateText: {
            parts: [
                {path: "/shockInputData/cap"},
                {path: "/shockInputData/floor"},
            ],
            formatter: function(capValue, floorValue) {
                if (!self._isValidCapOrFloorValue(floorValue)) {
                    return self.rb.getText("modelSensitivityAnalysisPanel.capFloor.valueStateMessage.txt");
                }
                else if (!self._isValidCapAndFloorRange(capValue, floorValue)) {
                    return self.rb.getText("modelSensitivityAnalysisPanel.floorRangeError.valueStateMessage.txt");
                }
                else {
                    return null;
                }
            }
        },
        selectTextOnPress: false
    });
    /*
     * FIXME this is a hack for HC 2.x
     * The commons input component calls an internal function _performValidation when
     * focus is removed (see sas.hc.m.Input.prototype.onfocusout). This replaces the
     * value state and value state text with whatever the component validation resolves
     * to. The component validation is by field type, in our case number, so as long as
     * the input value is a valid number, when focus is removed the fields is restored
     * to a valid state. Recommend HC provide a way to specify custom validation logic
     * to run along with or in place of the current internal validation scheme.
     *
     * Defect: S1195414
     */
    self.floorInput._performValidation = function(){};    // disable commons validation that removes our validation
    self.floorInput._performLiveValidation = function(){};    // disable commons validation that removes our validation

    return new sas.hc.ui.layout.VerticalLayout({content: [floorLabel, self.floorInput]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createFadeInInputLayout = function() {
    var self = this;

    var fadeInLabel;
    if (self.getApplicationLibraryName() && self.getFadeInHelpKey()) {
        fadeInLabel = new sas.risk.common.widgets.HelpLabel({
            text: self.rb.getText("modelSensitivityAnalysisPanel.fadeIn.label.txt"),
            library: self.getApplicationLibraryName(),
            key: self.getFadeInHelpKey()
        });
    }
    else {
        fadeInLabel = new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.fadeIn.label.txt")
        });
    }
    fadeInLabel.addStyleClass("riskInputLabel");

    self.fadeInInput = new sas.hc.m.Input({
        width: "8rem",
        type: sap.m.InputType.Number,
        enableGroupingSeparator: false,
        value: "{/shockInputData/fadeIn}",
        valueLiveUpdate: true,
        maxLength: 10,
        selectTextOnPress: false,
        enabled: {
            parts: [{path: "/shockInputData/sourceTypeKey"}],
            formatter: function(sourceTypeKey) {
                if (sourceTypeKey === self.sourceTypes.instrument.key) {
                    self.getModel().setProperty("/shockInputData/fadeIn", null);
                    return false;
                }
                return true;
            }
        },
        liveChange: function(event) {
            if (self._isValidFadeValue(event.getParameter("newValue"))) {
                self.fadeInInput.setValueState(sap.ui.core.ValueState.None);
                self.fadeInInput.setValueStateText(null);
            }
            else {
                self.fadeInInput.setValueState(sap.ui.core.ValueState.Error);
                self.fadeInInput.setValueStateText("Specify an integer from 1 to 2147483647.");
            }
        }
    });
    /*
     * FIXME this is a hack for HC 2.x
     * The commons input component calls an internal function _performValidation when
     * focus is removed (see sas.hc.m.Input.prototype.onfocusout). This replaces the
     * value state and value state text with whatever the component validation resolves
     * to. The component validation is by field type, in our case number, so as long as
     * the input value is a valid number, when focus is removed the fields is restored
     * to a valid state. Recommend HC provide a way to specify custom validation logic
     * to run along with or in place of the current internal validation scheme.
     *
     * Defect: S1195414
     */
    self.fadeInInput._performValidation = function(){};    // disable commons validation that removes our validation
    self.fadeInInput._performLiveValidation = function(){};    // disable commons validation that removes our validation

    return new sas.hc.ui.layout.VerticalLayout({content: [fadeInLabel, self.fadeInInput]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createFadeOutInputLayout = function() {
    var self = this;

    var fadeOutLabel;
    if (self.getApplicationLibraryName() && self.getFadeOutHelpKey()) {
        fadeOutLabel = new sas.risk.common.widgets.HelpLabel({
            text: self.rb.getText("modelSensitivityAnalysisPanel.fadeOut.label.txt"),
            library: self.getApplicationLibraryName(),
            key: self.getFadeOutHelpKey()
        });
    }
    else {
        fadeOutLabel = new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.fadeOut.label.txt")
        });
    }
    fadeOutLabel.addStyleClass("riskInputLabel");

    self.fadeOutInput = new sas.hc.m.Input({
        width: "8rem",
        type: sap.m.InputType.Number,
        enableGroupingSeparator: false,
        value: "{/shockInputData/fadeOut}",
        valueLiveUpdate: true,
        maxLength: 10,
        selectTextOnPress: false,
        enabled: {
            parts: [{path: "/shockInputData/sourceTypeKey"}],
            formatter: function(sourceTypeKey) {
                if (sourceTypeKey === self.sourceTypes.instrument.key) {
                    self.getModel().setProperty("/shockInputData/fadeOut", null);
                    return false;
                }
                return true;
            }
        },
        liveChange: function(event) {
            if (self._isValidFadeValue(event.getParameter("newValue"))) {
                self.fadeOutInput.setValueState(sap.ui.core.ValueState.None);
                self.fadeOutInput.setValueStateText(null);
            }
            else {
                self.fadeOutInput.setValueState(sap.ui.core.ValueState.Error);
                self.fadeOutInput.setValueStateText("Specify an integer from 1 to 2147483647.");
            }
        }
    });
    /*
     * FIXME this is a hack for HC 2.x
     * The commons input component calls an internal function _performValidation when
     * focus is removed (see sas.hc.m.Input.prototype.onfocusout). This replaces the
     * value state and value state text with whatever the component validation resolves
     * to. The component validation is by field type, in our case number, so as long as
     * the input value is a valid number, when focus is removed the fields is restored
     * to a valid state. Recommend HC provide a way to specify custom validation logic
     * to run along with or in place of the current internal validation scheme.
     *
     * Defect: S1195414
     */
    self.fadeOutInput._performValidation = function(){};    // disable commons validation that removes our validation
    self.fadeOutInput._performLiveValidation = function(){};    // disable commons validation that removes our validation

    return new sas.hc.ui.layout.VerticalLayout({content: [fadeOutLabel, self.fadeOutInput]}).addStyleClass("riskInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createPackageInputsLayout = function() {
    var layoutContent = [];

    if (this.getWorkGroup()) {
        layoutContent.push(this._createWorkGroupLayout());
    }

    layoutContent.push(this._createBaselineLayout(this.getScenarios()));

    if (this.getAllowOutputDataSetName() === true) {
        layoutContent.push(this._createOutputDataSetInputLayout());
    }

    return new sas.hc.ui.layout.HorizontalLayout({content: layoutContent}).addStyleClass("msaPackageInputLayout");
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createBaselineLayout = function() {
    return this.baselineLayout = new sas.hc.ui.layout.VerticalLayout({content: [this._createBaselineContent()]});
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createBaselineContent = function() {
    var self = this;
    var baselineContent = [];

    var baselineScenarioLabel = new sas.hc.m.Label({
        text: this.rb.getText("modelSensitivityAnalysisPanel.baselineScenario.label.txt"),
        required: true
    });
    baselineScenarioLabel.addStyleClass("riskInputLabel");
    baselineContent.push(baselineScenarioLabel);

    var baselineScenarioValueControl;
    var scenarios = this.getScenarios();
    if (scenarios && scenarios.length === 1) {
        baselineScenarioValueControl = new sas.hc.m.Label({
            text: scenarios[0].getDisplayName()
        });
        baselineScenarioValueControl.addStyleClass("riskValueLabel");
        this.getModel().setProperty("/baselineName", scenarios[0].getProperty("name"));
    }
    else {
        var scenarioItems = [];
        scenarioItems.push(new sas.hc.ui.core.Item({
            key: "",
            text: ""
        }));
        _.each(scenarios, function(scenario) {
            scenarioItems.push(
                new sas.hc.ui.core.Item({
                    key: scenario.getName(),
                    text: scenario.getDisplayName()
                })
            );
            scenarioItems[scenarioItems.length - 1].name = scenario.getName();
        });

        baselineScenarioValueControl = new sas.hc.m.Select({
            width: "15rem",
            items: scenarioItems,
            selectedKey: "{/baselineName}",
            change: function(evt) {
                self._fireAnalysisChange();
            }
        });
    }
    baselineContent.push(baselineScenarioValueControl);

    return baselineContent;
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._updateBaselineContent = function() {
    this.baselineLayout.destroyContent();

    var self = this;
    _.each(this._createBaselineContent(), function(baselineContentItem) {
        self.baselineLayout.addContent(baselineContentItem);
    });
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createWorkGroupLayout = function() {
    var self = this;

    var workGroupLabel = new sas.hc.m.Label({
        text: this.rb.getText("modelSensitivityAnalysisPanel.workGroup.label.txt"),
    });
    workGroupLabel.addStyleClass("riskInputLabel");

    var workGroupValueLabel = new sas.hc.m.Label({
        text: this.getWorkGroup().getDisplayName()
    }).addStyleClass("riskValueLabel");

    return new sas.hc.ui.layout.VerticalLayout({content: [workGroupLabel, workGroupValueLabel]});
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createOutputDataSetInputLayout = function() {
    var self = this;

    var outputDataSetNameLabel = new sas.hc.m.Label({
        text: self.rb.getText("modelSensitivityAnalysisPanel.outputDatasetName.label.txt"),
        required: true
    });
    outputDataSetNameLabel.addStyleClass("riskInputLabel");

    self.outputDataSetNameInput = new sas.hc.m.Input({
        width: "15rem",
        type: sap.m.InputType.Text,
        disableValidation: false,  // disable default input validation
        value: "{/outputDataSetName}",
        valueLiveUpdate: true,
        maxLength: 32,
        selectTextOnPress: false,
        liveChange: function (event) {
            var outputDataSetNameValue = event.getParameter("value");
            if (outputDataSetNameValue && !sas.risk.common.utils.CommonUtil.isValidSASNameWithoutDot(outputDataSetNameValue)) {
                self.outputDataSetNameInput.setValueState(sap.ui.core.ValueState.Error);
                self.outputDataSetNameInput.setValueStateText(self.rb.getText("modelSensitivityAnalysisPanel.outputDatasetName.valueStateText.txt"));
            }
            else {
                var customValidator = self.getOutputDataSetNameValidator();
                if (typeof customValidator === "function") {
                    var validatorState = customValidator(outputDataSetNameValue);
                    self.outputDataSetNameInput.setValueState(validatorState.valueState);
                    self.outputDataSetNameInput.setValueStateText(validatorState.valueStateText);
                }
                else {
                    self.outputDataSetNameInput.setValueState(sap.ui.core.ValueState.None);
                    self.outputDataSetNameInput.setValueStateText(null);
                }
            }
            self._fireAnalysisChange();
        }
    });

    return new sas.hc.ui.layout.VerticalLayout({content: [outputDataSetNameLabel, self.outputDataSetNameInput]});
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createConfigurationEntryLayout = function() {
    var layout = new sas.hc.ui.commons.layout.BorderLayout({height: "100%", width: "100%"});

    var topArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
        size: "4.125rem",
        content: this._createPackageInputsLayout()
    });
    layout.setTop(topArea);

    var centerArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
        content: this._createConfigurationEntryTable()
    });
    layout.setCenter(centerArea);

    return layout;
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createShocksTableToolbar = function() {
    var self = this;

    self.removeShocksButton = new sas.hc.m.Button({
        icon: sas.icons.HC.DELETE,
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.delete.tooltip"),
        enabled: false,
        press: function() {
            var selectedIndices = self.shocksTable.getSelectedIndices();
            var selectedEntries = [];
            _.each(selectedIndices, function(selectedIndex) {
                var context = self.shocksTable.getContextByIndex(selectedIndex);
                var path = context.sPath;
                selectedEntries.push(self.shocksTable.getModel().getProperty(path));
            });

            var updatedEntries = [];
            _.each(self.getModel().getProperty("/shocks"), function(entry) {
                if(sas.risk.common.utils.CommonUtil.findIndex(selectedEntries, function(selectedEntry) {return selectedEntry.id === entry.id;}) === -1) {
                    updatedEntries.push(entry);
                }
            });

            self.shocksTable.clearSelection();

            self.getModel().setProperty("/shocks", updatedEntries);
            self._handleShocksChanged();
            self._handleInputChange();
        }
    });

    return new sas.hc.m.Toolbar({
        content: [
            new sas.hc.m.Label({
                text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.title.txt")
            }).addStyleClass("riskMSAConfigurationTableTitle"),
            new sas.hc.m.ToolbarSpacer(),
            self.removeShocksButton
        ]
    });
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createConfigurationEntryTable = function() {
    var self = this;

    self.shocksTable = new sas.hc.ui.table.Table({
        width: "100%",
        selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
        showSelectionControls: true,
        showTableOptions: false,
        enableColumnReordering: false,
        enableColumnFreeze: false,
        toolbar: self._createShocksTableToolbar(),
        visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
        minAutoRowCount: 1, // required for scrolling with < 5 rows
        rowHeight: 38,  // SAP is setting checkbox cell height to 38, so do this to match
        rowSelectionChange: function(evt) {
            self.removeShocksButton.setEnabled(self.shocksTable.getSelectedIndices().length > 0);
        }
    });
    self.shocksTable.allowTextSelection(false);

    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('scenarioName'),{
        width: "12rem",
        label: new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.scenarioName.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.scenarioName.column.txt"),
        template: self._createShockNameFieldTemplate(),
        sortProperty: "scenarioName",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('sourceType'),{
        width: "10rem",
        label: new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.sourceType.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.sourceType.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({text: "{sourceType}"}),
        sortProperty: "sourceType",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('sourceName'),{
        width: "10rem",
        label: new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.sourceName.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.sourceName.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({text: "{sourceName}"}),
        sortProperty: "sourceName",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('shockType'),{
        width: "7rem",
        label: new sas.hc.m.Label({
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.shockType.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.shockType.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({text: "{shockType}"}),
        sortProperty: "shockType",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('shockValue'),{
        width: "7rem",
        label: new sas.hc.m.Label({
            textAlign: sap.ui.core.TextAlign.Right,
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.shockValue.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.shockValue.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({
            textAlign: sap.ui.core.TextAlign.Right,
            text: "{shockValue}"
        }),
        sortProperty: "shockValue",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('floor'),{
        width: "6rem",
        label: new sas.hc.m.Label({
            textAlign: sap.ui.core.TextAlign.Right,
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.floor.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.floor.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({
            textAlign: sap.ui.core.TextAlign.Right,
            text: "{floor}"
        }),
        sortProperty: "floor",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('cap'),{
        width: "6rem",
        label: new sas.hc.m.Label({
            textAlign: sap.ui.core.TextAlign.Right,
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.cap.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.cap.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({
            textAlign: sap.ui.core.TextAlign.Right,
            text: "{cap}"
        }),
        sortProperty: "cap",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('fadeIn'),{
        width: "6rem",
        label: new sas.hc.m.Label({
            textAlign: sap.ui.core.TextAlign.Right,
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.fadeIn.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.fadeIn.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({
            textAlign: sap.ui.core.TextAlign.Right,
            text: "{fadeIn}"
        }),
        sortProperty: "fadeIn",
        showFilterMenuEntry: false
    }));
    self.shocksTable.addColumn(new sas.hc.ui.table.Column(self._createId('fadeOut'),{
        width: "6rem",
        label: new sas.hc.m.Label({
            textAlign: sap.ui.core.TextAlign.Right,
            text: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.fadeOut.column.txt")
        }),
        tooltip: self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.fadeOut.column.txt"),
        template: new sas.risk.common.widgets.RiskColumnText({
            textAlign: sap.ui.core.TextAlign.Right,
            text: "{fadeOut}"
        }),
        sortProperty: "fadeOut",
        showFilterMenuEntry: false
    }));

    self.shocksTable.bindRows("/shocks");

    return self.shocksTable;
};


// internal data management

/**
 * Builds the internal set of risk types.
 * @param riskSourceData
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createRiskSourceTypes = function() {
    this.sourceTypes = {};

    this._createRiskFactorType();
    this._createRiskFactorArrayType();
    this._createRiskFactorGroupType();
    this._createRiskFactorDriverType();
    this._createInstrumentVariableType();

    this._updateRiskSourceTypes();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createRiskFactorType = function() {
    this.sourceTypes.riskFactor = {
        key: "riskFactor",
        serverKey: 201,
        displayName: this.rb.getText("modelSensitivityAnalysisPanel.sourceType.riskFactor.txt"),
        selectDialogTitle: this.rb.getText("modelSensitivityAnalysisPanel.riskSource.riskFactors.dialog.title.txt"),
        sources: this.getRiskFactors()
    };
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createRiskFactorArrayType = function() {
    this.sourceTypes.riskFactorArray = {
        key: "riskFactorArray",
        serverKey: 202,
        displayName: this.rb.getText("modelSensitivityAnalysisPanel.sourceType.riskFactorArray.txt"),
        selectDialogTitle: this.rb.getText("modelSensitivityAnalysisPanel.riskSource.riskFactorArrays.dialog.title.txt"),
        sources: this.getRiskFactorArrays()
    };
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createRiskFactorGroupType = function() {
    this.sourceTypes.riskFactorGroup = {
        key: "riskFactorGroup",
        serverKey: 203,
        displayName: this.rb.getText("modelSensitivityAnalysisPanel.sourceType.riskFactorGroup.txt"),
        selectDialogTitle: this.rb.getText("modelSensitivityAnalysisPanel.riskSource.riskFactorGroups.dialog.title.txt"),
        sources: this.getRiskFactorGroups()
    };
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createRiskFactorDriverType = function() {
    this.sourceTypes.driver = {
        key: "driver",
        serverKey: 204,
        displayName: this.rb.getText("modelSensitivityAnalysisPanel.sourceType.driver.txt"),
        selectDialogTitle: this.rb.getText("modelSensitivityAnalysisPanel.riskSource.drivers.dialog.title.txt"),
        sources: this.getRiskFactorDrivers()
    };
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createInstrumentVariableType = function() {
    this.sourceTypes.instrument = {
        key: "instrument",
        serverKey: 301,
        displayName: this.rb.getText("modelSensitivityAnalysisPanel.sourceType.instrument.txt"),
        selectDialogTitle: this.rb.getText("modelSensitivityAnalysisPanel.riskSource.instruments.dialog.title.txt"),
        sources: this.getInstrumentVariables()
    };
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._updateRiskSourceTypes = function() {
    var sourceTypeItems = [];

    sourceTypeItems.push({key: "", displayName: ""});
    _.each(this.sourceTypes, function(sourceType) {
        if (sourceType.sources && sourceType.sources.length > 0) {
            sourceTypeItems.push(sourceType);
        }
    });

    this.getModel().setProperty("/sourceTypeItems", sourceTypeItems);
};

/**
 * Builds the internal set of shock types.
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createShockTypes = function() {
    this.shockTypes = {};

    this.shockTypes.absolute = {
        key: "absolute",
        serverKey: "ABS",
        displayName: this.rb.getText("modelSensitivityAnalysisPanel.shockType.absolute.txt")
    };

    this.shockTypes.relative = {
        key: "relative",
        serverKey: "REL",
        displayName: this.rb.getText("modelSensitivityAnalysisPanel.shockType.relative.txt")
    };
};

/**
 * Returns the shock type items for the shock type select control. Built from the internal set of
 * shock types.
 *
 * @returns {Array}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._getShockTypeItems = function() {
    var shockTypeItems = [];

    shockTypeItems.push(new sas.hc.ui.core.Item({
        key: "",
        text: ""
    }));

    _.each(this.shockTypes, function(shockType) {
        shockTypeItems.push(new sas.hc.ui.core.Item({
            key: shockType.key,
            text: shockType.displayName
        }));
    });

    return shockTypeItems;
};

/**
 *
 * @returns {number}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._getNextShockId = function() {
    if(!this.shockIdCounter) {
        this.shockIdCounter = 0;
    }
    this.shockIdCounter++;
    return this.shockIdCounter;
};

/**
 * Creates an analysis configuration entry for each shock value specified and sets them on the model.
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createShocks = function() {
    var self = this;

    var dataModel = self.getModel();
    var shocks = dataModel.getProperty("/shocks");
    var shockInputData = self.getModel().getProperty("/shockInputData");

    var shockValueStrings = shockInputData.shockValues.trim().split(this.shockValueDelimiter);

    _.each(shockValueStrings, function(shockValueString) {
        var shock = {};

        shock.id = self._getNextShockId();
        shock.scenarioName = self._generateShockName(
            shockInputData.sourceName,
            shockInputData.shockTypeKey,
            shockValueString
        );
        shock.sourceTypeKey = shockInputData.sourceTypeKey;
        shock.sourceType = self.sourceTypes[shockInputData.sourceTypeKey].displayName;
        shock.sourceName = shockInputData.sourceName;
        shock.shockTypeKey = shockInputData.shockTypeKey;
        shock.shockType = self.shockTypes[shockInputData.shockTypeKey].displayName;
        shock.shockValue = Number(shockValueString);
        shock.cap = shockInputData.cap;
        shock.floor = shockInputData.floor;
        shock.fadeIn = shockInputData.fadeIn;
        shock.fadeOut = shockInputData.fadeOut;

        shocks.push(shock);
    });

    // process the entries to make sure names are unique
    self._handleShocksChanged();

    // update the model
    dataModel.setProperty("/shocks", shocks);
    self._handleInputChange();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._validateShocks = function() {
    var self = this;

    var shockNames = {};
    var shocks = self.getModel().getProperty("/shocks");

    // build up an object with indexes keyed by scenarioName
    var index = 0;
    _.each(shocks, function(shock) {
        // if shock name is missing, set it to an error state
        if (!shock.scenarioName) {
            self.getModel().setProperty("/shocks/" + index + "/valueState", sap.ui.core.ValueState.Error);
            self.getModel().setProperty("/shocks/" + index + "/valueStateText", self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.scenarioNameMissing.column.error.tooltip"));
        }
        else if (shockNames[shock.scenarioName.toUpperCase()]) {
            shockNames[shock.scenarioName.toUpperCase()].push(index);
        }
        else {
            shockNames[shock.scenarioName.toUpperCase()] = [index];
        }
        index++;
    });

    // figure out if there are duplicate shock names
    _.each(Object.keys(shockNames), function(shockName) {
        var scenarioNameIndexes = shockNames[shockName];
        if (scenarioNameIndexes.length > 1) {
            _.each(scenarioNameIndexes, function(scenarioNameIndex) {
                self.getModel().setProperty("/shocks/" + scenarioNameIndex + "/valueState", sap.ui.core.ValueState.Error);
                self.getModel().setProperty("/shocks/" + scenarioNameIndex + "/valueStateText", self.rb.getText("modelSensitivityAnalysisPanel.shocks.table.scenarioNameDuplicate.column.error.tooltip"));
            });
        }
        else {
            self.getModel().setProperty("/shocks/" + scenarioNameIndexes[0] + "/valueState", sap.ui.core.ValueState.None);
            self.getModel().setProperty("/shocks/" + scenarioNameIndexes[0] + "/valueStateText", null);
        }
    });
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._handleShocksChanged = function() {
    this._validateShocks();
    this._fireAnalysisChange();
};


// utility functions

/**
 * Creates a unique internal id for this control from the specified baseId.
 * @param baseId
 * @returns {string}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createId = function(baseId) {
    var myId = this.getId();
    return myId ? myId + "-" + baseId : baseId;
};

/**
 * Generates the scenarioName for the analysis configuration entry. The scenarioName is in the following format:
 *
 *     [sourceName]_[p|m][shockValue][P*]
 *
 * [p|m] - p is applied if the number is positive, m is applied if negative.
 * [P*] - applied only if the shock values are Relative
 *
 * @param {String} sourceName
 * @param {String} shockTypeKey
 * @param {String} shockValue
 * @returns {string}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._generateShockName = function(sourceName, shockTypeKey, shockValue) {
    var shockSuffix = "_" + (shockValue < 0 ? ("m" + (shockValue * -1)) : "p" + shockValue);

    if(shockTypeKey === "relative") {
        shockSuffix += "P";
    }

    var sourceNameLength = sourceName.length;
    var shockSuffixLength = shockSuffix.length;
    var untrimmedLength = sourceNameLength + shockSuffixLength;

    return untrimmedLength <= 32 ? (sourceName + shockSuffix) : sourceName.substring(0, (sourceNameLength - (untrimmedLength - 32))) + shockSuffix;
};

/**
 * Returns true if the specified value is a valid cap or floor value (+/- number of 13 digits or fewer).
 * @param {String} value
 * @returns {*|boolean}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._isValidCapOrFloorValue = function(value) {
    if (!value) {
        return true;
    }
    else if (!isNaN(Number(value))) {
        var valueArray = value.match(/[0-9]/g);
        return valueArray && valueArray.length <= 15;
    }
    return false;
};

/**
 * Returns true if the specified capValue is greater than the specified floor value.
 * @param {String} capValue
 * @param {String} floorValue
 * @returns {boolean}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._isValidCapAndFloorRange = function(capValue, floorValue) {
    if (!!capValue && !!floorValue) {
        return Number(capValue) > Number(floorValue);
    }
    return true;
};

/**
 * Returns true if the specified value is a valid fade-in or fade-out value (posivitive integet less than or equal to 1000).
 * @param {String} value
 * @returns {*|boolean}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._isValidFadeValue = function(value) {
    return !value || (sas.risk.common.utils.CommonUtil.isPositiveInteger(value) && parseInt(value) <= sas.risk.common.utils.CommonConstants.MAX_HORIZON);
};

/**
 * Returns true if all shock values are valid (space separated numeric values).
 * @param {String} shockValuesString
 * @returns {*|boolean}
 * @private
 */
sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._isValidShockValues = function(shockValuesString) {
    var shockValues = shockValuesString.trim().split(this.shockValueDelimiter);
    return _.every(shockValues, function(shockValue) {
        if(!shockValue) {
            return true;
        }
        else if(!isNaN(Number(shockValue))) {
            var shockValueNumberArray = shockValue.match(/[0-9]/g);
            return shockValueNumberArray && shockValueNumberArray.length <= 15;
        }
        return false;
    });
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._createShockNameFieldTemplate = function() {
    var self = this;

    var shockNameTextFieldTemplate = new sas.hc.m.Input({
        maxLength: 32,
        selectTextOnPress: false,
        disableValidation: false,
        value: "{scenarioName}",
        valueState: "{valueState}",
        valueStateText: "{valueStateText}",
        valueLiveUpdate: true,
        liveChange: $.proxy(self._handleShocksChanged, self)
    });

    return shockNameTextFieldTemplate;
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._handleInputChange = function() {
    this.fireInputChange();
};

sas.risk.common.msa.RiskModelSensitivityAnalysisBuilder.prototype._fireAnalysisChange = function() {
    var baselineNameExists = !!(this.getModel().getProperty("/baselineName"));

    var outputDataSetValid = true;
    if (this.getAllowOutputDataSetName() === true) {
        var outputDataSetName = this.getModel().getProperty("/outputDataSetName");
        var outputDataSetNameExists = !!outputDataSetName;
        var outputDataSetNameValid = !!(!this.outputDataSetNameInput || this.outputDataSetNameInput.getValueState() === sap.ui.core.ValueState.None);

        outputDataSetValid = outputDataSetNameExists && outputDataSetValid;
    }

    var shocks = this.getModel().getProperty("/shocks");
    var shocksExist = !!(shocks && shocks.length > 0);
    var shocksValid = !!(_.every(shocks, function (entry) {
        return entry.valueState === sap.ui.core.ValueState.None;
    }));

    this.fireAnalysisChange({
        isValid: baselineNameExists && outputDataSetValid && shocksExist && shocksValid
    });
};