jQuery.sap.require('sas.risk.common.utils.Router');
jQuery.sap.require('sas.risk.scenariomanager.ScenarioAPI');
jQuery.sap.require("sas.hc.m.MessageBox");
jQuery.sap.require('sas.risk.common.utils.SASStudioProxy');
jQuery.sap.require('sas.risk.scenariomanager.ViewManager');
jQuery.sap.require("sas.hc.m.MessageBox");
jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");

sap.ui.controller('sas.risk.scenariomanager.Shell', {
    onInit: function () {
        var ctrl = this;
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: 'sas.risk.scenariomanager.nls.messagebundle' });
        ctrl.rb = i18N.getResourceBundle();
        view.setModel(i18N, 'i18N');

        if (sas && sas.applicationSwitcher) {
            sas.applicationSwitcher.updateTitle('RiskScenarioManager', sas.config.getAppName());
            sas.applicationSwitcher.subscribe("OpenScenario", function (type,
                                                                        data) {
                sas.risk.scenariomanager.ViewManager.setAppSwitchData(data);
                if (sas.risk.scenariomanager.ViewManager.isInitializationComplete()) {
                    ctrl.openScenarioWithAppSwitchData();
                } else {
                    ctrl.appSwitchIntervalId = setInterval(function () {
                        if (sas.risk.scenariomanager.ViewManager.isInitializationComplete()) {
                            ctrl.openScenarioWithAppSwitchData();
                        }
                    }, 100);
                }
            });

            sas.applicationSwitcher.subscribe("CreateScenario", function (type,
                                                                          data) {
                sas.risk.scenariomanager.ViewManager.setAppSwitchData(data);
                if (sas.risk.scenariomanager.ViewManager.isInitializationComplete()) {
                    ctrl.createScenarioWithAppSwitchData();
                } else {
                    ctrl.appSwitchIntervalId = setInterval(function () {
                        if (sas.risk.scenariomanager.ViewManager.isInitializationComplete()) {
                            ctrl.createScenarioWithAppSwitchData();
                        }
                    }, 100);
                }
            });

            sas.applicationSwitcher.subscribe("AppSwitcher.AppShow", function (type,
                                                                               data) {
                // Verify that Scenario Manager is the currently displayed app
                if (data.appId === sas.config.getAppId()) {
                    // Only refresh the scenarios after the initial load.
                    if (sas.risk.scenariomanager.ViewManager.isInitializationComplete()) {
                        var mgr = sas.risk.scenariomanager.ViewManager.getScenarioManagerView();
                        if (mgr) {
                            var editor = sas.risk.scenariomanager.ViewManager.getScenarioEditorView();
                            // If the editor is open, then we do not want to refresh the scenarios in the manager view.
                            if (!editor) {
                                mgr.getController().refreshModel();
                            }
                        }
                    }
                }
            });
        }
        $.when(sas.risk.scenariomanager.ScenarioAPI.loadUserName()).done(function (user) {
             ctrl.loadRootLinks();
         }).fail(function (jqxhr) {
            ctrl.onServerFailure(jqxhr);
        });
    },

    loadRootLinks: function () {
        var ctrl = this;
        $.when(sas.risk.scenariomanager.ScenarioAPI.loadRootLinks()).done(function (rootLinks) {
            $.when(sas.risk.scenariomanager.ScenarioAPI.loadWorkGroups()).done(function (workGroups) {
                ctrl.initializeRouter();
                ctrl.initializeSASStudioProxy();
            }).fail(function (jqxhr) {
                ctrl.onServerFailure(jqxhr);
            });
        }).fail(function (jqxhr) {
            ctrl.onServerFailure(jqxhr);
        });
    },

    onServerFailure: function (event) {
        var ctrl = this;

        var title = ctrl.rb.getText('error.window.title');
        var msg = ctrl.rb.getText('error.message.txt');
        var response = event.responseJSON;
        if (response) {
            sas.risk.common.widgets.MessageDetailsBox.error(response, title);
        } else {
            sas.hc.m.MessageBox.error(msg, title);
        }
    },

    openScenarioWithAppSwitchData: function () {
        var ctrl = this;
        if (this.appSwitchIntervalId) {
            clearInterval(this.appSwitchIntervalId);
            this.appSwitchIntervalId = undefined;
        }
        var scenario = sas.risk.scenariomanager.ViewManager.getAppSwitchData();
        sas.risk.scenariomanager.ViewManager.setAppSwitchData(null);

        if (!scenario.id || scenario.id < 0) {
            scenario.id = 0;
            if (!scenario.scenarioName) {
                // We do not have enough information to proceed.
                // TODO POST-171: better error message. Something along the lines of "missing scenario name?
                // Not needed for 171 because it should be an edge case.
                sas.hc.m.MessageBox.error(ctrl.rb.getText('error.message.txt'), ctrl.rb.getText('error.window.title'));
                return;
            }
        }
        if (scenario.id == 0 && scenario.scenarioName) {
            // The work group, name, and version were passed in via appswitch. We need to lookup the scenario and find its id.
            var name = scenario.scenarioName;
            var version = scenario.scenarioVersion;
            var workGroupName = scenario.workGroup;
            var mgr = sas.risk.scenariomanager.ViewManager.getScenarioManagerView();
            var matchingScenario = mgr.getController().findScenario(workGroupName, name, version);
            // Was a matching scenario found?
            if (matchingScenario) {
                var clone = sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(matchingScenario);
                clone.readOnly = scenario.readOnly;
                clone.appSwitchReadOnly = scenario.appSwitchReadOnly;
                clone.appSwitchDataSourceName = scenario.appSwitchDataSourceName;
                clone.appSwitchDataSourceParms = scenario.appSwitchDataSourceParms;
                clone.appSwitchDataSourceId = scenario.appSwitchDataSourceId;
                clone.appSwitchDataSourceType = scenario.appSwitchDataSourceType;
                scenario = clone;
            } else {
                // The scenario could not be found. We do not want to continue attempting to open the scenario. Throw an
                // error and don't continue opening/changing the editor.
                // TODO post-171: need a better error message, something like "scenario {0} in work group {1} with version {2} was not found"
                // too late to add this message now and it should be an edge case.
                sas.hc.m.MessageBox.error(ctrl.rb.getText('error.message.txt'), ctrl.rb.getText('error.window.title'));
                return;
            }
        }

        var editorView = sas.risk.scenariomanager.ViewManager.getScenarioEditorView();

        // If the editor is open, then we need to handle that before trying to open the appswitch scenario.
        if (editorView) {
            var editorCtrl = editorView.getController();
            var isDirty = editorCtrl.isDirty();

            if (!isDirty) {
                editorCtrl.openScenario(scenario);
            } else {
                var title = ctrl.rb.getText('scenarioEditor.dirty.title.txt');
                var msg = ctrl.rb.getText('scenarioEditor.dirty.question.txt');
                var actions = [sas.hc.m.MessageBox.Action.OK, sas.hc.m.MessageBox.Action.CANCEL];
                sas.hc.m.MessageBox.warning(msg, title, actions, null, function (btn) {
                    if (btn === sas.hc.m.MessageBox.Action.OK) {
                        editorCtrl.openScenario(scenario);
                    }
                });
            }
        } else {
            sas.risk.scenariomanager.ViewManager.loadScenarioEditor([scenario]);
        }
    },
    
    createScenarioWithAppSwitchData: function () {
        if (this.appSwitchIntervalId) {
            clearInterval(this.appSwitchIntervalId);
            this.appSwitchIntervalId = undefined;
        }
        var data = sas.risk.scenariomanager.ViewManager.getAppSwitchData();
        sas.risk.scenariomanager.ViewManager.setAppSwitchData(null);
        var mgr = sas.risk.scenariomanager.ViewManager.getScenarioManagerView();
        if (mgr) {
            var editorView = sas.risk.scenariomanager.ViewManager.getScenarioEditorView();
            // Check if the editor is open.
            if (!editorView) {
                // No scenario is currently open, we can open the create dialog.
                mgr.getController().createScenarioWithAppSwitchData(data);
            } else if (!editorView.getController().isDirty()){
                // A scenario is open but is not in the dirty state, we can close the current and open the create dialog.
                editorView.getController().onBreadCrumbClick();
                mgr.getController().createScenarioWithAppSwitchData(data);
            } else {
                // A scenario is open and is dirty. We must save off the create information and wait until the user closes the scenario.
                var editorCtrl = editorView.getController();
                editorCtrl.openCreateDialogOnClose = true;
                editorCtrl.savedForLaterAppSwitchData = data;
                editorCtrl.onBreadCrumbClick();
            }
        }
    },

    initializeSASStudioProxy: function () {
        var smPath = sas.config.contextPath;
        var path = smPath.substring(0, smPath.lastIndexOf('/'));
        var urlSASStudio = path + "/SASLogon/Director?_directive=SASStudioMidTierLogon";
        sas.risk.common.utils.SASStudioProxy.setStudioURL(urlSASStudio);
    },

    initializeRouter: function () {
        var oRouter = new sas.risk.common.utils.Router(
            {
                ScenarioManager: {
                    pattern: 'Scenarios',
                    view: 'ScenarioManager',
                    viewPath: "sas.risk.scenariomanager.view",
                    viewId: 'ScenarioManagerView'
                },
                ScenarioEditor: {
                    pattern: 'Scenarios/{id}/Editor',
                    view: 'ScenarioEditor',
                    viewPath: "sas.risk.scenariomanager.view",
                    viewId: 'ScenarioEditorView'
                },
                ModelSensitivityAnalysis: {
                    pattern: 'ModelSensitivityAnalysis/{scenario}',
                    view: 'ModelSensitivityAnalysis',
                    viewPath: "sas.risk.scenariomanager.view",
                    viewId: 'ModelSensitivityAnalysis'
                },
                Default: {
                    pattern: '',
                    view: 'ScenarioManager',
                    viewPath: "sas.risk.scenariomanager.view",
                    viewId: 'ScenarioManagerView'
                }
            },
            {
                targetControl: 'ShellArea',
                targetAggregation: 'content',
                viewType: sap.ui.core.mvc.ViewType.JS,
                viewPath: 'sas.risk.scenariomanager',
                clearTarget: true
            }
        );

        //router initialize here
        oRouter.register('router');
        sap.ui.getCore().attachInit(function () {
            oRouter.initialize();
        });
    }
});