jQuery.sap.require("sas.risk.scenariomanager.EventBusConstants");

sas.risk.scenariomanager.ViewManager = {
    _router: sap.ui.core.routing.Router.getRouter('router'),
    _initializationComplete: false,
    _appSwitchData: null,

    getScenarioEditorView: function () {
        return sap.ui.getCore().byId("ScenarioEditorView");
    },

    getScenarioManagerView: function () {
        return sap.ui.getCore().byId("ScenarioManagerView");
    },

    getRouter: function () {
        if (undefined === sas.risk.scenariomanager.ViewManager._router) {
            sas.risk.scenariomanager.ViewManager._router = sap.ui.core.routing.Router.getRouter('router');
        }
        return sas.risk.scenariomanager.ViewManager._router;
    },

    isInitializationComplete: function () {
        return sas.risk.scenariomanager.ViewManager._initializationComplete;
    },

    setInitializationComplete: function (completed) {
        sas.risk.scenariomanager.ViewManager._initializationComplete = completed;
    },

    getAppSwitchData: function () {
        return sas.risk.scenariomanager.ViewManager._appSwitchData;
    },

    setAppSwitchData: function (appSwitchData) {
        if (appSwitchData) {
            if ((appSwitchData.scenarioId && appSwitchData.scenarioId < 0) || (appSwitchData.id && appSwitchData.id < 0)) {
                // MIP scenarios will come in with a negative ID. We need to ignore this and do the lookup by work group, name, and version.
                appSwitchData.id = 0;
                appSwitchData.scenarioId = undefined;
            }
            if (appSwitchData.scenarioId !== undefined) {
                appSwitchData.id = appSwitchData.scenarioId;
            }
            // HPRisk may pass this in as scenarioWorkGroup instead of just workGroup, in that case we will convert it here.
            if (appSwitchData.scenarioWorkGroup !== undefined && appSwitchData.workGroup === undefined) {
                appSwitchData.workGroup = appSwitchData.scenarioWorkGroup;
            }
            if (appSwitchData.dataSourceId !== undefined) {
                appSwitchData.appSwitchDataSourceId = appSwitchData.dataSourceId;
                appSwitchData.dataSourceId = undefined;
            }
            if (appSwitchData.dataSourceName !== undefined) {
                appSwitchData.appSwitchDataSourceName = appSwitchData.dataSourceName;
                appSwitchData.dataSourceName = undefined;
            }
            if (appSwitchData.dataSourceType !== undefined) {
                appSwitchData.appSwitchDataSourceType = appSwitchData.dataSourceType;
                appSwitchData.dataSourceType = undefined;
            }
            if (appSwitchData.dataSourceParms !== undefined) {
                appSwitchData.appSwitchDataSourceParms = appSwitchData.dataSourceParms;
                appSwitchData.dataSourceParms = undefined;
            }
            if (appSwitchData.readOnly !== undefined) {
                appSwitchData.appSwitchReadOnly = (true === appSwitchData.readOnly || "true" === String(appSwitchData.readOnly).toLowerCase());
                appSwitchData.readOnly = undefined;
            }
        }
        sas.risk.scenariomanager.ViewManager._appSwitchData = appSwitchData;
    },

    loadScenarioManager: function () {
        sas.risk.scenariomanager.ViewManager.getRouter().navTo('ScenarioManager', {}, true);
    },

    loadScenarioEditor: function (scenarios) {
        var scenario = { id: 0 };
        if (scenarios && scenarios[0]) {
            scenario = scenarios[0];
            if (!scenario.id || scenario.id < 0) {
                scenario.id = 0;
            }
        }

        if (scenario.id > 0 || !scenario.hasOwnProperty('appSwitchReadOnly')) {
            // S1325805: 3rd param is true because we want to REPLACE the browser history entry (do not allow back/forward buttons to affect our app)
            sas.risk.scenariomanager.ViewManager.getRouter().navTo('ScenarioEditor', {id: scenario.id}, true);

            /*
             * This is wrong - we need to handle this in the arguments handed to the view/controller at open!
             */
            var editor = sas.risk.scenariomanager.ViewManager.getScenarioEditorView();
            editor.getController().openScenario(scenario);
        } else {
            var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: 'sas.risk.scenariomanager.nls.messagebundle' }),
                rb = i18N.getResourceBundle(),
                title = rb.getText('error.window.title'),
                msg = rb.getText('scenarioManager.editorOpenFailedFromAppSwitch.error.txt');

            sas.hc.m.MessageBox.error(msg, title);
        }
    },

    loadModelSensitivityAnalysis: function (scenario) {
        sas.risk.scenariomanager.ViewManager.getRouter().navTo('ModelSensitivityAnalysis', { scenario: scenario.id }, true);

        var modelSensitivityAnalysisView = sap.ui.getCore().byId("ModelSensitivityAnalysis");
        modelSensitivityAnalysisView.getController().openModelSensitivityAnalysis(scenario);
    },

    loadScenarioFinalValues: function (scenario, context, scenarioTableHelper) {
       var view = sap.ui.getCore().byId("ScenarioFinalValues");
       if (undefined === view) {
            view = sap.ui.view("ScenarioFinalValues", {
                height: '100%',
                width: '100%',
                viewName: "sas.risk.scenariomanager.view.ScenarioFinalValues",
                type: sap.ui.core.mvc.ViewType.JS
            });
        }

        view.getController().openScenarioFinalValues(scenario, context, scenarioTableHelper);
        return view;
    }
};