jQuery.sap.declare("sas.risk.scenariomanager.model.PerturbationType");
jQuery.sap.require("sas.risk.scenariomanager.model.Scenario");

sap.ui.base.Object.extend("sas.risk.scenariomanager.model.PerturbationType", {
    constructor: function (ord, code, valueFn, changeFn) {
        var _rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.scenariomanager.nls");
        this.ord = ord;
        this.code = code;
        this.label = _rb.getText('perturbType.' + code.toLowerCase() + '.label.txt');
        this.valueFn = valueFn;
        this.changeFn = changeFn;
    }
});

sas.risk.scenariomanager.model.PerturbationType.NONE = new sas.risk.scenariomanager.model.PerturbationType(0, 'NONE',
    function (base, chg) {
        return base;
    },
    function (base, val) {
        return base;
    });

sas.risk.scenariomanager.model.PerturbationType.VALUE = new sas.risk.scenariomanager.model.PerturbationType(1, 'VALUE',
    function (base, chg) {
        return chg;
    },
    function (base, val) {
        return val;
    });

sas.risk.scenariomanager.model.PerturbationType.ABS = new sas.risk.scenariomanager.model.PerturbationType(2, 'ABS',
    function (base, chg) {
        return base + chg;
    },
    function (base, val) {
        return val - base;
    });

sas.risk.scenariomanager.model.PerturbationType.REL = new sas.risk.scenariomanager.model.PerturbationType(3, 'REL',
    function (base, chg) {
        return base * (1.0 + chg);
    },
    function (base, val) {
        return (base) ? val / base - 1.0 : 0.0;
    });

sas.risk.scenariomanager.model.PerturbationType.STD = new sas.risk.scenariomanager.model.PerturbationType(4, 'STD',
    function (rf, vb, base, chg) {
        var RiskVarConstants = sas.risk.scenariomanager.model.RiskVarConstants,
            MACLOGBIG = 7.0978271289338392e+02,
            MACLOGSMALL = -708.3964185;

        if (rf && vb) {
            if (rf.measurementLevel === RiskVarConstants.MLEVEL.INTERVAL) {
                return base + (chg * vb.baseValue);
            } else if (rf.measurementLevel === RiskVarConstants.MLEVEL.RATIO) {
                var delta = chg * vb.baseValue;
                if ((delta > MACLOGSMALL) && (delta < MACLOGBIG)) {
                    return base * Math.exp(delta);
                }
            }
        }

        return NaN;
    },
    function (rf, vb, base, val) {
        var RiskVarConstants = sas.risk.scenariomanager.model.RiskVarConstants;
        if (rf && vb) {
            if (rf.measurementLevel === RiskVarConstants.MLEVEL.INTERVAL) {
                return (val - base) / vb.baseValue;
            } else if (rf.measurementLevel === RiskVarConstants.MLEVEL.RATIO) {
                if (vb.baseValue && base) {
                    return Math.log(val / base) / vb.baseValue;
                }
            }
        }

        return Number.NaN;
    });

sas.risk.scenariomanager.model.PerturbationType.ABS_TIME = new sas.risk.scenariomanager.model.PerturbationType(5, 'ABS_TIME',
    function (base, chg) {
        return base + chg;
    },
    function (base, val) {
        return val - base;
    });

sas.risk.scenariomanager.model.PerturbationType.REL_TIME = new sas.risk.scenariomanager.model.PerturbationType(6, 'REL_TIME',
    function (base, chg) {
        return base * (1.0 + chg);
    },
    function (base, val) {
        return (base) ? val / base - 1.0 : 0.0;
    });

sas.risk.scenariomanager.model.PerturbationType.values = function () {
    return [
        sas.risk.scenariomanager.model.PerturbationType.NONE,
        sas.risk.scenariomanager.model.PerturbationType.VALUE,
        sas.risk.scenariomanager.model.PerturbationType.ABS,
        sas.risk.scenariomanager.model.PerturbationType.REL,
        sas.risk.scenariomanager.model.PerturbationType.STD,
        sas.risk.scenariomanager.model.PerturbationType.ABS_TIME,
        sas.risk.scenariomanager.model.PerturbationType.REL_TIME
    ];
};

sas.risk.scenariomanager.model.PerturbationType.prototype.getOrdinal = function () {
    return this.ord;
};

sas.risk.scenariomanager.model.PerturbationType.prototype.getCode = function () {
    return this.code;
};

sas.risk.scenariomanager.model.PerturbationType.prototype.getLabel = function () {
    return this.label;
};

// change is optional; we'll read if from the stress if not provided (it allows the caller to pre-compute the value if
// the corresponding perturbation were to be changed to the given argument)

// At least one of stress & variableName must be non-null; we'll derive the null one if necessary

sas.risk.scenariomanager.model.PerturbationType.prototype.computeValue = function (context,
                                                                                   scenario,
                                                                                   stress,
                                                                                   variableName,
                                                                                   hzIdx,
                                                                                   change) {
    if (variableName && !stress) {
        stress = sas.risk.scenariomanager.model.Scenario.getStress(scenario, variableName, context);
    }

    if (stress && !variableName) {
        variableName = stress.variable;
    }

    var baseValue = this._getBaseValue(context, scenario, stress, variableName, hzIdx);
    if (baseValue === undefined || baseValue === null) {
        baseValue = Number.NaN;
    }

    if (hzIdx < 0) {
        return baseValue;
    }

    if (change === null || change === undefined || isNaN(change)) {
        change = stress.perturbations[hzIdx].value;
    }

    var value = 0;
    if (this === sas.risk.scenariomanager.model.PerturbationType.STD) {
        var rf = context.getObject(variableName);
        var vol = context.getObject(rf.volatilityByName);
        value = this.valueFn(rf, vol, +baseValue, +change);
    }
    else {
        value = this.valueFn(+baseValue, +change);
    }

    var cacheKey = (stress.variable === variableName) ? 'values' : 'values.for.' + variableName;
    if (!stress.hasOwnProperty(cacheKey)) {
        stress[cacheKey] = [];
    }

    stress[cacheKey][hzIdx] = value;
    return value;
};

//change is optional; we'll read if from the stress if not provided (it allows the caller to pre-compute the value if
//the corresponding perturbation were to be changed to the given argument)

//At least one of stress & variableName must be non-null; we'll derive the null one if necessary

sas.risk.scenariomanager.model.PerturbationType.prototype.computeChange = function (context,
                                                                                    scenario,
                                                                                    stress,
                                                                                    variableName,
                                                                                    hzIdx,
                                                                                    value) {
    if (variableName && !stress) {
        stress = sas.risk.scenariomanager.model.Scenario.getStress(scenario, variableName, context);
    }

    if (stress && !variableName) {
        variableName = stress.variable;
    }

    var baseValue = this._getBaseValue(context, scenario, stress, variableName, hzIdx);
    if (baseValue === undefined || baseValue === null) {
        baseValue = Number.NaN;
    }

    if (this === sas.risk.scenariomanager.model.PerturbationType.STD) {
        var rf = context.getObject(variableName);
        var vol = context.getObject(rf.volatilityByName);
        return this.changeFn(rf, vol, +baseValue, +value);
    }

    return this.changeFn(+baseValue, +value);
};

sas.risk.scenariomanager.model.PerturbationType.prototype._getBaseValue = function (context,
                                                                                    scenario,
                                                                                    stress,
                                                                                    variableName,
                                                                                    hzIdx) {
    function _isHorizonBased(type) {
        return type === sas.risk.scenariomanager.model.PerturbationType.ABS_TIME ||
               type === sas.risk.scenariomanager.model.PerturbationType.REL_TIME;
    }

    function _getBaseValueFromContext(variableName) {
        var rf = context.getObject(variableName);
        return (rf) ? rf.baseValue : Number.NaN;
    }

    function _getPerturbValue(scen, str, idx) {
        var pertType = sas.risk.scenariomanager.model.PerturbationType.byCode(str.perturbType);
        var cacheKey = (str.variable === variableName) ? 'values' : 'values.for.' + variableName;
        if (!str.hasOwnProperty(cacheKey)) {
            str[cacheKey] = _.range(str.perturbations.length);
            _.map(str[cacheKey], function (n, i, values) {
                values[i] = pertType.computeValue(context, scen, str, variableName, i);
            });
        }

        return str[cacheKey][idx];
    }

    if (_isHorizonBased(this)) {
        return (hzIdx > 0) ? _getPerturbValue(scenario, stress, hzIdx - 1) : _getBaseValueFromContext(variableName);
    }

    if (scenario.baseline == null) {
        return _getBaseValueFromContext(variableName);
    }

    if (hzIdx >= 0) {
        var baseStress = sas.risk.scenariomanager.model.Scenario.getStress(scenario.baseline, variableName, context);
        if (baseStress) {
            return _getPerturbValue(scenario.baseline, baseStress, hzIdx);
        }
    }

    return _getBaseValueFromContext(variableName);
};

sas.risk.scenariomanager.model.PerturbationType.byCode = function (code) {
    return (!code) ? null :
           _.findWhere(sas.risk.scenariomanager.model.PerturbationType.values(), { code: code.toUpperCase() });
};

sas.risk.scenariomanager.model.PerturbationType.getDisplayText = function (code) {
    var type = sas.risk.scenariomanager.model.PerturbationType.byCode(code);
    return (type) ? type.label : '';
};

sas.risk.scenariomanager.model.PerturbationType.getTypes = function (rf, hzCount) {
    var constants = sas.risk.scenariomanager.model.RiskVarTypeConstants,
        perturbationTypes = [];

    if (rf && rf.objectType && rf.objectType === constants.RISKFACTOR_OBJTYPE) {
        perturbationTypes.push('VALUE');
    }

    perturbationTypes.push('ABS');
    perturbationTypes.push('REL');

    if (rf && rf.objectType && rf.objectType === constants.RISKFACTOR_OBJTYPE && rf.volatility) {
        perturbationTypes.push('STD');
    }

    if (hzCount && hzCount > 1) {
        perturbationTypes.push('ABS_TIME');
        perturbationTypes.push('REL_TIME');
    }

    return perturbationTypes;
};