jQuery.sap.declare('sas.risk.scenariomanager.utils.ScenarioPropertyController');
jQuery.sap.require("sas.risk.scenariomanager.utils.ScenarioEditorUtil");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.risk.scenariomanager.model.IntervalTypeConstants");
jQuery.sap.require("sas.risk.scenariomanager.ScenarioAPI");

sap.ui.base.Object.extend('sas.risk.scenariomanager.utils.ScenarioPropertyController', {
    constructor: function (fragment, title) {
        var i18N = new sap.ui.model.resource.ResourceModel({bundleName: 'sas.risk.scenariomanager.nls.messagebundle'});
        this.rb = i18N.getResourceBundle();

        var dialog = sap.ui.jsfragment(fragment, this);
        sap.ui.getCore().getMessageManager().registerObject(dialog, true);
        dialog.setModel(i18N, 'i18N');

        if (title) {
            dialog.setTitle(title);
        }

        this.view = dialog;
    }
});

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setOKHandler = function (handler) {
    this.okHandler = handler;
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.cloneScenario = function (host, scenario) {
    var ctrl = this;
    this.isClone = true;
    for (var index = 0; index < scenario.stresses.length; ++index) {
        scenario.stresses[index].perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.unmapPerturbations(scenario.stresses[index].perturbations);
        scenario.stresses[index] = _.omit(scenario.stresses[index], ['parent', 'children']);
    }
    var clone = sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(scenario);
    clone.name = clone.filename = "";
    // Check if the cache source was unset
    if (!clone.sourceId || clone.sourceId === "0") {
        // Only set the cache source to the cloned scenario if no cache source was set. Otherwise we need to maintain the original cache source setting.
        clone.sourceId = "" + clone.id;
    }
    clone.id = "0";

    var appSwitchData;

    ctrl.openView(host, appSwitchData, function (ctxt, dataSources) {
        var copy = "";
        var vers = (clone.scenarioVersion) ? clone.scenarioVersion.trim() : "";
        if (vers) {
            var regex = new RegExp("^(.*)" + ctrl.rb.getText("scenarioPropertyDialog.copy.txt") + "\\s*(\\d*)?$");
            var match = regex.exec(vers);
            if (match) {
                vers = match[1].trim();
                copy = parseInt(match[2]) + 1 || 1;
            }
        }

        var suggested;
        do {
            suggested = (vers) ? ctrl.rb.getText("scenarioPropertyDialog.copied.version.txt", [vers, copy])
                : ctrl.rb.getText("scenarioPropertyDialog.sample.version.txt", [copy]);
            copy = (copy) ? ++copy : 1;
        } while (ctrl.notUnique(ctxt, clone.workGroup, clone.name, clone.label, suggested));

        clone.scenarioVersion = suggested;
        ctrl.setModel(clone);
    });
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.createScenario = function (host, appSwitchData) {
    var ctrl = this;
    var scenario = {
        id: 0,
        baselineId: 0,
        stresses: [],
        links: [],

        periodType: sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE,
        periods: [sas.risk.scenariomanager.utils.ScenarioEditorUtil.getNewPeriod()]
    };

    ctrl.openView(host, appSwitchData, function (ctxt, dataSources, availableWorkGroups) {
        var suggested;
        var copy = 1;
        var defaultWorkGroup = sas.risk.scenariomanager.ScenarioAPI.getRecommendedPrimaryWorkGroup();
        if (!defaultWorkGroup) {
            defaultWorkGroup = availableWorkGroups[0];
        }
        do {
            suggested = ctrl.rb.getText("scenarioPropertyDialog.sample.name.txt", [copy++]);
        } while (ctrl.notUnique(ctxt, defaultWorkGroup.name, ctrl.mungeName(suggested), suggested));

        scenario.workGroup = defaultWorkGroup.name;
        scenario.label = suggested;
        scenario.name = ctrl.mungeName(suggested);
        if (appSwitchData) {
            if (appSwitchData.workGroup) {
                var appSwitchWorkGroup = _.find(availableWorkGroups, function (workGroup) {
                    return workGroup.name.toLocaleUpperCase() === appSwitchData.workGroup.toLocaleUpperCase();
                });
                if (appSwitchWorkGroup) {
                    scenario.workGroup = appSwitchWorkGroup.name;
                }
            }
            if (appSwitchData.scenarioName) {
                scenario.label = appSwitchData.scenarioName;
                scenario.name = ctrl.mungeName(appSwitchData.scenarioName);
            }
            if (appSwitchData.scenarioVersion) {
                scenario.scenarioVersion = appSwitchData.scenarioVersion;
            }
            if (appSwitchData.appSwitchDataSourceName || appSwitchData.appSwitchDataSourceId) {
                scenario.dataSourceId = appSwitchData.appSwitchDataSourceId;
                if (appSwitchData.appSwitchDataSourceId) {
                    scenario.dataSourceName = appSwitchData.appSwitchDataSourceId; // Use id as the name, that way it doesn't have the .rskcdesc extension, which can cause problems when trying to run in HPRisk
                } else {
                    scenario.dataSourceName = appSwitchData.appSwitchDataSourceName;
                }

                // TODO: need to populate dataSourceType, create the link to the stress context
                if (appSwitchData.appSwitchDataSourceType == 'stress') {
                    scenario.dataSourceType = 'S';
                } else if (appSwitchData.appSwitchDataSourceType == 'price') {
                    scenario.dataSourceType = 'P';
                }

                var cubeWorkGroup, serviceName, relPath;
                if (appSwitchData.appSwitchDataSourceParms) {
                    scenario.dataSourceArgs = sas.risk.scenariomanager.ScenarioAPI.encodeArgsAsQuery(appSwitchData.appSwitchDataSourceParms);
                    if (appSwitchData.appSwitchDataSourceParms.workGroup) {
                        var workGroupsForCubes = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getWorkGroupsForCubes(scenario.workGroup);
                        var appSwitchDataSourceWorkGroup = _.find(workGroupsForCubes, function (workGroup) {
                            return workGroup.toLocaleUpperCase() === appSwitchData.appSwitchDataSourceParms.workGroup.toLocaleUpperCase();
                        });
                        if (appSwitchDataSourceWorkGroup) {
                            cubeWorkGroup = appSwitchDataSourceWorkGroup;
                        }
                    }
                    serviceName = appSwitchData.appSwitchDataSourceParms.application;
                    relPath = appSwitchData.appSwitchDataSourceParms.relPath;
                }

                var cubeNotFound = function (scen) {
                    jQuery.sap.log.error('AppSwitch cube not found or does not have a stressContext, clearing data source fields.');
                    // No stress context available for the cube, we can't use it.
                    scen.dataSourceId = undefined;
                    scen.dataSourceName = undefined;
                    scen.dataSourceArgs = undefined;
                    scen.dataSourceType = undefined;
                };

                var ds;
                if (serviceName) {
                    // find the stressContext link from the selected dataSource for the scenario
                    ds = sas.risk.scenariomanager.utils.ScenarioEditorUtil.findCubeInFolders(dataSources, cubeWorkGroup, serviceName, relPath, appSwitchData.appSwitchDataSourceId, appSwitchData.appSwitchDataSourceName);
                }
                if (ds) {
                    sas.risk.scenariomanager.ScenarioAPI.getCubeSelf(ds).done(function (cubeSelf) {
                        var dataSourceGetterLink = _.findWhere(cubeSelf.links, {rel: "stressContext"});
                        if (dataSourceGetterLink) {
                            ctrl.setSelectedDataSource(cubeSelf);
                            scenario.links.push(dataSourceGetterLink);
                            scenario.dataSourceId = ds.id;
                            scenario.dataSourceName = ds.name;
                        } else {
                            cubeNotFound(scenario);
                        }
                    }).fail(function (jqxhr) {
                        // App-switch data source info was no good - clear out the data source values
                        cubeNotFound(scenario);
                    });
                } else if (appSwitchData.appSwitchDataSourceParms) {
                    sas.risk.scenariomanager.ScenarioAPI.getCube(appSwitchData.appSwitchDataSourceId, appSwitchData.appSwitchDataSourceParms).done(function (cube) {
                        if (cube) {
                            sas.risk.scenariomanager.ScenarioAPI.getCubeSelf(cube).done(function (cubeSelf) {
                                var dataSourceGetterLink = _.findWhere(cubeSelf.links, {rel: "stressContext"});
                                if (dataSourceGetterLink) {
                                    ctrl.setSelectedDataSource(cubeSelf);
                                    scenario.links.push(dataSourceGetterLink);
                                    scenario.dataSourceId = cube.id;
                                    scenario.dataSourceName = cube.name;
                                } else {
                                    cubeNotFound(scenario);
                                }
                            }).fail(function (jqxhr) {
                                cubeNotFound(scenario);
                            });
                        } else {
                            // App-switch data source info was no good - clear out the data source values
                            cubeNotFound(scenario);
                        }
                    }).fail(function (jqxhr) {
                        // App-switch data source info was no good - clear out the data source values
                        cubeNotFound(scenario);
                    });
                } else {
                    cubeNotFound(scenario);
                }
            }
        }

        ctrl.setModel(scenario);
    });
};


sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.getView = function () {
    return this.view;
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.openView = function (host, appSwitchData, ctxtHandler) {
    var ctrl = this;
    ctrl.host = host;
    var availableWorkGroups = sas.risk.scenariomanager.ScenarioAPI.getWorkGroups();

    var dataLoadedSuccessfully = function (scenarioData, dataSources) {
        ctxtHandler(scenarioData, dataSources, availableWorkGroups);
        ctrl.setContext(scenarioData);
        ctrl.setDataSources(dataSources);
        ctrl.setWorkGroups(availableWorkGroups);

        // select default data source
        var ds = ctrl.view.getModel().getProperty('/dataSourceName'),
            selected = _.find(dataSources, {name: ds});

        if (ds && !selected) {
            selected = {
                id: ctrl.model.dataSourceId,
                name: ctrl.model.dataSourceName,
                label: ctrl.model.dataSourceName.replace('.rskdesc', '')
            };

            if (ctrl.model.dataSourceType == 'S') {
                selected.type = 'stress';
            } else if (ctrl.model.dataSourceType == 'P') {
                selected.type = 'price';
            }
        }
        ctrl.setSelectedDataSource(selected);

        host.closeBusyDialog("scenarioManager.loadScenarioAndCubeLists.txt");
        ctrl.view.open();
        // These values may have been specified via appswitch, validate the UI to be safe.
        ctrl.validateLabelVersion(ctrl.model.workGroup, ctrl.model.name, ctrl.model.scenarioVersion);
    };

    var loadingFailure = function (event) {
        host.closeBusyDialog("scenarioManager.loadScenarioAndCubeLists.txt");
        host.onServerFailure(event);
    };

    host.openBusyDialog("scenarioManager.loadScenarioAndCubeLists.txt");
    $.when(sas.risk.scenariomanager.ScenarioAPI.getScenarioHorizons()).done(function (scenarioData) {
        // Can the user view cubes? (Does the user have the cubes link?)
        if (sas.risk.scenariomanager.ScenarioAPI.getCubesVisible()) {
            var appSwitchDataSourceWorkGroup;
            // If app-switch data is available, grab it.
            if (appSwitchData && appSwitchData.appSwitchDataSourceParms) {
                appSwitchDataSourceWorkGroup = appSwitchData.appSwitchDataSourceParms.workGroup;
            }
            var workGroupsForCubes = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getWorkGroupsForCubes(appSwitchDataSourceWorkGroup);

            $.when(sas.risk.scenariomanager.ScenarioAPI.getCubes(workGroupsForCubes)).done(function (dataSources) {
                dataLoadedSuccessfully(scenarioData, dataSources);
            }).fail(function (event) {
                // The cube link existed, but the call failed. Gracefully handle the failure by displaying an error message
                // and opening the dialog behind it. By passing in an empty array for data sources, the option to change
                // data sources will be disabled in the UI.
                dataLoadedSuccessfully(scenarioData, []);
                loadingFailure(event);
            });
        } else {
            // The user doesn't have the ability to get the cubes, so we will simply use an empty array for data sources. This
            // will allow the dialog to open without error and the change data source option will be disabled in the UI.
            dataLoadedSuccessfully(scenarioData, []);
        }
    }).fail(loadingFailure);
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setView = function (view) {
    this.view = view;
    if (this.model) {
        this.updateViewState();
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.getModel = function () {
    return this.model;
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setModel = function (model) {
    this.model = model;
    if (this.view) {
        this.view.setModel(new sas.risk.common.models.RiskJSONModel(model));
        this.updateViewState(true);
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setContext = function (scenarioData) {
    this.context = {scenarios: scenarioData};
    if (this.view) {
        this.view.setModel(new sas.risk.common.models.RiskJSONModel(this.context), 'ctxt');
        this.updateViewState(true);
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setWorkGroups = function (availableWorkGroups) {
    this.availableWorkGroups = availableWorkGroups;
    if (this.view) {
        this.view.setModel(new sas.risk.common.models.RiskJSONModel(this.availableWorkGroups), 'availableWorkGroups');
        this.updateViewState(true);
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setDataSources = function (dataSources) {
    this.dataSources = dataSources;
    if (this.view) {
        this.view.setModel(new sas.risk.common.models.RiskJSONModel(this.dataSources), 'dataSources');
        this.updateViewState(true);
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setSelectedDataSource = function (selected) {
    this.selectedDataSource = selected;
    if (this.view) {
        this.view.setModel(new sas.risk.common.models.RiskJSONModel(this.selectedDataSource), 'selectedDataSource');
        this.updateViewState(true);
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.setSelectedBaselineScenario = function (selected) {
    this.selectedBaselineScenario = selected;
    if (this.view) {
        this.view.setModel(new sas.risk.common.models.RiskJSONModel(this.selectedBaselineScenario), 'selectedBaselineScenario');
        this.updateViewState(true);
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onOKButtonClick = function () {
    var view = this.getView();
    view.close();
    this.okHandler();
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onNameChanged = function (event) {
    var ctrl = this;
    var view = this.getView();
    var control = event.getSource();
    var label = control.getValue().trim();
    var versionWidget = sap.ui.getCore().byId(view.getId() + '.versionField');

    // S1195414 Workaround
    // We have to disable the built in validations in order for our uniqueness validation to be retained,
    // so we're also including our own required field logic
    if (!label) {
        control.setValueState(sap.ui.core.ValueState.Error);
        control.setValueStateText(ctrl.rb.getText("scenarioPropertyDialog.required.name.error.txt"));
        ctrl.updateViewState(false);
        versionWidget.setValueState(sap.ui.core.ValueState.None);
        return;
    } else if (ctrl.isClone && ctrl.model.scenarioVersion) {
        ctrl.model.scenarioVersion = '';
    }

    var version = ctrl.model.scenarioVersion;
    if (!version) {
        version = '';
    }

    ctrl.validateLabelVersion(ctrl.model.workGroup, label, version.trim());
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onWorkGroupChanged = function (event) {
    var ctrl = this;
    var control = event.getSource();
    var selectedWorkGroup = control.getSelectedItem().data("workGroup");

    if (!selectedWorkGroup) {
        control.setValueState(sap.ui.core.ValueState.Error);
        control.setValueStateText(ctrl.rb.getText("scenarioPropertyDialog.required.workGroup.error.txt"));
        ctrl.updateViewState(false);
        return;
    }

    var label = ctrl.model.label;
    var version = ctrl.model.scenarioVersion;
    if (!version) {
        version = '';
    }

    var workGroupsForCubes = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getWorkGroupsForCubes(selectedWorkGroup.name);

    // Because the work group has changed, the data source selected may no longer be valid. If a data source is set and
    // the data source's work group is NOT PUBLIC, then we need to reset it and reset the repository checkbox.
    if (ctrl.model.dataSourceName && ctrl.model.dataSourceWorkGroup && ctrl.model.dataSourceWorkGroup.toLocaleUpperCase() !== "PUBLIC") {
        ctrl.onDataSourceSelected(undefined);
    }

    ctrl.host.openBusyDialog("scenarioManager.loadScenarioAndCubeLists.txt");
    $.when(sas.risk.scenariomanager.ScenarioAPI.getCubes(workGroupsForCubes)).done(function (dataSources) {
        ctrl.host.closeBusyDialog("scenarioManager.loadScenarioAndCubeLists.txt");
        ctrl.setDataSources(dataSources);
    }).fail(function (event) {
        ctrl.host.closeBusyDialog("scenarioManager.loadScenarioAndCubeLists.txt");
        ctrl.host.onServerFailure(event);
        ctrl.setDataSources([]);
    });

    // clear the baseline
    ctrl.setSelectedBaselineScenario(undefined);

    ctrl.validateLabelVersion(selectedWorkGroup.name, label, version.trim());
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onVersionChanged = function (event) {
    var ctrl = this;
    var view = this.getView();

    // Not much point in validating version if the required label is missing
    var label = ctrl.model.label;
    if (!label) {
        var versionWidget = sap.ui.getCore().byId(view.getId() + '.versionField');
        versionWidget.setValueState(sap.ui.core.ValueState.None);
        return;
    }

    var control = event.getSource();
    var version = control.getValue().trim();
    ctrl.validateLabelVersion(ctrl.model.workGroup, label.trim(), version);
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.validateLabelVersion = function (workGroup, label, version) {
    var ctrl = this;
    var view = this.getView();

    var labelWidget = sap.ui.getCore().byId(view.getId() + '.nameField');
    var versionWidget = sap.ui.getCore().byId(view.getId() + '.versionField');

    var name = ctrl.mungeName(label);
    var existing = ctrl.notUnique(ctrl.context.scenarios, workGroup, name, label, version);

    if (existing) {
        labelWidget.setValueState(sap.ui.core.ValueState.Error);
        var msg;
        // var publicWorkGroup = sas.risk.scenariomanager.ScenarioAPI.getPublicWorkGroup();
        // if (publicWorkGroup && workGroup === publicWorkGroup.name) {
        //     workGroup = ctrl.rb.getText("workGroup.public.txt");
        // }
        if (version) {
            msg = ctrl.rb.getText("scenarioPropertyDialog.unique.name.version.error.txt", [label, version, workGroup]);
            labelWidget.setValueStateText(msg);
            versionWidget.setValueState(sap.ui.core.ValueState.Error);
            versionWidget.setValueStateText(msg);
        } else {
            msg = ctrl.rb.getText("scenarioPropertyDialog.unique.name.error.txt", [label, workGroup]);
            labelWidget.setValueStateText(msg);
            versionWidget.setValueState(sap.ui.core.ValueState.None);
        }
        ctrl.updateViewState(false);
    } else {
        // TODO: we need to know if the value state was error, and if it was, was it this validation that failed?
        // Otherwise, we need to leave the value state alone

        labelWidget.setValueState(sap.ui.core.ValueState.None);
        versionWidget.setValueState(sap.ui.core.ValueState.None);
        ctrl.updateViewState(true);
        ctrl.model.name = name;
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onDataSourceSelected = function (dataSource) {
    // SOPHIE TODO: This function should be combined with ScenarioEditor's onChangeDataSourceOkay function, they're basically clones.
    var ctrl = this;
    var scenario = ctrl.getModel();

    var getDataSourceArgs = function (dataSource) {
        var selfLink = sas.risk.scenariomanager.ScenarioAPI.findOptionalLink(dataSource.links, {rel: 'self'});
        if (selfLink) {
            var tokens = selfLink.href.split("?");
            return (tokens.length < 2) ? "" : tokens[1];
        }

        return '';
    };

    if (dataSource) {
        scenario.dataSourceId = dataSource.id;
        scenario.dataSourceName = dataSource.id; // Use id as the name, that way it doesn't have the .rskcdesc extension, which can cause problems when trying to run in HPRisk
        scenario.dataSourceArgs = getDataSourceArgs(dataSource);
        scenario.dataSourceWorkGroup = dataSource.workGroup;

        // use the stressContext link form the dataSource for the scenario
        var dslink = _.findWhere(dataSource.links, {rel: 'stressContext'});
        if (dslink) {
            scenario.links = [dslink];
        }

    } else {
        scenario.dataSourceId = undefined;
        scenario.dataSourceName = undefined;
        scenario.dataSourceArgs = undefined;

        scenario.links = [];
    }

    ctrl.setSelectedDataSource(dataSource);

    ctrl.updateViewState();
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.createId = function (suffix) {
    if (this.host) {
        return this.host.createId(suffix);
    } else {
        return "ScenarioPropertyController_" + suffix;
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.byId = function (suffix) {
    var id = this.createId(suffix);
    return sap.ui.getCore().byId(id);
};
/*
 * Set the scenario to use the given datasource.
 * The data passed is that of the entry as it appears in the context listing.
 *
 * {  "id", "name", "lib", "label", links: { ... }
 */
sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onChangeDataSourceOkay = function (selected) {
    var deferred = $.Deferred();
    var ctrl = this;

    sas.risk.scenariomanager.ScenarioAPI.getCubeSelf(selected).done(function (context) {
        var currentContext = sas.risk.scenariomanager.ScenarioAPI.findOptionalLink(context.links, {rel: 'stressContext'});
        if (currentContext !== undefined) {
            ctrl.onDataSourceSelected(context);
            deferred.resolve();
        } else {
            sas.hc.m.MessageBox.error(ctrl.rb.getText("changeDataSource.noContext.fmt.txt", context.name), ctrl.rb.getText("changeDataSource.error.txt"));
            deferred.reject();
        }

    }).fail(function (event) {
        deferred.reject(event);
    });

    return deferred;
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onBrowseForDataSource = function () {
    // need the list sorted by first type (stress/price) and then by name.
    // can't get the table dialog to group.
    var ctrl = this;
    var view = this.getView();

    var cubelist = this.dataSources;

    //  the list is suppose to be sorted by  type then by name.
    var sortedData = cubelist;//.slice();

    var oTableSelectDialog = ctrl.byId('sas.risk.scenariomanager.view.DataSourcesSelectDialog');
    if (!oTableSelectDialog) {
        oTableSelectDialog = sap.ui.jsfragment('sas.risk.scenariomanager.view.DataSourcesSelectDialog', ctrl);
    }

    var dialogModel = oTableSelectDialog.getModel();
    if (!dialogModel) {
        dialogModel = new sas.risk.common.models.RiskJSONModel();
        oTableSelectDialog.setModel(dialogModel);
    }

    dialogModel.setProperty("/dataSources", {folders: sortedData});
    dialogModel.setProperty("/selectedDataSource", ctrl.selectedDataSource);

    oTableSelectDialog.attachAfterClose(function () {
        var selectedDataSource = sap.ui.getCore().byId("createDataSourceField").getInputProperty("value");
        if (!selectedDataSource || selectedDataSource == undefined) {
            sap.ui.getCore().byId("createRepoCheckbox").setSelected(true);
            sap.ui.getCore().byId("createRepoCheckbox").fireSelect();
        }
    });

    oTableSelectDialog.setModel(view.getModel('i18N'), 'i18N');
    oTableSelectDialog.open();
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onBrowseForScenario = function () {
    // need the list sorted by first type (stress/price) and then by name.
    // can't get the table dialog to group.
    var ctrl = this;
    var view = this.getView();

    var oTableSelectDialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ScenarioSelectDialog');
    if (!oTableSelectDialog) {
        oTableSelectDialog = sap.ui.jsfragment('sas.risk.scenariomanager.view.ScenarioSelectDialog', ctrl);
    }

    var dialogModel = oTableSelectDialog.getModel();
    if (!dialogModel) {
        dialogModel = new sas.risk.common.models.RiskJSONModel();
        oTableSelectDialog.setModel(dialogModel);
    }

    $.when(this.getAvailableBaselineScenarios(ctrl.model.workGroup)).done(function (availableBaselineScenarios) {
        dialogModel.setProperty("/scenarios", availableBaselineScenarios);
    });
    dialogModel.setProperty("/workGroup", ctrl.model.workGroup);
    dialogModel.setProperty("/selectedBaselineScenario", ctrl.selectedBaselineScenario);
    dialogModel.setProperty("/selectedDataSource", ctrl.selectedDataSource);

    oTableSelectDialog.setModel(view.getModel('i18N'), 'i18N');
    oTableSelectDialog.open();
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.getAvailableBaselineScenarios = function (workGroup) {
    var deferred = $.Deferred();

    sas.risk.scenariomanager.ScenarioAPI.getScenarioHorizons(workGroup).done(function (scenarios) {
        var availableBaselines = [];

        _.each(scenarios, function (scen) {
            if (scen.baselineId === "0" || scen.baselineId === undefined || scen.baselineId === null) {
                // assemble the available baselines
                availableBaselines.push(scen);
            }
        });

        deferred.resolve(availableBaselines);
    }).fail(function (event) {
        deferred.reject(event);
    });

    return deferred;
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.onScenarioSelected = function (base) {
    var ctrl = this;
    var scenario = ctrl.getModel();

    if (!base) {
        // todo: check if this is correct
        // reset the scenario to initial values
        scenario.periods = [sas.risk.scenariomanager.utils.ScenarioEditorUtil.getNewPeriod()];
        scenario.periodType = sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE;
        scenario.baselineId = "0";
        scenario.stresses = [];
        scenario.links = [];
    } else {
        scenario.periods = base.periods;
        scenario.periodType = base.periodType;

        // If the user hasn't set a version, use the version from the baseline
        // TODO: Check with users to see if this is really convenient
        if (!scenario.scenarioVersion) {
            scenario.scenarioVersion = base.scenarioVersion;
        }
        scenario.baselineId = base.id;

        // based on the current control flow, we can safely initialize the links to include the get the baseline's data
        // source link.
        // TODO: When this becomes possible in a scenario properties panel, then we'll need to let the user know
        // that the context is different than the baseline, and this should be become conditional -- setting the context
        // link only in cases where the scenario is being created.
        var dslink = _.findWhere(base.links, {rel: 'stressContext'});
        if (dslink) {
            scenario.links = [dslink];
        }
        scenario.stresses = [];
    }
    ctrl.setSelectedBaselineScenario(base);
    this.updateViewState();
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.mungeName = function (label) {
    var CTRL_CHARS = /[\x00-\x1F\x7F\u2029\u202b]/g,
        name = label.replace(CTRL_CHARS, "");

    name = name.replace(/\//g, "_");
    return (name.length <= 32) ? name : name.substring(0, 32);
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.updateViewState = function (valid) {
    this.view.getModel().updateBindings();

    var okButton = this.view.getBeginButton();
    if (okButton) {
        okButton.setEnabled(valid);
    }
};

sas.risk.scenariomanager.utils.ScenarioPropertyController.prototype.notUnique = function (ctxt, workGroup, name, label, version) {
    var upWorkGroup = workGroup ? workGroup.toUpperCase().trim() : undefined;
    var upName = name.toUpperCase().trim();
    var upLabel = label.toUpperCase().trim();
    var upVersion = (version) ? version.toUpperCase().trim() : "";
    return _.find(ctxt, function (scen) {
        // TODO: empty workgroup is equivalent to public workgroup
        return ((scen.workGroup) ? (scen.workGroup.toUpperCase() === upWorkGroup) :
                (scen.workGroup === upWorkGroup || upWorkGroup.length === 0)) &&
            (scen.name.toUpperCase() === upName || scen.label.toUpperCase() === upLabel) &&
            ((scen.scenarioVersion) ? scen.scenarioVersion.toUpperCase() === upVersion : upVersion.length === 0);
    });
};
