jQuery.sap.declare("sas.risk.scenariomanager.utils.ScenarioTableHelper");
jQuery.sap.require("sas.risk.scenariomanager.model.RiskVarTypeConstants");
jQuery.sap.require("sas.risk.scenariomanager.utils.ScenarioEditorUtil");

sap.ui.base.Object.extend("sas.risk.scenariomanager.utils.ScenarioTableHelper", {
    constructor: function (scenario, context) {
        this.update(scenario, context);
    }
});

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.update = function (scenario, context) {
    var that = this;
    this.rb = sap.ui.getCore().getLibraryResourceBundle("sas.risk.scenariomanager.nls");
    this.scenario = scenario;
    this.context = context;
    this.scenarioIndex = undefined;
    this.comma4 = new sap.ui.model.type.Float({
        decimals: 4,
        groupingEnabled: true
    });
    this.riskFactorRootIndex = -1;
    this.groupRootIndex = -1;
    this.arrayRootIndex = -1;
    this.driverRootIndex = -1;
    this.hiddenStresses = [];
    this.rootNode = {};
    this.rootNode.children = [];
    this.setColumns(scenario.periods);
    this.checkColumnVisibility();

    _.each(this.scenario.stresses, function (stress) {
        var oStress = that.context.getObject(stress.variable);
        if (undefined !== oStress && null !== oStress) {
            that.addRiskSourceToTreeData(that.rootNode, stress.variable, oStress.objectType, stress.perturbType, stress.perturbations);
        } else {
            that.hiddenStresses.push(stress);
        }
    });
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getScenario = function () {
    return this.scenario;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getContext = function () {
    return this.context;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getRiskFactorRootIndex = function () {
    return this.riskFactorRootIndex;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getGroupRootIndex = function () {
    return this.groupRootIndex;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getArrayRootIndex = function () {
    return this.arrayRootIndex;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getDriverRootIndex = function () {
    return this.driverRootIndex;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getHiddenStresses = function () {
    return this.hiddenStresses;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getRootNode = function () {
    return this.rootNode;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getColumns = function () {
    return this.rootNode.columns;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.setColumns = function (periods) {
    this.rootNode.columns = [{ columnType: "variable" }, { columnType: "baseValue" }, { columnType: "changeType" }];
    Array.prototype.push.apply(this.rootNode.columns, periods);
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.checkColumnVisibility = function () {
    var context = this.getContext();
    _.each(this.rootNode.columns, function (column) {
        if ("period" === column.columnType) {
            column.visible = sas.risk.scenariomanager.settings.ApplicationSettings.getShowObsoleteHorizons()
                          || !sas.risk.scenariomanager.utils.ScenarioEditorUtil.isHorizonObsolete(column, context.getRunDate());
        }
    });
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.addRiskSourceToTreeData = function (riskSourcesTreeData,
                                                                                               riskSourceName,
                                                                                               riskSourceType,
                                                                                               riskSourcePerturbType,
                                                                                               riskSourcePerturbations,
                                                                                               parentNodeIndex) {
    var that = this;
    var parentNode = null;
    var insertIndex;
    switch (riskSourceType) {
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE:
            if (this.riskFactorRootIndex === -1) {
                insertIndex = 0;
                // Risk factors do not yet exist, the risk factor node needs to be added...
                this.createTopLevelNode(riskSourceType,
                                        { children: [] },
                                        riskSourcesTreeData,
                                        insertIndex);
                this.shiftStoredRootIndices(true, riskSourceType);
            }
            parentNode = riskSourcesTreeData.children[this.riskFactorRootIndex];
            break;
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE:
            if (this.arrayRootIndex === -1) {
                insertIndex = 0;
                // Arrays do not yet exist, the array node needs to be added...
                if (this.riskFactorRootIndex > -1) {
                    insertIndex = 1;
                }
                this.createTopLevelNode(riskSourceType,
                                        { children: [] },
                                        riskSourcesTreeData,
                                        insertIndex);
                this.shiftStoredRootIndices(true, riskSourceType);
            }
            parentNode = riskSourcesTreeData.children[this.arrayRootIndex];
            break;
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE:
            if (this.groupRootIndex === -1) {
                insertIndex = 0;
                // Groups do not yet exist, the group node needs to be added...
                if (this.riskFactorRootIndex > -1) {
                    insertIndex = insertIndex + 1;
                }
                if (this.arrayRootIndex > -1) {
                    insertIndex = insertIndex + 1;
                }
                this.createTopLevelNode(riskSourceType,
                                        { children: [] },
                                        riskSourcesTreeData,
                                        insertIndex);
                this.shiftStoredRootIndices(true, riskSourceType);
            }
            parentNode = riskSourcesTreeData.children[this.groupRootIndex];
            break;
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE:
            if (this.driverRootIndex === -1) {
                insertIndex = 0;
                // Drivers do not yet exist, the root node needs to be added
                if (this.riskFactorRootIndex > -1) {
                    insertIndex = insertIndex + 1;
                }
                if (this.arrayRootIndex > -1) {
                    insertIndex = insertIndex + 1;
                }
                if (this.groupRootIndex > -1) {
                    insertIndex = insertIndex + 1;
                }
                this.createTopLevelNode(riskSourceType, { children: [] }, riskSourcesTreeData, insertIndex);
                this.shiftStoredRootIndices(true, riskSourceType);
            }
            parentNode = riskSourcesTreeData.children[this.driverRootIndex];
            break;
        default :
            break;
    }

    var perturbTypes = sas.risk.scenariomanager.utils.ScenarioTableHelper.getValidPerturbTypesForRiskSource(that, riskSourceName, riskSourceType, riskSourcePerturbations);

    var lineage = [parentNode];

    // Was the add successful?
    if (undefined === this.addRiskSourceToParent(riskSourcesTreeData, riskSourceName, riskSourceType, riskSourcePerturbType, riskSourcePerturbations, perturbTypes, lineage, parentNodeIndex, true)) {
        // The add was not successful. Check the parent to ensure other children exist, if not we need to remove the parent.
        if (null !== parentNode && parentNode.children.length === 0) {
            // The empty parent node should be removed from the tree.
            riskSourcesTreeData.children.splice(insertIndex, 1);
            // The stored root indices must also be adjusted.
            this.shiftStoredRootIndices(false, riskSourceType);
        }
    }
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.checkForVolatility = function (context, riskSource) {
    if (riskSource.objectType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) {
        return riskSource.hasOwnProperty("volatilityByName");
    } else {
        var hasVolatility = true;
        if (riskSource.riskFactorNames && riskSource.riskFactorNames.length > 0) {
            for (var riskFactorIndex = 0; riskFactorIndex < riskSource.riskFactorNames.length; ++riskFactorIndex) {
                var rf = context.getObject(riskSource.riskFactorNames[riskFactorIndex], sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE);
                hasVolatility = hasVolatility && sas.risk.scenariomanager.utils.ScenarioTableHelper.checkForVolatility(context, rf);
                if (!hasVolatility) {
                    // No need to keep checking, we found one without volatility.
                    return hasVolatility;
                }
            }
        }
        if (riskSource.riskFactorArrayNames && riskSource.riskFactorArrayNames.length > 0) {
            for (var riskFactorArrayIndex = 0; riskFactorArrayIndex < riskSource.riskFactorArrayNames.length; ++riskFactorArrayIndex) {
                var arr = context.getObject(riskSource.riskFactorArrayNames[riskFactorArrayIndex], sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE);
                hasVolatility = hasVolatility && sas.risk.scenariomanager.utils.ScenarioTableHelper.checkForVolatility(context, arr);
                if (!hasVolatility) {
                    // No need to keep checking, we found one without volatility.
                    return hasVolatility;
                }
            }
        }
        if (riskSource.riskFactorDriverNames && riskSource.riskFactorDriverNames.length > 0) {
            for (var riskFactorDriverIndex = 0; riskFactorDriverIndex < riskSource.riskFactorDriverNames.length; ++riskFactorDriverIndex) {
                var driver = context.getObject(riskSource.riskFactorDriverNames[riskFactorDriverIndex], sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE);
                hasVolatility = hasVolatility && sas.risk.scenariomanager.utils.ScenarioTableHelper.checkForVolatility(context, driver);
                if (!hasVolatility) {
                    // No need to keep checking, we found one without volatility.
                    return hasVolatility;
                }
            }
        }
        return hasVolatility;
    }
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.createTopLevelNode = function (objectType,
                                                                                          stresses,
                                                                                          rootNode,
                                                                                          indexInRoot) {
    switch (objectType) {
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE:
            this.riskFactorRootIndex = indexInRoot;
            stresses.variable = this.rb.getText("scenarioEditorTable.riskfactor.txt");
            break;
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE:
            this.arrayRootIndex = indexInRoot;
            stresses.variable = this.rb.getText("scenarioEditorTable.array.txt");
            break;
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE:
            this.groupRootIndex = indexInRoot;
            stresses.variable = this.rb.getText("scenarioEditorTable.group.txt");
            break;
        case sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE:
            this.driverRootIndex = indexInRoot;
            stresses.variable = this.rb.getText("scenarioEditorTable.driver.txt");
            break;
    }
    stresses.readOnly = true;
    stresses.rowType = "topLevel";
    stresses.objectType = objectType;
    rootNode.children.splice(indexInRoot, 0, stresses);
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.shiftStoredRootIndices = function (isInsert, changedType) {
    var shiftValue = 1; // Default is to assume insert (shift following nodes by +1).
    // Was a new top level element added?
    if (false === isInsert) {
        // A top level node was removed, any following nodes must be shifted by -1.
        shiftValue = -1;
        // Update the root index for the removed type.
        if (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) {
            this.riskFactorRootIndex = -1;
        } else if (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE) {
            this.arrayRootIndex = -1;
        } else if (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE) {
            this.groupRootIndex = -1;
        } else if (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE) {
            this.driverRootIndex = -1;
        }
    }

    if ((changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) ||
        (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE) ||
        (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE)) {
        // Adding or removing the risk factor, array, or group root can affect the index of the driver root.
        if (this.driverRootIndex > -1) {
            this.driverRootIndex = this.driverRootIndex + shiftValue;
        }
    }

    if ((changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) ||
        (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE)) {
        // Adding or removing the risk factor and array roots can affect the index of the group root.
        if (this.groupRootIndex > -1) {
            this.groupRootIndex = this.groupRootIndex + shiftValue;
        }
    }

    if (changedType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) {
        // Adding or removing risk factor root can affect the index of the array root.
        if (this.arrayRootIndex > -1) {
            this.arrayRootIndex = this.arrayRootIndex + shiftValue;
        }
    }

};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.setDefaultPerturbationInfo = function (riskSourceNode) {
    var scenario = this.getScenario();
    var numHorizons = scenario.periods.length;
    var perturbations = new Array(numHorizons);
    sas.risk.scenariomanager.utils.ScenarioEditorUtil.arrayFill(perturbations, 0);
    perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(perturbations);

    riskSourceNode.perturbType = "REL";
    riskSourceNode.perturbations = perturbations;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.addRiskSourceToParent = function (riskSourcesTreeData,
                                                                                               riskSourceName,
                                                                                               riskSourceType,
                                                                                               riskSourcePerturbType,
                                                                                               riskSourcePerturbations,
                                                                                               perturbTypes,
                                                                                               lineage,
                                                                                               parentNodeIndex,
                                                                                               pullNewValues) {
    var contextData = this.getContext();
    var riskSource = contextData.getObject(riskSourceName, riskSourceType);
    if (riskSource) {
        var parentNode = lineage[lineage.length - 1];
        if (undefined === parentNode.children) {
            parentNode.children = [];
        }
        var newIndex;
        if (parentNodeIndex !== undefined) {
            newIndex = parentNodeIndex;
        } else {
            newIndex = parentNode.children.length;
        }

        var indexObj = this.getFromIndex(riskSourceType, riskSourceName);

        var riskSourceNode = {
            variable: riskSource.name,
            baseValue: this.getBaseValueFromContext(this.getContext(), riskSource.name),
            perturbTypes: perturbTypes
        };
        if (riskSourcePerturbType && riskSourcePerturbations) {
            riskSourceNode.perturbType = riskSourcePerturbType;
            riskSourceNode.perturbations = riskSourcePerturbations;
        } else if (parentNode.perturbType && parentNode.perturbations) {
            riskSourceNode.perturbType = parentNode.perturbType;
            riskSourceNode.perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(parentNode.perturbations);
        } else {
            this.setDefaultPerturbationInfo(riskSourceNode);
        }

        var parentIsTopLevel = (parentNode.rowType === "topLevel");
        riskSourceNode.readOnly = !parentIsTopLevel;
        lineage.push(riskSourceNode);

        if (riskSourceType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) {
            if (!indexObj || (parentIsTopLevel && !indexObj.individual) ||
                ((parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE) && !indexObj.group) ||
                (parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE) ||
                (parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE)) {

                // // Are we adding risk factors within a group? If so, we made need to prune some duplicates.
                // if (parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE) {
                //     var parentDetails = contextData.getObject(parentNode.variable, parentNode.rowType);
                //     if (parentDetails.riskFactorArrayNames && parentDetails.riskFactorArrayNames.length > 0) {
                //         // Iterate through the group's arrays until one is found that contains the risk factor (if one exists)
                //         var foundArray = _.find(parentDetails.riskFactorArrayNames, function (arrayName) {
                //             var arrayDetails = contextData.getObject(arrayName, sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE);
                //             return _.find(arrayDetails.riskFactorNames, function(riskFactorName){
                //                 return riskFactorName === riskSource.name;
                //             });
                //         });
                //         // Only add the risk factor to the group if it isn't already accounted for in one of the grouped arrays
                //         if (undefined !== foundArray) {
                //             // Don't do the add (pruning)
                //             return;
                //         }
                //     }
                // }
                riskSourceNode.rowType = riskSourceType;
                this.adjustIndex(true, riskSourcesTreeData, riskSourceType, riskSourceName, lineage);
            } else {
                // the user is trying to add a risk factor that already exists as an individual. For now, just ignoring...
                // later may want to change its location in the list.
                return;
            }
        } else if (riskSourceType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE ||
                   riskSourceType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE) {
            if (!indexObj || (parentIsTopLevel && !indexObj.individual) ||
                ((parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE) && !indexObj.group) ||
                (parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE)) {

                riskSourceNode.rowType = riskSourceType;
                if (riskSource.riskFactorNames && riskSource.riskFactorNames.length > 0) {
                    for (var arrayChildIndex = 0; arrayChildIndex < riskSource.riskFactorNames.length; ++arrayChildIndex) {
                        this.addRiskSourceToParent(riskSourcesTreeData, // tree data
                                                   riskSource.riskFactorNames[arrayChildIndex], // risk source name
                                                   sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE, // risk source type
                                                   undefined, // perturbation type
                                                   undefined, // perturbations
                                                   sas.risk.scenariomanager.model.PerturbationType.values(),
                                                   lineage.slice(), // lineage
                                                   undefined, // index in parent
                                                   false); // pull new values
                    }
                }
                if (riskSource.riskFactorArrayNames && riskSource.riskFactorArrayNames.length > 0) {
                    for (var driverArrayChildIndex = 0; driverArrayChildIndex < riskSource.riskFactorArrayNames.length; ++driverArrayChildIndex) {
                        this.addRiskSourceToParent(riskSourcesTreeData, // tree data
                                                   riskSource.riskFactorArrayNames[driverArrayChildIndex], // risk source name
                                                   sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE, // risk source type
                                                   undefined, // perturbation type
                                                   undefined, // perturbations
                                                   sas.risk.scenariomanager.model.PerturbationType.values(),
                                                   lineage.slice(), // lineage
                                                   undefined, // index in parent
                                                   false); // pull new values
                    }
                }
                if (riskSource.riskFactorDriverNames && riskSource.riskFactorDriverNames.length > 0) {
                    for (var subDriverChildIndex = 0; subDriverChildIndex < riskSource.riskFactorDriverNames.length; ++subDriverChildIndex) {
                        this.addRiskSourceToParent(riskSourcesTreeData, // tree data
                                                   riskSource.riskFactorDriverNames[subDriverChildIndex], // risk source name
                                                   sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE, // risk source type
                                                   undefined, // perturbation type
                                                   undefined, // perturbations
                                                   sas.risk.scenariomanager.model.PerturbationType.values(),
                                                   lineage.slice(), // lineage
                                                   undefined, // index in parent
                                                   false); // pull new values
                    }
                }
                this.adjustIndex(true, riskSourcesTreeData, riskSourceType, riskSourceName, lineage);
            } else {
                // the user is trying to add a risk factor that already exists as an individual. For now, just ignoring...
                // later may want to change its location in the list.
                return;
            }
        } else if (riskSourceType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE) {
            if (!indexObj || (parentIsTopLevel && !indexObj.individual)) {
                riskSourceNode.rowType = riskSourceType;
                if (riskSource.riskFactorNames && riskSource.riskFactorNames.length > 0) {
                    for (var groupChildIndex = 0; groupChildIndex < riskSource.riskFactorNames.length; ++groupChildIndex) {
                        this.addRiskSourceToParent(riskSourcesTreeData, // tree data
                                                   riskSource.riskFactorNames[groupChildIndex], // risk source name
                                                   sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE, // risk source type
                                                   undefined, // perturbation type
                                                   undefined, // perturbations
                                                   sas.risk.scenariomanager.model.PerturbationType.values(),
                                                   lineage.slice(), // lineage
                                                   undefined, // index in parent
                                                   false); // pull new values
                    }
                }
                if (riskSource.riskFactorArrayNames && riskSource.riskFactorArrayNames.length > 0) {
                    for (var groupArrayChildIndex = 0; groupArrayChildIndex < riskSource.riskFactorArrayNames.length; ++groupArrayChildIndex) {
                        this.addRiskSourceToParent(riskSourcesTreeData,
                                                   riskSource.riskFactorArrayNames[groupArrayChildIndex], // risk source name
                                                   sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE, // risk source type
                                                   undefined, // perturbation type
                                                   undefined, // perturbations
                                                   sas.risk.scenariomanager.model.PerturbationType.values(),
                                                   lineage.slice(), // lineage
                                                   undefined, // index in parent
                                                   false); // pull new values
                    }
                }
                this.adjustIndex(true, riskSourcesTreeData, riskSourceType, riskSourceName, lineage);
            } else {
                // the user is trying to add a risk factor that already exists as an individual. For now, just ignoring...
                // later may want to change its location in the list.
                return;
            }
        } else {
            return;
        }

        riskSourceNode.parent = parentNode;
        parentNode.children.splice(newIndex, 0, riskSourceNode);
        if (pullNewValues) {
            this.pullNewValues(riskSourceNode);
        }
    } else {
        return;
    }
    return { riskSource: riskSource, index: newIndex };
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.adjustIndex = function (isAdd,
                                                                                   riskSourcesTreeData,
                                                                                   type,
                                                                                   name,
                                                                                   lineage) {
    if (!this.scenarioIndex) {
        this.scenarioIndex = {
            riskFactors: {},
            arrays: {},
            groups: {},
            drivers: {}
        };
    }

    var obj = null,
        upperName = name.toUpperCase();

    if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE === type) {
        if (isAdd && !this.scenarioIndex.riskFactors[upperName]) {
            // Default structure for risk factors
            this.scenarioIndex.riskFactors[upperName] = { individual: null, arrays: {}, group: null, drivers: {} };
        }
        obj = this.scenarioIndex.riskFactors[upperName];
    } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE === type) {
        if (isAdd && !this.scenarioIndex.arrays[upperName]) {
            // Default structure for arrays
            this.scenarioIndex.arrays[upperName] = { individual: null, group: null, drivers: {} };
        }
        obj = this.scenarioIndex.arrays[upperName];
    } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE === type) {
        if (isAdd && !this.scenarioIndex.groups[upperName]) {
            // Default structure for groups
            this.scenarioIndex.groups[upperName] = { individual: null };
        }
        obj = this.scenarioIndex.groups[upperName];
    } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE === type) {
        if (isAdd && !this.scenarioIndex.drivers[upperName]) {
            // Default structure for drivers
            this.scenarioIndex.drivers[upperName] = { individual: null, driver: null }; // drivers can have subdrivers
        }
        obj = this.scenarioIndex.drivers[upperName];
    }

    var riskSourceNode = null;
    var parentNode = null;
    if (lineage) {
        if (lineage.length > 0 && lineage[lineage.length - 1]) {
            riskSourceNode = lineage[lineage.length - 1];
            if (lineage.length > 1 && lineage[lineage.length - 2]) {
                parentNode = lineage[lineage.length - 2];
            }
        }
    }

    var sourceIndex = null;
    if (isAdd) {
        sourceIndex = {
            node: riskSourceNode,
            lineage: lineage
        };
    }

    if (!parentNode || !parentNode.rowType || parentNode.rowType === "topLevel") {
        obj.individual = sourceIndex;
    } else if (parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE) {
        if (!obj.arrays[parentNode.variable]) {
            obj.arrays[parentNode.variable] = {
                individual: null, // depth of 3
                group: null, // depth of 4
                drivers: {}
            };
        }

        if (lineage.length === 3 && lineage[0].objectType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE) {
            // Adding to an individual array
            obj.arrays[parentNode.variable].individual = sourceIndex;
        } else if (lineage.length === 4 && lineage[0].objectType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE) {
            // Adding to a grouped array
            obj.arrays[parentNode.variable].group = sourceIndex;
        } else if (lineage.length === 4 && lineage[0].objectType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE) {
            if (!obj.arrays[parentNode.variable].drivers[lineage[1].variable]) {
                obj.arrays[parentNode.variable].drivers[lineage[1].variable] = {
                    individual: null, // depth of 3
                    driver: null // depth of 4
                };
            }
            // Adding to an array in an individual driver
            obj.arrays[parentNode.variable].drivers[lineage[1].variable].individual = sourceIndex;

        } else if (lineage.length === 5 && lineage[0].objectType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE) {
            if (!obj.arrays[parentNode.variable].drivers[lineage[1].variable]) {
                obj.arrays[parentNode.variable].drivers[lineage[1].variable] = {
                    individual: null, // depth of 3
                    driver: null // depth of 4
                };
            }
            // Adding to an array in a sub-driver
            obj.arrays[parentNode.variable].drivers[lineage[1].variable].driver = sourceIndex;
        }

    } else if (parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE) {
        obj.group = sourceIndex;
    } else if (parentNode.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE) {
        // Is it a subdriver?
        if (sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE === type) {
            obj.driver = sourceIndex;

        } else {
            if (!obj.drivers[parentNode.variable]) {
                obj.drivers[parentNode.variable] = {
                    individual: null, // depth of 3
                    driver: null // depth of 4
                };
            }

            if (lineage.length === 3 && lineage[0].objectType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE) {
                // Adding to an individual driver
                obj.drivers[parentNode.variable].individual = sourceIndex;
            } else if (lineage.length === 4 && lineage[0].objectType === sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE) {
                // Adding to a driver as a subdriver
                obj.drivers[parentNode.variable].driver = sourceIndex;
            }
        }
    }
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getFromIndex = function (type, name) {
    var upperName = name.toUpperCase();
    if (this.scenarioIndex) {
        if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE === type) {
            return this.scenarioIndex.riskFactors[upperName];
        } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE === type) {
            return this.scenarioIndex.arrays[upperName];
        } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE === type) {
            return this.scenarioIndex.groups[upperName];
        } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE === type) {
            return this.scenarioIndex.drivers[upperName];
        }
    }
    return undefined;
};

// We need to get an existing node with perturbations. In this case, precedence is not important, so sorting is unnecessary.
sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getNodeFromIndex = function (indexObj) {
    var arrayNames = null;
    if (indexObj.arrays && Object.keys(indexObj.arrays).length > 0) {
        arrayNames = Object.keys(indexObj.arrays);
    }
    var driverNames;
    if (indexObj.drivers && Object.keys(indexObj.drivers).length > 0) {
        driverNames = Object.keys(indexObj.drivers);
    } else if (indexObj.driver) {
        driverNames = [indexObj.driver.node.variable];
    }

    if (arrayNames && arrayNames.length > 0) {
        for (var arrayIndex = 0; arrayIndex < arrayNames.length; ++arrayIndex) {
            var arrayName = arrayNames[arrayIndex];
            if (indexObj.arrays[arrayName].individual) {
                // An individual array exists, grab it. (precedence doesn't matter for this use)
                return indexObj.arrays[arrayName].individual.node;
            }
            if (indexObj.arrays[arrayName].group) {
                // A grouped array exists, grab it. (precedence doesn't matter for this use)
                return indexObj.arrays[arrayName].group.node;
            }

            if (indexObj.arrays[arrayName].drivers && Object.keys(indexObj.arrays[arrayName].drivers)) {
                var driverNamesForArray = Object.keys(indexObj.arrays[arrayName].drivers);
                for (var arrayDriverIndex = 0; arrayDriverIndex < driverNamesForArray.length; ++arrayDriverIndex) {
                    var arrayDriverIndexObj = indexObj.arrays[arrayName].drivers[driverNamesForArray[arrayDriverIndex]];
                    if (arrayDriverIndexObj.individual) {
                        // A driver in an array exists, grab it. (precedence doesn't matter for this use)
                        return arrayDriverIndexObj.individual.node;
                    }
                    if (arrayDriverIndexObj.driver) {
                        // A subdriver in an array exists, grab it. (precedence doesn't matter for this use)
                        return arrayDriverIndexObj.driver.node;
                    }
                }
            }
        }
    }
    if (indexObj.group) {
        // Group exists
        return indexObj.group.node;
    }
    if (driverNames && driverNames.length > 0) {
        for (var driverIndex = 0; driverIndex < driverNames.length; ++driverIndex) {
            var driverName = driverNames[driverIndex];
            if (indexObj.drivers && indexObj.drivers[driverName]) {
                if (indexObj.drivers[driverName].individual) {
                    // A driver exists individually, grab it.
                    return indexObj.drivers[driverName].individual.node;
                }
                if (indexObj.drivers[driverName].driver) {
                    // A driver exists within another driver, grab it.
                    return indexObj.drivers[driverName].driver.node;
                }
            } else if (indexObj.driver && indexObj.driver.node.variable === driverName) {
                return indexObj.driver.node;
            }
        }
    }
};

/*
 * Signal to the risk source that it needs to check to see if it has the latest value for the specified (horizon) index.
 * If no (horizon) index is specified, all will be checked. First need to find the occurrence with the highest precedence,
 * then go through all other occurrences and update them with the value from the first (and override information).
 *
 * Precedence order is as follows:
 * 1. Individual risk factor
 * 2. Individual array (first found instance in a top-down search, as risk factors can exist in multiple arrays at once)
 * 3. Group or grouped array (first found instance in a top-down search, as the individual group may exist below the grouped array)
 * 4. Driver, sub-driver, or array in a driver (first found instance in a top-down search, as these can exist in different orders)
 */
sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.pullNewValues = function (riskFactor, index) {
    var horizonIndex;
    var horizonEndIndex;
    if (undefined === index) {
        // Index was not specified, all horizons are to be checked for updates.
        horizonIndex = 0;
        horizonEndIndex = riskFactor.perturbations.length - 1;
    } else {
        // Index was specified, only the current horizon should be checked for updates. (Or no horizon, if -1)
        horizonIndex = index;
        horizonEndIndex = index;
    }

    var indexObj = this.getFromIndex(riskFactor.rowType, riskFactor.variable);
    if (!indexObj)
        return;

    var valueToUse;
    var perturbationType;
    var overrideInfo; // { type: "individual", name: "myRF" } or { type: "array", name: "myArray" }
    var parent;
    var lineageIndex;
    var perturbType;
    var finalValueToUse;

    var individualDrivers;
    var subDrivers;
    var arrayDrivers = [];
    if (sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE !== riskFactor.rowType) {
        individualDrivers = this.getObjectsByIndexType(sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE, indexObj, "individual");
        subDrivers = this.getObjectsByIndexType(sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE, indexObj, "driver");
        var arraysInDriversArr = this.getObjectsByIndexType(sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE, indexObj, "drivers");
        for (var arraysInDriversIndex = 0; arraysInDriversIndex < arraysInDriversArr.length; ++arraysInDriversIndex) {
            var arrayDriverNames = Object.keys(arraysInDriversArr[arraysInDriversIndex]);
            for (var arrayDriverNamesIndex = 0; arrayDriverNamesIndex < arrayDriverNames.length; ++arrayDriverNamesIndex) {
                var arrayInDriverIndexObj = arraysInDriversArr[arraysInDriversIndex][arrayDriverNames[arrayDriverNamesIndex]];
                if (arrayInDriverIndexObj.driver) {
                    arrayDrivers.push(arrayInDriverIndexObj.driver);
                }
                if (arrayInDriverIndexObj.individual) {
                    arrayDrivers.push(arrayInDriverIndexObj.individual);
                }
            }
        }
    } else {
        // Special handling for subdrivers
        individualDrivers = [];
        if (indexObj.individual) {
            individualDrivers.push(indexObj.individual);
        }
        subDrivers = [];
        if (indexObj.driver) {
            subDrivers.push(indexObj.driver);
        }
    }
    var orderedDrivers;
    if (0 < individualDrivers.length || 0 < subDrivers.length || 0 < arrayDrivers.length) {
        // Sort all DRIVERS by precedence so we can find the highest
        Array.prototype.push.apply(individualDrivers, subDrivers);
        Array.prototype.push.apply(individualDrivers, arrayDrivers);
        orderedDrivers = this.orderByPrecedence(individualDrivers, 1);
    }

    // If the individual exists in the scenario, all values are to be updated without prejudice
    if (indexObj.individual && ((sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE !== indexObj.individual.node.rowType) ||
                                (orderedDrivers[0].lineage[1].variable === indexObj.individual.node.variable))) {
        var arrayNames = null;
        if (indexObj.arrays && Object.keys(indexObj.arrays).length > 0) {
            arrayNames = Object.keys(indexObj.arrays);
        }
        var driverNames;
        if (indexObj.drivers && Object.keys(indexObj.drivers).length > 0) {
            driverNames = Object.keys(indexObj.drivers);
        } else if (indexObj.driver) {
            driverNames = [indexObj.driver.node.variable];
        }
        this.updatePerturbationTypeAndBaseValueFromIndividual(indexObj, arrayNames, driverNames);
        // Need to iterate through the horizons, we might be updating a single horizon or all horizons (depending on what triggered the pull)
        while (horizonIndex > -1 && horizonIndex <= horizonEndIndex) {
            valueToUse = indexObj.individual.node.perturbations[horizonIndex].value;

            // calc finalValueToUse
            perturbType = sas.risk.scenariomanager.model.PerturbationType.byCode(riskFactor.perturbType);
            finalValueToUse = perturbType.computeValue(this.context, this.scenario, riskFactor, riskFactor.variable, horizonIndex, valueToUse);
            riskFactor.perturbations[horizonIndex].finalValue = finalValueToUse;

            // go through all instances and update the value to the individual value
            if (arrayNames && arrayNames.length > 0) {
                for (var arrayIndex = 0; arrayIndex < arrayNames.length; ++arrayIndex) {
                    var arrayName = arrayNames[arrayIndex];
                    if (indexObj.arrays[arrayName].individual) {
                        // An array exists individually.
                        this.addValueAndOverrideInfo(indexObj.arrays[arrayName].individual.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
                    }
                    if (indexObj.arrays[arrayName].group) {
                        // An array exists within a group.
                        this.addValueAndOverrideInfo(indexObj.arrays[arrayName].group.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
                    }

                    if (indexObj.arrays[arrayName].drivers && Object.keys(indexObj.arrays[arrayName].drivers)) {
                        var driverNamesForArray = Object.keys(indexObj.arrays[arrayName].drivers);
                        for (var arrayDriverIndex = 0; arrayDriverIndex < driverNamesForArray.length; ++arrayDriverIndex) {
                            var arrayDriverIndexObj = indexObj.arrays[arrayName].drivers[driverNamesForArray[arrayDriverIndex]];
                            if (arrayDriverIndexObj.individual) {
                                // An array exists within a driver.
                                this.addValueAndOverrideInfo(arrayDriverIndexObj.individual.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
                            }
                            if (arrayDriverIndexObj.driver) {
                                // An array exists within a sub-driver.
                                this.addValueAndOverrideInfo(arrayDriverIndexObj.driver.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
                            }
                        }
                    }
                }
            }
            if (indexObj.group) {
                // Group exists
                this.addValueAndOverrideInfo(indexObj.group.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
            }
            if (driverNames && driverNames.length > 0) {
                for (var driverIndex = 0; driverIndex < driverNames.length; ++driverIndex) {
                    var driverName = driverNames[driverIndex];
                    if (indexObj.drivers && indexObj.drivers[driverName]) {
                        if (indexObj.drivers[driverName].individual) {
                            // A driver exists individually.
                            this.addValueAndOverrideInfo(indexObj.drivers[driverName].individual.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
                        }
                        if (indexObj.drivers[driverName].driver) {
                            // A driver exists within another driver.
                            this.addValueAndOverrideInfo(indexObj.drivers[driverName].driver.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
                        }
                    } else if (indexObj.driver && indexObj.driver.node.variable === driverName) {
                        this.addValueAndOverrideInfo(indexObj.driver.node.perturbations[horizonIndex], valueToUse, "individual", riskFactor.variable, finalValueToUse);
                    }
                }
            }

            ++horizonIndex;
        }
    } else {
        // The individual wasn't updated, so now we need to go through and determine the occurrence of the risk source
        // with the highest precedent, then apply that to all other instances (and children, if any exist).

        // A risk source being included in an individual array gets the second highest precedent (the first instance found, in a top-down search).
        var individualArrays = this.getObjectsByIndexType(sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE, indexObj, "individual");
        var orderedArrays;
        if (0 < individualArrays.length) {
            // Sort the ARRAYS by precedence so we can find the highest.
            orderedArrays = this.orderByPrecedence(individualArrays, 1);
        }

        // A risk source being included in an individual group OR grouped array gets the third highest precedent (the first
        // instance found, in a top-down search - could be a direct child of a group or a child of a grouped array).
        var groupedArrays = this.getObjectsByIndexType(sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE, indexObj, "group");
        var orderedObjects;
        if (0 < groupedArrays.length) {
            // If the risk factor exists in a group directly, it also needs to be ordered at this time.
            if (indexObj.group) {
                groupedArrays.push(indexObj.group);
            }
            // Sort the GROUPS by precedence so that we can find the highest.
            orderedObjects = this.orderByPrecedence(groupedArrays, 1);
        }

        do {
            valueToUse = undefined;
            perturbType = undefined;
            finalValueToUse = undefined;
            perturbationType = undefined;
            overrideInfo = null;
            parent = null;
            if (orderedArrays) {
                // Ordered arrays exist, iterate through all of them. First to get the value, then to set it and the
                // override information for all other instances.
                for (var arrIndex = 0; arrIndex < orderedArrays.length; ++arrIndex) {
                    lineageIndex = 1; // Individual arrays are always at a depth of 1 in the lineage array.
                    parent = orderedArrays[arrIndex].lineage[lineageIndex];
                    if (horizonIndex > -1) {
                        if (undefined === valueToUse) {
                            valueToUse = parent.perturbations[horizonIndex].value;
                            orderedArrays[arrIndex].node.perturbations[horizonIndex].value = valueToUse;

                            perturbType = sas.risk.scenariomanager.model.PerturbationType.byCode(orderedArrays[arrIndex].node.perturbType);
                            finalValueToUse = perturbType.computeValue(this.context, this.scenario, orderedArrays[arrIndex].node, orderedArrays[arrIndex].node.variable, horizonIndex, orderedArrays[arrIndex].node.perturbations[horizonIndex].value);
                            orderedArrays[arrIndex].node.perturbations[horizonIndex].finalValue = finalValueToUse;

                            // Setup override information for any other instances of the risk source.
                            overrideInfo = {
                                type: parent.rowType,
                                name: parent.variable
                            };
                        } else {
                            // The value was determined by a previously found occurrence, we just need to set it and the override information here.
                            this.addValueAndOverrideInfo(orderedArrays[arrIndex].node.perturbations[horizonIndex], valueToUse, overrideInfo.type, overrideInfo.name, finalValueToUse);
                        }
                    }
                    if (undefined === perturbationType) {
                        perturbationType = parent.perturbType;
                    }
                    orderedArrays[arrIndex].node.perturbType = perturbationType;
                }
            }

            if (indexObj.group || orderedObjects) {
                if (indexObj.group && 0 === groupedArrays.length) {
                    lineageIndex = 1;
                    parent = indexObj.group.lineage[lineageIndex];
                    if (horizonIndex > -1) {
                        if (undefined === valueToUse) {
                            // Take the value from the group object.
                            valueToUse = parent.perturbations[horizonIndex].value;
                            indexObj.group.node.perturbations[horizonIndex].value = valueToUse;

                            perturbType = sas.risk.scenariomanager.model.PerturbationType.byCode(indexObj.group.node.perturbType);
                            finalValueToUse = perturbType.computeValue(this.context, this.scenario, indexObj.group.node, indexObj.group.node.variable, horizonIndex, indexObj.group.node.perturbations[horizonIndex].value);
                            indexObj.group.node.perturbations[horizonIndex].finalValue = finalValueToUse;

                            // Setup override information for any other instances of the risk source.
                            overrideInfo = {
                                type: parent.rowType,
                                name: parent.variable
                            };
                        } else {
                            // The value was determined by a previously found occurrence, we just need to set it and the override information here.
                            this.addValueAndOverrideInfo(indexObj.group.node.perturbations[horizonIndex], valueToUse, overrideInfo.type, overrideInfo.name, finalValueToUse);
                        }
                    }
                    if (undefined === perturbationType) {
                        perturbationType = parent.perturbType;
                    }
                    indexObj.group.node.perturbType = perturbationType;
                } else if (orderedObjects) {
                    for (var objectIndex = 0; objectIndex < orderedObjects.length; ++objectIndex) {
                        // We might be dealing with a group or a grouped array, so the index in the lineage will always
                        // be 2 less than the length of the lineage array (last value in lineage is the object itself).
                        lineageIndex = orderedObjects[objectIndex].lineage.length - 2;
                        parent = orderedObjects[objectIndex].lineage[lineageIndex];
                        if (horizonIndex > -1) {
                            if (undefined === valueToUse) {
                                valueToUse = parent.perturbations[horizonIndex].value;
                                orderedObjects[objectIndex].node.perturbations[horizonIndex].value = valueToUse;

                                perturbType = sas.risk.scenariomanager.model.PerturbationType.byCode(orderedObjects[objectIndex].node.perturbType);
                                finalValueToUse = perturbType.computeValue(this.context, this.scenario, orderedObjects[objectIndex].node, orderedObjects[objectIndex].node.variable, horizonIndex, orderedObjects[objectIndex].node.perturbations[horizonIndex].value);
                                orderedObjects[objectIndex].node.perturbations[horizonIndex].finalValue = finalValueToUse;

                                // Setup override information for any other instances of the risk source.
                                overrideInfo = {
                                    type: parent.rowType,
                                    name: parent.variable
                                };
                            } else {
                                // The value was determined by a previously found occurrence, we just need to set it and the override information here.
                                this.addValueAndOverrideInfo(orderedObjects[objectIndex].node.perturbations[horizonIndex], valueToUse, overrideInfo.type, overrideInfo.name, finalValueToUse);
                            }
                        }
                        if (undefined === perturbationType) {
                            perturbationType = parent.perturbType;
                        }
                        orderedObjects[objectIndex].node.perturbType = perturbationType;
                    }
                }
            }

            if (orderedDrivers) {
                // Ordered drivers exist, iterate through all of them. First to get the value, then to set it and the
                // override information for all other instances.
                for (var orderedDriverIndex = 0; orderedDriverIndex < orderedDrivers.length; ++orderedDriverIndex) {
                    // lineageIndex = 1; // Individual drivers are always at a depth of 1 in the lineage array.
                    lineageIndex = orderedDrivers[orderedDriverIndex].lineage.length - 2;
                    parent = orderedDrivers[orderedDriverIndex].lineage[lineageIndex];
                    if (horizonIndex > -1) {
                        if (undefined === valueToUse) {
                            valueToUse = parent.perturbations[horizonIndex].value;
                            orderedDrivers[orderedDriverIndex].node.perturbations[horizonIndex].value = valueToUse;

                            perturbType = sas.risk.scenariomanager.model.PerturbationType.byCode(orderedDrivers[orderedDriverIndex].node.perturbType);
                            finalValueToUse = perturbType.computeValue(this.context, this.scenario, orderedDrivers[orderedDriverIndex].node, orderedDrivers[orderedDriverIndex].node.variable, horizonIndex, orderedDrivers[orderedDriverIndex].node.perturbations[horizonIndex].value);
                            orderedDrivers[orderedDriverIndex].node.perturbations[horizonIndex].finalValue = finalValueToUse;

                            // Setup override information for any other instances of the risk source.
                            overrideInfo = {
                                type: parent.rowType,
                                name: parent.variable
                            };
                        } else {
                            // The value was determined by a previously found occurrence, we just need to set it and the override information here.
                            this.addValueAndOverrideInfo(orderedDrivers[orderedDriverIndex].node.perturbations[horizonIndex], valueToUse, overrideInfo.type, overrideInfo.name, finalValueToUse);
                        }
                    }
                    if (undefined === perturbationType) {
                        perturbationType = parent.perturbType;
                    }
                    orderedDrivers[orderedDriverIndex].node.perturbType = perturbationType;
                }
            }
            ++horizonIndex;
        } while (horizonIndex <= horizonEndIndex);
    }

    // go through all children (if there are any) and trigger them to pull new values
    if (riskFactor.children) {
        for (var childIndex = 0; childIndex < riskFactor.children.length; ++childIndex) {
            this.pullNewValues(riskFactor.children[childIndex], index);
        }
    }
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.addValueAndOverrideInfo = function (pertObject,
                                                                                               valueToUse,
                                                                                               overrideType,
                                                                                               overrideName, 
                                                                                                 finalValueToUse) {
    pertObject.value = valueToUse;
    pertObject.overrideInfo = {
        type: overrideType,
        name: overrideName
    };
    pertObject.finalValue = finalValueToUse;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.checkDescendantsForOverrides = function (children, idx) {
    if (children && children.length > 0) {
        for (var childIndex = 0; childIndex < children.length; ++childIndex) {
            var pertObject = children[childIndex].perturbations[idx];
            if (undefined !== pertObject.overrideInfo && null !== pertObject.overrideInfo) {
                return true;
            }
            if (this.checkDescendantsForOverrides(children[childIndex].children, idx)) {
                return true;
            }
        }
    }
    return false;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.updatePerturbationTypeAndBaseValueFromIndividual = function (indexObj,
                                                                                                                        arrayNames,
                                                                                                                        driverNames) {
    var pertType = indexObj.individual.node.perturbType;
    var baseValue = indexObj.individual.node.baseValue;
    // Update the base value and perturbation types for the cloned/read-only risk factors
    // go through all instances and update the value to the individual value
    if (arrayNames && arrayNames.length > 0) {
        for (var arrayIndex = 0; arrayIndex < arrayNames.length; ++arrayIndex) {
            var arrayName = arrayNames[arrayIndex];
            if (indexObj.arrays[arrayName].individual) {
                indexObj.arrays[arrayName].individual.node.perturbType = pertType;
                indexObj.arrays[arrayName].individual.node.baseValue = baseValue;
            }
            if (indexObj.arrays[arrayName].group) {
                indexObj.arrays[arrayName].group.node.perturbType = pertType;
                indexObj.arrays[arrayName].group.node.baseValue = baseValue;
            }
            if (indexObj.arrays[arrayName].drivers && Object.keys(indexObj.arrays[arrayName].drivers)) {
                var driverNamesForArray = Object.keys(indexObj.arrays[arrayName].drivers);
                for (var arrayDriverIndex = 0; arrayDriverIndex < driverNamesForArray.length; ++arrayDriverIndex) {
                    var arrayDriverIndexObj = indexObj.arrays[arrayName].drivers[driverNamesForArray[arrayDriverIndex]];
                    if (arrayDriverIndexObj.individual) {
                        // An array exists within a driver.
                        arrayDriverIndexObj.individual.node.perturbType = pertType;
                        arrayDriverIndexObj.individual.node.baseValue = baseValue;
                    }
                    if (arrayDriverIndexObj.driver) {
                        // An array exists within a sub-driver.
                        arrayDriverIndexObj.driver.node.perturbType = pertType;
                        arrayDriverIndexObj.driver.node.baseValue = baseValue;
                    }
                }
            }
        }
    }
    if (indexObj.group) {
        // Group exists
        indexObj.group.node.perturbType = pertType;
        indexObj.group.node.baseValue = baseValue;
    }
    if (driverNames && driverNames.length > 0) {
        for (var driverIndex = 0; driverIndex < driverNames.length; ++driverIndex) {
            var driverName = driverNames[driverIndex];
            if (indexObj.drivers && indexObj.drivers[driverName]) {
                if (indexObj.drivers[driverName].individual) {
                    indexObj.drivers[driverName].individual.node.perturbType = pertType;
                    indexObj.drivers[driverName].individual.node.baseValue = baseValue;
                }
                if (indexObj.drivers[driverName].driver) {
                    indexObj.drivers[driverName].driver.node.perturbType = pertType;
                    indexObj.drivers[driverName].driver.node.baseValue = baseValue;
                }
            } else if (indexObj.driver && indexObj.driver.node.variable === driverName) {
                indexObj.driver.node.perturbType = pertType;
                indexObj.driver.node.baseValue = baseValue;
            }
        }
    }
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getObjectsByIndexType = function (objectType,
                                                                                             indexObj,
                                                                                             indexType) {
    var objectsOfType = [];
    var indexingStr;
    if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE === objectType) {
        indexingStr = "arrays";
    } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE === objectType) {
        indexingStr = "drivers";
    } else {
        //unsupported objectType provided
        return objectsOfType;
    }
    var objectNames = null;
    if (indexObj[indexingStr] && Object.keys(indexObj[indexingStr]).length > 0) {
        objectNames = Object.keys(indexObj[indexingStr]);
        for (var objectIndex = 0; objectIndex < objectNames.length; ++objectIndex) {
            var objectName = objectNames[objectIndex];
            if (indexObj[indexingStr][objectName][indexType]) {
                objectsOfType.push(indexObj[indexingStr][objectName][indexType]);
            }
        }
    }
    return objectsOfType;
};

// index objects must belong to the same top level node (ie. arrays)
sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.orderByPrecedence = function (indexObjects, levelInLineage) {
    var orderedObjects = [];
    if (!indexObjects || 0 === indexObjects.length) {
        return orderedObjects;
    }
    // shortcut if only one exists
    if (1 === indexObjects.length) {
        orderedObjects.push(indexObjects[0]);
        return orderedObjects;
    }

    // use the first element to grab the top level node...
    var topLevelNode = indexObjects[0].lineage[levelInLineage - 1];
    _.each(topLevelNode.children, function (child) {
        var found = _.filter(indexObjects, function (indexObj) {
            return indexObj.lineage[levelInLineage].variable === child.variable;
        });
        if (found) {
            Array.prototype.push.apply(orderedObjects, found);
        }
    });

    return orderedObjects;
};

sas.risk.scenariomanager.utils.ScenarioTableHelper.prototype.getBaseValueFromContext= function (context, variableName) {
    var baseValue = '';
    if (variableName !== undefined && variableName !== null) {
        var rf = context.getObject(variableName);
        if (rf && rf.baseValue) {
            baseValue = rf.baseValue;
        }
    }
    return baseValue;
};

// Intentionally static
sas.risk.scenariomanager.utils.ScenarioTableHelper.getValidPerturbTypesForRiskSource = function(that, riskSourceName, riskSourceType, riskSourcePerturbations) {
    var allPerturbTypes = sas.risk.scenariomanager.model.PerturbationType.values();
    var perturbTypes = [];
    _.each(allPerturbTypes, function (pType) {
        switch (pType) {
            case sas.risk.scenariomanager.model.PerturbationType.ABS: // fall-through
            case sas.risk.scenariomanager.model.PerturbationType.REL:
                // Always valid.
                perturbTypes.push(pType);
                break;
            case sas.risk.scenariomanager.model.PerturbationType.VALUE:
                // Only valid for single risk factors.
                if (riskSourceType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) {
                    perturbTypes.push(pType);
                }
                break;
            case sas.risk.scenariomanager.model.PerturbationType.STD:
                var riskSource = that.context.getObject(riskSourceName, riskSourceType);
                // If single RF: valid if the risk factor has an associated volatility.
                // If array, driver, or group: valid if all members have an associated volatility.
                if (sas.risk.scenariomanager.utils.ScenarioTableHelper.checkForVolatility(that.context, riskSource)) {
                    perturbTypes.push(pType);
                }
                break;
            case sas.risk.scenariomanager.model.PerturbationType.ABS_TIME: // fall-through
            case sas.risk.scenariomanager.model.PerturbationType.REL_TIME:
                if ((that.scenario.periods.length > 1) || (that.scenario.periods.length === 1 && (that.scenario.periods[0].horizon !== -1 || !!that.scenario.periods[0].date))) {
                    perturbTypes.push(pType);
                }
                break;
            default:
                // Note that "NONE" is not listed above. It is only included in the ENUM for degenerate cases and shouldn't be available in the UI.
                break;
        }
    });
    return perturbTypes;
};

/*
 * returns a boolean indicating if empty. criteria is
 * expected !null && length!=0  .    Also , if the
 * list is empty, a single row is added to the list with
 * specific values.   This also checks for that entry.
 *
 * Intentionally static
 */
sas.risk.scenariomanager.utils.ScenarioTableHelper.isPeriodListEmpty = function (periodList) {
    var isEmpty = (!periodList || null === periodList || undefined === periodList || periodList.length === 0);

    if (isEmpty) {
        return true;
    }

    if (periodList.length > 1) {
        return false;
    }

    var row = periodList[0];
    if (!row) {
        return true;
    }

    var periodUnset = ( !row.periodType || null === row.periodType || undefined === row.periodType || row.periodType.length === 0 ||
                        row.periodType === sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE );
    var valueUnset = ( !row.horizon || null === row.horizon || undefined === row.horizon || row.horizon === -1);
    var dateUnset = ( !row.date || null === row.date || undefined === row.date || row.date.length === 0 );

    return (periodUnset && valueUnset && dateUnset);
};