jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sap.ui.model.type.Float");
jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require('sas.hc.m.BusyDialog');
jQuery.sap.require('sas.hc.m.TableSelectDialog');
jQuery.sap.require("sas.risk.common.utils.NavChecker");
jQuery.sap.require('sas.risk.scenariomanager.model.IntervalTypeConstants');
jQuery.sap.require("sas.risk.scenariomanager.model.PerturbationType");
jQuery.sap.require("sas.risk.scenariomanager.model.RiskVarConstants");
jQuery.sap.require("sas.risk.scenariomanager.model.ScenarioContext");
jQuery.sap.require('sas.risk.scenariomanager.EventBusConstants');
jQuery.sap.require('sas.risk.scenariomanager.ScenarioAPI');
jQuery.sap.require('sas.risk.scenariomanager.utils.ScenarioEditorUtil');
jQuery.sap.require('sas.risk.scenariomanager.utils.ScenarioPropertyController');
jQuery.sap.require('sas.risk.scenariomanager.ViewManager');
jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");
jQuery.sap.require("sas.risk.scenariomanager.utils.ScenarioTableHelper");
jQuery.sap.require('sas.risk.scenariomanager.widgets.ScenariosTable');

sap.ui.controller('sas.risk.scenariomanager.view.ScenarioEditor', {

    onInit: function () {
        this.router = sas.risk.scenariomanager.ViewManager.getRouter();
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: 'sas.risk.scenariomanager.nls.messagebundle' });
        view.setModel(i18N, 'i18N');
        this.rb = i18N.getResourceBundle();
        this._subscribeToSettingsChanged();

        this.navChecker = new sas.risk.common.utils.NavChecker(sas.config.getAppId(),
            this.rb.getText("scenarioEditor.dirty.question.txt"), this.router);
        this.navChecker.start($.proxy(view.destroy, view));
        this.horizonDirty = false;
        this._cellsInEditMode = 0;
        this.dirty = false;
        this.editable = false;
        this.scenarioOpen = false;
        this.cleaningUp = false;

        // Setup interval dropdown
        var intervalTypeUtil = new sas.risk.scenariomanager.utils.IntervalTypeUtil();
        view.typeSelect.bindItems("/", new sas.hc.ui.core.Item({ key: "{key}", text: "{text}" }));
        var intervals = intervalTypeUtil.getIntervalsAndDisplayText().sort(function (a, b) {
            return a.text.localeCompare(b.text);
        });
        view.typeSelect.setModel(new sas.risk.common.models.RiskJSONModel(intervals));

        this.cubesVisible = sas.risk.scenariomanager.ScenarioAPI.getCubesVisible();
    },

    /*
     * Cleanup the scenario editor prior to opening a new scenario in the existing editor view.
     **/
    cleanup: function () {
        this.cleaningUp = true;
        this.closeAllBusyDialogs();
        var view = this.getView();

        this.setOrUpdateHorizonModelData([]);
        this.horizonDirty = false;
        this._cellsInEditMode = 0;
        this.dirty = false;
        this.editable = false;

        if (view.lockedMessage.getVisible() === true) {
            view.perturbationPanel.getTop().setSize("0");
            view.perturbationPanel.getTop().setVisible(false);
            view.lockedMessage.setVisible(false);
        }

        // close any open plots
        this.showHidePlotSplitter(false);
        if (view.plots) {
            view.plots.getPlotChooser().destroyItems();
            view.plots.destroy();
            delete view.plots;
        }

        // We might be cleaning up (exiting) before the scenario table helper was created, so we need to check that it exists before attempting to destroy it.
        if (this.scenarioTableHelper) {
            this.scenarioTableHelper.destroy();
            delete this.scenarioTableHelper;
        }

        this.openCreateDialogOnClose = false;
        this.savedForLaterAppSwitchData = undefined;

        this.navChecker.setDirty(false);
        this.scenarioOpen = false;
    },

    onExit: function () {
        if (this.openCreateDialogOnClose === true) {
            var mgr = sas.risk.scenariomanager.ViewManager.getScenarioManagerView();
            mgr.getController().createScenarioWithAppSwitchData(this.savedForLaterAppSwitchData);
        }

        this.cleanup();
        this.navChecker.stop();
    },

    getRouter: function () {
        return this.router;
    },

    setupRiskSourcesModel: function () {
        var view = this.getView();
        var model = view.riskSourcesTree.getModel();
        var unassignedGroupName = this.rb.getText("riskVarTree.unassignedGroup.txt");
        var modelData;
        var contextData = this.getContext();
        var allowExpandAllRiskSources = true;
        if (contextData && contextData.riskFactorsByName) {
            // PERFORMANCE/STABILITY WORKAROUND S1358507: We don't want to allow expand all if there are lots of risk factors in the context
            allowExpandAllRiskSources = (2500 > Object.keys(contextData.riskFactorsByName).length);
        }
        view.expandAllRiskSources.setEnabled(allowExpandAllRiskSources);
        // To be safe, we're going to disable the expand all in the perturbation table at the same time as the risk sources tree.
        view.expandAll.setEnabled(allowExpandAllRiskSources);

        modelData = this.buildTreeNodes(contextData, unassignedGroupName);
        if (model) {
            model.setData(modelData);
        } else {
            model = new sas.risk.common.models.RiskJSONModel(modelData);
            view.riskSourcesTree.setModel(model);
            if (modelData.children && modelData.children.length === 1) {
                view.riskSourcesTree.expand(0);
            }
        }
    },

    loadScenariosForWorkGroup: function (scenario) {
        var that = this;
        var view = this.getView();
        // Load the existing set of scenarios (for plotting, perhaps for validation (the property controller currently
        // does this, perhaps it can use this copy instead)), linking up the baselines & setting display label
        sas.risk.scenariomanager.ScenarioAPI.getScenarios(scenario.workGroup).done(function (scenarios) {
            var scenModel = new sas.risk.common.models.RiskJSONModel(scenarios);
            var availableBaselines = [];
            var availableSources = [];

            _.each(scenarios, function (scen) {
                scen.displayLabel = (scen.scenarioVersion) ?
                                    that.rb.getText("scenario.nameAndVersionFormat.txt", [scen.label,
                                                                                          scen.scenarioVersion]) :
                                    scen.label;
                // no need for e13n; used only internally
                scen.viewName = 'scenario_' + scen.id;
                // link up the baselines
                if (scen.baselineId !== undefined && scen.baselineId !== null && scen.baselineId !== "0") {
                    scen.baseline = _.find(scenarios, function (b) {
                        return b.id === scen.baselineId;
                    });
                }

                if (scenario.id !== scen.id) {
                    // assemble the available sources
                    availableSources.push(scen);
                    // assemble the available baselines
                    if (scen.baselineId === "0" || scen.baselineId === undefined || scen.baselineId === null) {
                        availableBaselines.push(scen);
                    }
                }
            });
            view.setModel(scenModel, 'scenarioExtent');

            view.setModel(new sas.risk.common.models.RiskJSONModel(availableBaselines), 'availableBaselines');
            view.setModel(new sas.risk.common.models.RiskJSONModel(availableSources), 'availableSources');
        });
    },

    openScenario: function (scenario) {
        var ctrl = this;

        if (this.scenarioOpen) {
            this.cleanup();
            this.cleaningUp = false;
        }

        var removeDataSource = function (scen) {
            scen.links = _.reject(scen.links, function (link) {
                return link.rel && link.rel === "stressContext";
            });

            scen.dataSourceName = undefined;
        };

        // KLUDGE: manipulating the links this way is generally frowned upon (we're overriding one based on
        // runtime information not available in the database)
        var overrideDataSource = function (cube, scen) {
            sas.risk.scenariomanager.ScenarioAPI.getCubeSelf(cube).done(function (cubeSelf) {
                var dataSourceGetterLink = _.findWhere(cubeSelf.links, { rel: "stressContext" });
                if(dataSourceGetterLink) {
                    removeDataSource(scen);
                    scen.links.push(dataSourceGetterLink);
                    scen.dataSourceId = cube.id;
                    scen.dataSourceName = cube.name;
                } else {
                    cubeNotFound();
                }
                ctrl._loadContext(scen);
            }).fail(function (jqxhr) {
                cubeNotFound(jqxhr);
                ctrl._loadContext(scen);
            });
        };

        // For an existing scenario, load the current details and associated context from the midtier...
        // TODO: bypass loading the context if it's the same as the what may already be loaded?
        if (scenario.id) {
            ctrl.openBusyDialog("scenarioEditor.loadScenarioList.txt");
            sas.risk.scenariomanager.ScenarioAPI.getScenarioData(scenario).done(function (scenarioDetails) {
                ctrl.closeBusyDialog("scenarioEditor.loadScenarioList.txt");
                if (scenario.appSwitchReadOnly) {

                    // We only care about the app switcher read-only flag if the scenario isn't already read-only.  IF
                    // we're overriding this, remove all of the links that update the scenario
                    if (scenarioDetails.readOnly === false) {
                        scenarioDetails.links = _.reject(scenarioDetails.links, function (link) {
                            return link.rel && (link.rel === "update" || link.rel === "delete" || link.rel === "deleteCascade" || link.rel === "lock");
                        });
                    }
                }

                // Are we opening using a specific data source from appswitching?
                if (scenario.appSwitchDataSourceId !== undefined && scenario.appSwitchDataSourceParms !== undefined) {
                    // Grab the link information for the data source we want to use (the scenario doesn't current have it)
                    sas.risk.scenariomanager.ScenarioAPI.getCube(scenario.appSwitchDataSourceId, scenario.appSwitchDataSourceParms)
                       .done(function (cube) {
                           if (cube) {
                               overrideDataSource(cube, scenarioDetails);
                           } else {
                               cubeNotFound();
                               ctrl._loadContext(scenarioDetails);
                           }
                       })
                       .fail(function (jqxhr) {
                           cubeNotFound(jqxhr);
                           // load Context has it's own busy.
                           ctrl._loadContext(scenarioDetails);
                       });
                } else if (scenario.appSwitchDataSourceName !== undefined) {
                    if (scenario.appSwitchDataSourceName === null || scenario.appSwitchDataSourceName === "" || scenario.appSwitchDataSourceName === "Scenario Repository") {
                        removeDataSource(scenarioDetails);

                        // load Context has it's own busy.
                        ctrl._loadContext(scenarioDetails);
                    } else {
                        var workGroupsForCubes = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getWorkGroupsForCubes(scenario.workGroup);
                        // The data source name was specified by the application switcher, so use that instead of what was saved last.
                        sas.risk.scenariomanager.ScenarioAPI.getCubesRoot(workGroupsForCubes).done(function (response) {
                            var cubeWorkGroup, serviceName, relPath;
                            if (scenario.appSwitchDataSourceParms) {
                                if (scenario.appSwitchDataSourceParms.workGroup) {
                                    var appSwitchDataSourceWorkGroup = _.find(workGroupsForCubes, function (workGroup) {
                                        return workGroup.toLocaleUpperCase() === scenario.appSwitchDataSourceParms.workGroup.toLocaleUpperCase();
                                    });
                                    if (appSwitchDataSourceWorkGroup) {
                                        cubeWorkGroup = appSwitchDataSourceWorkGroup;
                                    }
                                }
                                serviceName = scenario.appSwitchDataSourceParms.application;
                                relPath = scenario.appSwitchDataSourceParms.relPath;
                            }
                            var cubeToUse = sas.risk.scenariomanager.utils.ScenarioEditorUtil.findCubeInFolders(response.items, cubeWorkGroup, serviceName, relPath, scenario.appSwitchDataSourceId, scenario.appSwitchDataSourceName);
                            if (cubeToUse) {
                                overrideDataSource(cubeToUse, scenarioDetails);
                            } else {
                                cubeNotFound();
                                // load Context has it's own busy.
                                ctrl._loadContext(scenarioDetails);
                            }
                        }).fail(function (jqxhr) {
                            cubeNotFound(jqxhr);
                            // load Context has it's own busy.
                            ctrl._loadContext(scenarioDetails);
                        });
                    }
                } else {
                    // load Context has it's own busy.
                    ctrl._loadContext(scenarioDetails);
                }
            }).fail(function (event) {
                ctrl.closeBusyDialog("scenarioEditor.loadScenarioList.txt");
                ctrl.onServerFailure(event);
            });
        } else if (!ctrl.getContext()) {
            // ... For a new scenario, load the default context (scenario.datasource is empty, which loads the repository
            // context here) if we don't already have a context loaded
            ctrl._loadContext(scenario);
        } else {
            // ... we know we have a context here, just reuse it for the new scenario
            ctrl.setScenario(scenario);
            ctrl.setupRiskSourcesModel();
        }

        var cubeNotFound = function () {
            var nameOrId = scenario.appSwitchDataSourceName ? scenario.appSwitchDataSourceName : scenario.appSwitchDataSourceId;
            sas.hc.m.MessageBox.warning(
                ctrl.rb.getText('scenarioEditor.appSwitchCubeNotFound.warning.txt',
                                [nameOrId.replace(/\.rskcdesc$/i, '')]),
                ctrl.rb.getText('scenarioEditor.cubeNotFound.warning.title'),
                [sas.hc.m.MessageBox.Action.OK]
            );
        };
    },

    _loadContext: function (scenario) {
        var ctrl = this;
        ctrl.openBusyDialog("scenarioEditor.loadCubeDetails.txt");
        sas.risk.scenariomanager.ScenarioAPI.getScenarioContext(scenario).done(function (context) {
            ctrl.setContext(context);
            ctrl.closeBusyDialog("scenarioEditor.loadCubeDetails.txt");
            ctrl.setScenario(scenario);

            var link = sas.risk.scenariomanager.ScenarioAPI.findOptionalLink(scenario.links, { rel: 'stressContext' });
            if ((!link || link === undefined) && (scenario && scenario.dataSourceName && scenario.dataSourceName.length > 0)) {
                sas.hc.m.MessageBox.warning(
                    ctrl.rb.getText('scenarioEditor.cubeNotFound.warning.txt',
                                    [scenario.dataSourceName.replace(/\.rskcdesc$/i, '')]),
                    ctrl.rb.getText('scenarioEditor.cubeNotFound.warning.title'),
                    [sas.hc.m.MessageBox.Action.OK]
                );
            }
            ctrl.setupRiskSourcesModel();
            ctrl._checkDateRangeMismatch(scenario, ctrl.getContext());

        }).fail(function (event) {
            ctrl.closeBusyDialog("scenarioEditor.loadCubeDetails.txt");
            ctrl.onServerFailure(event);
        });
    },

    setContext: function (context) {
        var view = this.getView();

        if (undefined !== context && null !== context) {
            this.context = new sas.risk.scenariomanager.model.ScenarioContext(context);
            view.setModel(new sas.risk.common.models.RiskJSONModel(JSON.parse(JSON.stringify(this.context))), 'context');
        }

        if (view.plots) {
            view.plots.context = this.context;
        }
    },

    getContext: function () {
        return this.context;
    },

    // NOTE: context has to be loaded before the scenario is set: parts of the configuration of the UI depend on
    // knowing the environment in which the scenario is defined.  A call to openScenario eventually ends up calling
    // this method, so if we've called this before openScenario (which is possible based on how the editor is
    // configured by its owner), then we'll call openScenario if necessary and loop back around.
    setScenario: function (scenario) {
        var deferred = $.Deferred();
        var ctrl = this;
        var context = ctrl.getContext();

        if (!scenario) {
            // TODO: Why wouldn't there be a scenario here? Is this a possible path or can it go away?
            this._doSetScenario(scenario, context);
            deferred.resolve();
            return deferred;
        }

        this.loadScenariosForWorkGroup(scenario);

        if (!scenario.hasOwnProperty('periods')) {
            ctrl.openScenario(scenario);
            deferred.resolve();
            return deferred;
        }

        // If the scenario has a baseline, load it; it is used to compute perturbation values
        var baselineLink = sas.risk.scenariomanager.ScenarioAPI.findOptionalLink(scenario.links, { rel: 'baseline' });
        if (!baselineLink) {
            this._doSetScenario(scenario, context);
            deferred.resolve();
            return deferred;
        }

        ctrl.openBusyDialog("scenarioEditor.loadBaseline.txt");
        sas.risk.scenariomanager.ScenarioAPI.getScenarioFromLink(baselineLink).done(function (baselineData) {
            ctrl.closeBusyDialog("scenarioEditor.loadBaseline.txt");
            scenario.baseline = baselineData;
            ctrl._doSetScenario(scenario, context);
            deferred.resolve();
            return deferred;
        }).fail(function (jqxhr) {
            ctrl.closeBusyDialog("scenarioEditor.loadBaseline.txt");
            ctrl.onServerFailure(jqxhr);
            ctrl._doSetScenario(scenario, context);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    _doSetScenario: function (scen, context) {
        this.setDirty(false);
        this.resolveHorizonDates(scen, context);
        this.setOrUpdateScenarioModelData(scen);
        this.scenarioOpen = true;
    },

    updateAvailableBaselinesAndSources: function (scen) {
        var view = this.getView();
        var scenariosModel = view.getModel('scenarioExtent');
        if (scenariosModel) {
            var scenarios = scenariosModel.getData();

            var availableBaselines = [];
            var availableSources = [];
            _.each(scenarios, function (scenario) {
                if (scen.id !== scenario.id) {
                    // assemble the available sources
                    availableSources.push(scenario);
                    // assemble the available baselines
                    if (scenario.baselineId === "0" || scenario.baselineId === undefined || scenario.baselineId === null) {
                        availableBaselines.push(scenario);
                    }
                }
            });
        }

        view.setModel(new sas.risk.common.models.RiskJSONModel(availableBaselines), 'availableBaselines');
        view.setModel(new sas.risk.common.models.RiskJSONModel(availableSources), 'availableSources');
    },

    resolveHorizonDates: function (scenario, context) {
        var ctrl = this;

        // Note; even for MtM scenarios, there's at least one period by this point
        if (scenario.periods[0].horizon > 0) {
            ctrl.openBusyDialog("scenarioEditor.resolveHorizons.txt");
            sas.risk.scenariomanager.ScenarioAPI.resolveHorizonDates(scenario, context)
               .done(function () {
                   ctrl.closeBusyDialog("scenarioEditor.resolveHorizons.txt");
               })
               .fail(function (event) {
                   ctrl.closeBusyDialog("scenarioEditor.resolveHorizons.txt");
                   ctrl.onServerFailure(event);
               });
        }
    },

    getScenario: function () {
        var view = this.getView();
        var scenarioModel = view.getModel('scenario');
        if (scenarioModel) {
            return scenarioModel.getData();
        }
        return null;
    },

    setOrUpdateScenarioModelData: function (scenario) {
        var view = this.getView();
        var model = view.getModel('scenario');
        if (undefined === scenario) {
            scenario = {
                name: '\u00A0',
                stresses: []
            };
        }

        // this.deletedIndex = [];
        this.setEditable(scenario);

        if (undefined === model) {
            model = new sas.risk.common.models.RiskJSONModel(scenario);
            view.setModel(model, 'scenario');
        } else {
            model.setData(scenario);
        }

        this.setConsumingScenarios(scenario.id);
        this.setOrUpdateHorizonPanel(scenario);
        this.refresh(scenario, this.getContext());
    },

    setOrUpdateHorizonPanel: function (scenario) {
        var scenToUse = scenario;
        var hasBaseline = false;
        if (scenario.baseline && scenario.baseline.id !== undefined && scenario.baseline.id !== null && scenario.baseline.id !== "0") {
            hasBaseline = true;
            scenToUse = scenario.baseline;
        }
        var view = this.getView();
        var periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(sas.risk.scenariomanager.utils.ScenarioEditorUtil.getPeriods(scenToUse));

        periods = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPeriods(periods, true);
        view.horizonTable.setFirstVisibleRow(0);
        this.setOrUpdateHorizonModelData(periods);
        view.horizonTable.getModel().refresh();

        var selectedPeriodType;
        // If period type is specified and the horizon isn't -1, then use it.
        if (scenToUse.periodType && scenToUse.periods[0].horizon !== -1) {
            selectedPeriodType = scenToUse.periodType;
        } else {
            selectedPeriodType = sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE;
        }
        view.typeSelect.setSelectedKey(selectedPeriodType);

        // this.onIntervalSelected(true);
        var IntervalTypeUtil = new sas.risk.scenariomanager.utils.IntervalTypeUtil();
        var isDateType = IntervalTypeUtil.isImplicit(selectedPeriodType);
        view.horizonDateEditColumn.setVisible(isDateType && !hasBaseline);
        view.horizonDateValueColumn.setVisible(isDateType && hasBaseline);
        view.horizonInputEditColumn.setVisible(!isDateType && !hasBaseline);
        view.horizonInputValueColumn.setVisible(!isDateType && hasBaseline);

        view.horizonApplyButton.setEnabled(false);
        view.horizonCancelButton.setEnabled(false);

        if (view.typeSelect.getKeys().indexOf(scenToUse.periodType) === -1) {
            var typeSelectModel = view.typeSelect.getModel();
            if (!typeSelectModel) {
                typeSelectModel = new sas.risk.common.models.RiskJSONModel([]);
                view.typeSelect.setModel(typeSelectModel);
            }
            var intervals = typeSelectModel.getData();
            intervals.push({ key: scenToUse.periodType, text: scenToUse.periodType });
            typeSelectModel.setData(intervals);
        }

        // Enable/disable and show/hide various horizon editor components based on baseline/readonly/editable status
        var horizonsEditable = ((parseInt((scenario.baselineId ? scenario.baselineId :
                                           0), 10) === 0) && scenario.readOnly === false && this.isEditable());
        view.typeSelect.setEnabled(horizonsEditable);
        view.addHorizonButton.setVisible(horizonsEditable);
        view.deleteHorizonButton.setVisible(horizonsEditable);
        view.horizonPanel.getBottom().setVisible(horizonsEditable);
        if (horizonsEditable) {
            view.horizonTable.setSelectionMode(sas.hc.ui.table.SelectionMode.MultiToggle);
        } else {
            view.horizonTable.setSelectionMode(sas.hc.ui.table.SelectionMode.None);
        }

        // Apply readonly-specific settings
        if (scenario.readOnly === true || this.isEditable() === false) {
            view.perturbationPanel.getTop().setSize("3.5rem");
            view.perturbationPanel.getTop().setVisible(true);
            // Read only on the scenario actually means LOCKED, not editable is a generic "read only" status.
            if (scenario.readOnly === true) {
                view.lockedMessage.setText(this.rb.getText("scenarioEditor.lockedMessage.txt"));
            } else {
                view.lockedMessage.setText(this.rb.getText("scenarioEditor.readOnlyMessage.txt"));
            }
            view.lockedMessage.setVisible(true);
        }

    },

    togglePropertyPane: function (event) {
        var view = this.getView();
        if (view.layout.getPropertyPaneVisible()) {
            this.expandPropertyPane();
        } else {
            this.collapsePropertyPane();
        }
        if (!this.cleaningUp) {
            // Need to inform the wide table to refresh, otherwise we might not display all of the columns correctly in the
            // newly available (or no longer available) space
            view.perturbationTable.refreshScrollableContent();
        }
    },

    collapsePropertyPane: function () {
        var view = this.getView();
        var contentAreaFull = view.$().find(".sasPropertyPaneLayout_contentArea");
        contentAreaFull.addClass("sasPropertyPaneLayoutContent_leftPropertyPaneHidden");
        contentAreaFull.removeClass("sasPropertyPaneLayoutContent_leftPropertyPaneVisible");
    },

    expandPropertyPane: function () {
        var view = this.getView();
        var contentAreaWithPane = view.$().find(".sasPropertyPaneLayout_contentArea");
        contentAreaWithPane.addClass("sasPropertyPaneLayoutContent_leftPropertyPaneVisible");
        contentAreaWithPane.removeClass("sasPropertyPaneLayoutContent_leftPropertyPaneHidden");

    },

    _findOptionalLink: function (links, linkProps) {
        return sas.risk.scenariomanager.ScenarioAPI.findOptionalLink(links, linkProps);
    },

    refresh: function (scenario, context) {
        var that = this;
        var view = this.getView();
        var i18NModel = view.getModel('i18N');

        // If we have what we need then proceed with creating new model for tree table.
        if ((scenario !== null) &&
            (context !== undefined && context !== null) &&
            (i18NModel !== undefined)) {

            if (undefined === this.scenarioTableHelper) {
                this.scenarioTableHelper = new sas.risk.scenariomanager.utils.ScenarioTableHelper(scenario, this.getContext());
            } else {
                this.scenarioTableHelper.update(scenario, this.getContext());
            }
            this.setOrUpdatePerturbationTableModelData(this.scenarioTableHelper.getRootNode());
        }
    },

    setOrUpdatePerturbationTableModelData: function (rootNode) {
        //  this.openBusyDialog("scenarioEditor.updatingPerturbationData.txt");
        var view = this.getView();
        var model = view.perturbationTable.getModel();
        if (undefined === rootNode) {
            rootNode = { children: [], columns: [] };
        }

        if (undefined === model) {
            model = new sas.risk.common.models.RiskJSONModel(rootNode);
            view.perturbationTable.setModel(model);
        } else {
            model.setData(rootNode);
        }

        // After model is set, make sure action buttons are updated */
        this.toggleScenarioButtons();
        //  this.closeBusyDialog("scenarioEditor.updatingPerturbationData.txt");
    },

    getFlattenedStresses: function (includeHiddenStresses) {
        var view = this.getView();
        var stresses = [];
        var rootModel = view.perturbationTable.getModel();
        if (!rootModel) {
            return stresses;
        }

        var rootData = rootModel.getData().children;
        for (var i = 0; i < rootData.length; ++i) {
            Array.prototype.push.apply(stresses, rootData[i].children);
        }

        if (true === includeHiddenStresses && this.scenarioTableHelper.hiddenStresses !== undefined && this.scenarioTableHelper.hiddenStresses.length > 0) {
            Array.prototype.push.apply(stresses, this.scenarioTableHelper.hiddenStresses);
        }

        return stresses;
    },

    setDirty: function (dirty) {
        // A non-editable scenario can never be dirty...
        if (this.isEditable()) {
            if (dirty === undefined) {
                dirty = true;
            }
            if (this.navChecker) {
                this.navChecker.setDirty(dirty || this.horizonDirty);
            }
            this.dirty = dirty;
            this.toggleScenarioButtons();
            this.togglePerturbationTableButtons();
        }
    },

    isDirty: function () {
        return this.dirty;
    },

    isNewScenario: function () {
        var scenario = this.getScenario();
        return !scenario.id;
    },

    onDescriptionChanged: function (event) {
        this.setDirty(true);
    },

    onBrowseForScenario: function (title, scenarios, currentScenario, callBack, filterNoBaseline) {

        var scenariosPanel = new sas.risk.scenariomanager.widgets.ScenariosTable({
            filterNoBaseline: filterNoBaseline
        });
        var view = this.getView();
        var ctrl = this;

        var workGroup = new sas.hc.m.Text({
            text: {
                path: '/workGroup',
                formatter: function (workGroupName) {
                    if (!workGroupName) {
                        return "";
                    }
                    // The work groups are loaded by the shell, so we don't need deferred
                    var workGroups = sas.risk.scenariomanager.ScenarioAPI.getWorkGroups();
                    // Find the work group by its name
                    var workGroup = _.find(workGroups, function (workGroup) {
                        return workGroupName.toLocaleUpperCase() === workGroup.name.toLocaleUpperCase();
                    });
                    if (!workGroup) {
                        return workGroupName;
                    }
                    // If the work group has a display name, use it.
                    return workGroup.displayName ? workGroup.displayName : workGroup.name;
                }
            }
        });
        var workGroupLabel = new sas.hc.m.Label({
            text: this.rb.getText("scenarioSelectDialog.workGroupField.label.txt"),
            required: false,
            labelFor: workGroup
        });

        var form = new sas.hc.ui.layout.form.Form({
            layout: new sas.hc.ui.layout.form.ResponsiveGridLayout({
                columnsL: 1,
                labelSpanS: 12,
                labelSpanM: 12,
                labelSpanL: 12
            }),
            formContainers: [
                new sas.hc.ui.layout.form.FormContainer({
                    formElements: [
                        new sas.hc.ui.layout.form.FormElement({
                            label: workGroupLabel,
                            fields: [workGroup]
                        }),
                        new sas.hc.ui.layout.form.FormElement({
                            fields: [scenariosPanel]
                        })
                    ]
                })
            ]
        }).addStyleClass("stressExtraLargeForm");

        var model = new sas.risk.common.models.RiskJSONModel();
        model.setProperty("/workGroup", ctrl.getScenario().workGroup);
        model.setProperty("/scenarios", scenarios);
        model.setProperty("/selectedDataSource", view.getModel("context").getData().contextServerObject);

        var dialog = new sas.hc.m.Dialog({
            id: "sas.risk.scenariomanager.view.ScenarioEditorSelectDialog",
            title: this.rb.getText(title),
            content: form,
            afterClose: function () {
                dialog.destroy();
            },
            afterOpen: function () {
                if (currentScenario) {
                    scenariosPanel.setSelectedScenario(currentScenario);
                }
            }
        });

        var okayButton = new sas.hc.m.Button({
            id: "sas.risk.scenariomanager.view.ScenarioSelectDialog.okButton",
            text: this.rb.getText("button.ok.action.label"),
            press: function () {
                var selectedContextData = scenariosPanel.getSelectedScenario();
                callBack(ctrl, selectedContextData);
            },
            enabled: true
        });
        okayButton.addStyleClass("sasPrimaryButton");
        dialog.setBeginButton(okayButton);

        var cancelButton = new sas.hc.m.Button({
            text: this.rb.getText("button.cancel.action.label"),
            press: function () {
                dialog.close();
            }
        });

        dialog.setEndButton(cancelButton);
        dialog.setModel(model);
        dialog.open();
    },

    onSourceChanged: function (ctrl, selectedSource) {
        var scenario = ctrl.getScenario();
        scenario.sourceId = (selectedSource) ? selectedSource.id : undefined;
        ctrl.getView().getModel("scenario").refresh();
        var scenarioSelectDialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ScenarioEditorSelectDialog');
        if (scenarioSelectDialog) {
            scenarioSelectDialog.close();
        }
    },

    onBaselineChanged: function (ctrl, baselineScenarioSummary) {

        var that = ctrl;
        var scenario = that.getScenario();

        if (!baselineScenarioSummary) {
            // If the current scenario has no stresses or there is no baseline selected, then we do not need to display a prompt.
            // We can immediately apply the baseline changes.
            that._applyBaselineChange(scenario, baselineScenarioSummary);
            var scenarioSelectDialog = sap.ui.getCore()
                                          .byId('sas.risk.scenariomanager.view.ScenarioEditorSelectDialog');
            if (scenarioSelectDialog) {
                scenarioSelectDialog.close();
            }
        } else {

            that.openBusyDialog("scenarioEditor.loadScenarioList.txt");
            sas.risk.scenariomanager.ScenarioAPI.getScenarioData(baselineScenarioSummary)
               .done(function (baselineScenario) {
                   that.closeBusyDialog("scenarioEditor.loadScenarioList.txt");

                   if (scenario.stresses.length === 0) {
                       // If the current scenario has no stresses, then we do not need to display a prompt.
                       // Immediately apply the baseline changes.
                       that._applyBaselineChange(scenario, baselineScenario);
                   } else {
                       var warningMessage = "";
                       if (scenario.periods.length === baselineScenario.periods.length) {
                           if (0 !== that.compareHorizons(scenario.periods, baselineScenario.periods)) {
                               // The horizons are not the same, need to warn the user.
                               warningMessage = that.rb.getText("scenarioEditor.changeBaseline.horizonsChanging.txt");
                           }
                       } else {
                           if (0 > that.compareHorizons(scenario.periods, baselineScenario.periods)) {
                               // Horizons will be removed, but none will be added.
                               warningMessage = that.rb.getText("scenarioEditor.changeBaseline.horizonsChangingWithRemoval.txt");
                           } else {
                               warningMessage = that.rb.getText("scenarioEditor.changeBaseline.horizonsChanging.txt");
                           }
                       }

                       if ("" === warningMessage && scenario.dataSourceName !== baselineScenario.dataSourceName) {
                           warningMessage = that.rb.getText("scenarioEditor.changeBaseline.dataSourceMismatch.txt");
                       }

                       if ("" === warningMessage) {
                           var scenarioSelectDialog = sap.ui.getCore()
                                                         .byId('sas.risk.scenariomanager.view.ScenarioEditorSelectDialog');
                           if (scenarioSelectDialog) {
                               scenarioSelectDialog.close();
                           }
                           that._applyBaselineChange(scenario, baselineScenario);
                       } else {
                           // if the scenario isn't empty, need to prompt user about (potentially) trashing their data...
                           // * is the cube for the scenarios the same?
                           // * are the horizons the same?
                           var actions = [sas.hc.m.MessageBox.Action.OK, sas.hc.m.MessageBox.Action.CANCEL];
                           sas.hc.m.MessageBox.information(warningMessage, that.rb.getText("scenarioEditor.changeBaseline.warningTitle.txt"), actions, null, function (btn) {
                               if (btn === sas.hc.m.MessageBox.Action.OK) {
                                   var scenarioSelectDialog = sap.ui.getCore()
                                                                 .byId('sas.risk.scenariomanager.view.ScenarioEditorSelectDialog');
                                   if (scenarioSelectDialog) {
                                       scenarioSelectDialog.close();
                                   }
                                   that._applyBaselineChange(scenario, baselineScenario);
                               }
                           });
                       }
                   }
               }).fail(function (event) {
                that.closeBusyDialog("scenarioEditor.loadScenarioList.txt");
                that.onServerFailure(event);
            });
        }
    },

    _applyBaselineChange: function (scenario, baselineScenario) {
        if (baselineScenario) {
            scenario.baselineId = baselineScenario.id;
            scenario.baseline = baselineScenario;
        } else {
            scenario.baselineId = "0";
            scenario.baseline = undefined;
        }
        this.setOrUpdateHorizonPanel(scenario);
        this.onApplyHorizons();
        this.setDirty(true);
        this.getView().getModel("scenario").refresh();
    },

    handlePerturbationTypeChanged: function (event) {
        this.setDirty(true);
        var perturbationTypeInput = event.getSource();
        var updatedRiskFactor = this.getView().perturbationTable.getModel()
                                    .getProperty(perturbationTypeInput.getBindingContext().getPath());
        // re-calculate the change value based on the new change type, base value, and final value. Only applies to risk factors.
        if (updatedRiskFactor.baseValue !== undefined && updatedRiskFactor.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE) {
            var perturbType = sas.risk.scenariomanager.model.PerturbationType.byCode(updatedRiskFactor.perturbType);
            for (var index = 0; index < updatedRiskFactor.perturbations.length; ++index) {
                // calc new change value from the final value
                updatedRiskFactor.perturbations[index].value = perturbType.computeChange(this.scenarioTableHelper.context,
                                                                                         this.scenarioTableHelper.scenario,
                                                                                         updatedRiskFactor,
                                                                                         updatedRiskFactor.variable,
                                                                                         index,
                                                                                         updatedRiskFactor.perturbations[index].finalValue);
            }
        }

        this.scenarioTableHelper.pullNewValues(updatedRiskFactor, -1);
        this.getView().perturbationTable.getModel().refresh();
    },

    setEditable: function (scenario) {
        var view = this.getView();

        var hasUpdateLink = !!this._findOptionalLink(scenario.links, { rel: 'update' });
        this.editable = scenario !== undefined && scenario !== null && scenario.readOnly === false && (!scenario.id || hasUpdateLink);
        view.repositoryCheckBox.setEnabled(this.cubesVisible === true);
        view.dataSourceField.setEnabled(this.cubesVisible === true && !view.repositoryCheckBox.getSelected());

        if (view.plots) {
            view.plots.editable = this.editable;
        }
        if (view.perturbationTable) {
            view.perturbationTable.setEditable(this.editable);
        }
    },

    isEditable: function () {
        return this.editable;
    },

    setConsumingScenarios: function (scenarioId) {
        var that = this;
        var view = this.getView();
        view.setModel(new sas.risk.common.models.RiskJSONModel([]), 'consumingScenarios');

        // Don't check for baselines for new (0) scenarios
        if (scenarioId > 0) {
            sas.risk.scenariomanager.ScenarioAPI.getScenariosForBaseline(scenarioId)
               .done(function (consumingScenarios) {
                   var consumingScenariosModel = view.getModel('consumingScenarios');
                   consumingScenariosModel.setData(consumingScenarios);
               }).fail(function (jqxhr) {
                that.onServerFailure(jqxhr);
            });
        }

    },

    getSelectedRiskFactors: function () {
        var that = this;
        var view = this.getView();
        var table = view.perturbationTable;
        var indices = table.getSelectedIndices();

        var items = [];
        _.each(indices, function (selectedIdx) {
            var riskFactor = that.getRiskSourceFromPerturbationTable(selectedIdx);
            if (null !== riskFactor && "topLevel" !== riskFactor.rowType) {
                items.push(riskFactor);
            }
        });

        return items;
    },

    getRiskSourceFromPerturbationTable: function (index) {
        var view = this.getView();

        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            var selections = view.perturbationTable.getSelectedIndices();
            if (selections !== null && selections.length > 0) {
                index = selections[0];
            } else {
                index = -1;
            }
        }

        var riskSource = null;
        if (index > -1) {
            var context = view.perturbationTable.getContextByIndex(index);
            if (context) {
                riskSource = context.getObject();
            }
        }
        return riskSource;
    },

    togglePerturbationTableButtons: function () {
        var view = this.getView();
        var selectedIndices = view.perturbationTable.getSelectedIndices();
        var selectedRiskFactors = this.getSelectedRiskFactors();
        var readOnlySelected = false;
        // Iterate through the selected risk factors
        for (var index = 0; index < selectedRiskFactors.length; ++index) {
            // Check for the read-only flag
            if (selectedRiskFactors[index].readOnly) {
                // In some cases if any selected risk factor is read-only, then we want to disable the toolbar button entirely.
                readOnlySelected = true;
                break;
            }
        }
        // Scenario must be editable, at least one risk factor must be selected (check against the length of the selected
        // indices to ensure no top level nodes are also selected), and none of the selections can be read-only.
        var editableScenarioRiskFactorSelected = (this.isEditable()) &&
                                                 (0 < selectedRiskFactors.length) &&
                                                 (selectedIndices.length === selectedRiskFactors.length) &&
                                                 (!readOnlySelected);
        // Scenario doesn't need to be editable, nor does the risk factor (child risk factors that are read-only can be plotted).
        // However, not all risk factors are plottable
        var ctxt = this.getContext();
        view.plotRiskFactorsButton.setEnabled((0 < selectedRiskFactors.length) &&
                                              (selectedIndices.length === selectedRiskFactors.length) &&
                                              _.every(selectedRiskFactors, function (rf) {
                                                  return rf.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE ||
                                                         (rf.rowType === sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE &&
                                                          _.every(rf.children, function (child) {
                                                              var childName = child.variable;
                                                              var childRF = ctxt.getObject(childName);
                                                              // To be plottable, array members must all have a maturity value; 0 allows the curve
                                                              // to include an MtM-like value for comparison purposes (the curve itself has no base
                                                              // value, and the individual members each have a base value of their own)
                                                              return childRF && (!isNaN(childRF.maturity) || childRF.maturity === 0);
                                                          }));
                                              }));

        view.deleteRiskFactorsButton.setEnabled(editableScenarioRiskFactorSelected);
        view.moveUpButton.setEnabled(editableScenarioRiskFactorSelected && 1 === selectedRiskFactors.length); // TODO: Can't move when top row selected
        view.moveDownButton.setEnabled(editableScenarioRiskFactorSelected && 1 === selectedRiskFactors.length); // TODO: Can't move when bottom row selected

    },

    toggleScenarioButtons: function () {
        var view = this.getView();
        var editable = this.isEditable();
        var dirty = this.isDirty();
        var empty = view.perturbationTable.getModel().getData().children.length === 0;
        var hasHiddenStresses = this.scenarioTableHelper.hiddenStresses.length > 0;
        view.saveButton.setEnabled(true === editable && (false === empty || true === hasHiddenStresses) && true === dirty);
        view.saveAsButton.setEnabled((false === empty || true === hasHiddenStresses) && false === this.isNewScenario());
        view.runButton.setEnabled(false === empty && false === dirty && true === this.cubesVisible);
    },

    onRunScenario: function () {
        var scenario = this.getScenario();
        if (!this.isDirty()) {
            var appUrl = sas.applicationSwitcher.getApplicationUrl("HPRiskLogon");
            sas.applicationSwitcher.gotoApp("HPRiskLogon", appUrl, true);

            var dsType = (scenario.dataSourceType == 'S') ? 'stress' :
                         (scenario.dataSourceType == 'P') ? 'price' : '';

            // SOPHIE TODO: Need the appid for UI5 HPRisk (they currently don't have one) - or will it be the same?
            sas.applicationSwitcher.sendMessageToApp("HPRiskLogon", "RunScenario", {
                scenarioId: scenario.id,
                scenarioWorkGroup: scenario.workGroup,
                scenarioName: scenario.name,
                scenarioVersion: scenario.scenarioVersion,
                dataSourceId: scenario.dataSourceId,
                dataSourceType: dsType,
                dataSourceParms: sas.risk.scenariomanager.ScenarioAPI.encodeArgsAsObject(scenario.dataSourceArgs)
            });
        } else {
            sas.hc.m.MessageBox.error(this.rb.getText('scenarioEditor.runDisabledError.txt'), this.rb.getText('error.window.title'));
        }
    },

    onBreadCrumbClick: function () {
        sas.risk.scenariomanager.ViewManager.loadScenarioManager();
    },

    onSaveScenario: function () {
        var ctrl = this;
        var scenario = this.getScenario();
        scenario.stresses = this.getFlattenedStresses(true);
        if (scenario.id) {
            this._doPutScenario('update', 'scenarioEditor.saveScenario.txt');
        } else {
            this._doCreateScenario(scenario);
        }
    },

    onSaveAs: function () {
        var ctrl = this;
        var scenario = this.getScenario();
        scenario.stresses = this.getFlattenedStresses(true);

        var fragment = 'sas.risk.scenariomanager.view.SaveAsDialog';
        var title = ctrl.rb.getText('saveAsDialog.window.title');
        var subCtrl = new sas.risk.scenariomanager.utils.ScenarioPropertyController(fragment, title);
        subCtrl.setOKHandler(function () {
            ctrl.openBusyDialog('scenarioEditor.saveScenario.txt');
            var scenarioToSave = subCtrl.getModel();

            // TODO: can this call _doCreateScenario instead????
            // TODO: make sure either label or version has changed -- if not, this will be a save of the original
            sas.risk.scenariomanager.ScenarioAPI.saveScenarios([scenarioToSave]).done(function (saved) {
                sas.applicationSwitcher.broadcastMessage("ScenariosUpdated", []);
                sas.risk.scenariomanager.ScenarioAPI.getScenarioData(saved.items[0]).done(function (newScenario) {
                    ctrl.setScenario(newScenario);
                    ctrl.setDirty(false);
                    ctrl.closeBusyDialog('scenarioEditor.saveScenario.txt');
                    sas.eventBus.publish(sas.risk.scenariomanager.EventBusConstants.CHANNEL_NAME,
                                         sas.risk.scenariomanager.EventBusConstants.EXISTING_SCENARIO_MANAGER_OPENED,
                                         { hideBusy: true });
                }).fail(function (event) {
                    ctrl.closeBusyDialog('scenarioEditor.saveScenario.txt');
                    ctrl.onServerFailure(event);
                });
            }).fail(function (event) {
                ctrl.closeBusyDialog('scenarioEditor.saveScenario.txt');
                ctrl.onServerFailure(event);
            });
        });

        subCtrl.cloneScenario(ctrl, scenario);
    },

    _doCreateScenario: function (scenario) {
        var ctrl = this;

        ctrl.openBusyDialog('scenarioEditor.saveScenario.txt');
        sas.risk.scenariomanager.ScenarioAPI.createScenario(scenario).done(function (saved) {
            sas.applicationSwitcher.broadcastMessage("ScenariosUpdated", []);
            ctrl.setScenario(saved);
            sas.eventBus.publish(sas.risk.scenariomanager.EventBusConstants.CHANNEL_NAME,
                                 sas.risk.scenariomanager.EventBusConstants.EXISTING_SCENARIO_MANAGER_OPENED,
                                 { hideBusy: true });
            ctrl.setDirty(false);
            ctrl.closeBusyDialog('scenarioEditor.saveScenario.txt');
        }).fail(function (event) {
            ctrl.closeBusyDialog('scenarioEditor.saveScenario.txt');
            ctrl.onServerFailure(event);
        });
    },

    _doPutScenario: function (rel, busyKey) {
        var ctrl = this;
        var scenario = this.getScenario();

        ctrl.openBusyDialog(busyKey);
        sas.risk.scenariomanager.ScenarioAPI.putScenario(scenario, rel).done(function (scenarioDetails) {
            var dsType = (scenarioDetails.dataSourceType === 'S') ? 'stress' :
                (scenarioDetails.dataSourceType === 'P') ? 'price' : '';
            sas.applicationSwitcher.broadcastMessage("ScenariosUpdated", [{
                scenarioId: scenarioDetails.id,
                scenarioWorkGroup: scenarioDetails.workGroup,
                dataSourceId: scenarioDetails.dataSourceId,
                dataSourceType: dsType,
                dataSourceParms: sas.risk.scenariomanager.ScenarioAPI.encodeArgsAsObject(scenario.dataSourceArgs),
                scenarioName: scenarioDetails.name,
                scenarioVersion: scenarioDetails.scenarioVersion,
                action: "UPDATE"
            }]);
            ctrl.setScenario(scenarioDetails);
            ctrl.setDirty(false);
            ctrl.closeBusyDialog(busyKey);
            sas.eventBus.publish(sas.risk.scenariomanager.EventBusConstants.CHANNEL_NAME,
                                 sas.risk.scenariomanager.EventBusConstants.EXISTING_SCENARIO_MANAGER_OPENED,
                                 { hideBusy: true });
        }).fail(function (event) {
            ctrl.closeBusyDialog(busyKey);
            ctrl.onServerFailure(event);
        });
    },

    onScenarioFinalValues: function () {
        var scenario = this.getScenario();
        var context = this.getContext();
        if (scenario) {
            var finalValuesView = sas.risk.scenariomanager.ViewManager.loadScenarioFinalValues(scenario, context, this.scenarioTableHelper);
            this.finalValuesDialog = new sas.hc.m.Dialog({
                content: finalValuesView,
                stretch: true,
                horizontalScrolling: false,
                verticalScrolling: false,
                showHeader: false
            }).addStyleClass("fullScreenDialogShowingBanner");
            this.finalValuesDialog.open();
            return this.finalValuesDialog;
        }
    },

    closeFinalValues: function () {
        this.finalValuesDialog.close();
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Perturbation Table Toolbar
    //*****************************************************************************************************
    //*****************************************************************************************************

    onDeleteRiskFactors: function () {
        var that = this;
        var view = this.getView();
        var selectedRiskFactors = this.getSelectedRiskFactors();
        var riskSourcesTreeData = view.perturbationTable.getModel().getData();

        _.each(selectedRiskFactors, function (selectedRiskFactor) {
            if (selectedRiskFactor.parent.rowType === "topLevel") {
                var oldIndex = _.indexOf(selectedRiskFactor.parent.children, selectedRiskFactor);
                // remove the risk factor from it's parent in the table, which also removes it from the scenario.
                selectedRiskFactor.parent.children.splice(oldIndex, 1);

                var rowType = selectedRiskFactor.rowType || sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE;
                that.scenarioTableHelper.adjustIndex(false, riskSourcesTreeData, rowType, selectedRiskFactor.variable, null, null);

                var insertIndex;
                if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE === selectedRiskFactor.rowType) {
                    insertIndex = that.scenarioTableHelper.getRiskFactorRootIndex();
                } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE === selectedRiskFactor.rowType) {
                    insertIndex = that.scenarioTableHelper.getArrayRootIndex();
                } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE === selectedRiskFactor.rowType) {
                    insertIndex = that.scenarioTableHelper.getGroupRootIndex();
                } else if (sas.risk.scenariomanager.model.RiskVarTypeConstants.DRIVER_OBJTYPE === selectedRiskFactor.rowType) {
                    insertIndex = that.scenarioTableHelper.getDriverRootIndex();
                }

                // Check if the delete removed the last child from the parent. If so, we need to delete the parent from the table.
                if (selectedRiskFactor.parent.children.length === 0) {
                    // The empty parent node should be removed from the tree.
                    riskSourcesTreeData.children.splice(insertIndex, 1);
                    // The stored root indices must also be adjusted.
                    that.scenarioTableHelper.shiftStoredRootIndices(false, selectedRiskFactor.rowType);
                }

            }
        });

        view.perturbationTable.clearSelection();
        if (view.plots) {
            view.plots.onRiskSourcesAddedRemoved(selectedRiskFactors);
        }

        var scenario = this.getScenario();
        scenario.stresses = this.getFlattenedStresses(false);

        this.setDirty(true);
        view.perturbationTable.getModel().refresh();
        view.perturbationTable.getBinding("rows").refresh();
    },

    onMoveRiskFactorUp: function (event) {
        var view = this.getView();
        var selectedRiskFactors = this.getSelectedRiskFactors();
        if (1 === selectedRiskFactors.length) {
            var selectedRiskFactor = selectedRiskFactors[0];
            var siblings = selectedRiskFactor.parent.children;
            var riskSourcesTreeData = view.perturbationTable.getModel().getData();
            var oldIndex = _.indexOf(siblings, selectedRiskFactor);
            if (oldIndex > 0) {
                this.moveRiskFactor(selectedRiskFactor, siblings, riskSourcesTreeData, oldIndex - 1, oldIndex);
                var selectedTableIndex = view.perturbationTable.getSelectedIndices()[0];
                view.perturbationTable.clearSelection();
                // SOPHIE TODO: not quite... because if we're moving elements with children and they're expanded, then this won't be accurate... we want to move up/down one risk factor of the same level... not just one row.
                view.perturbationTable.setSelectedIndex(selectedTableIndex - 1);
            }
        }
    },

    onMoveRiskFactorDown: function (event) {
        var view = this.getView();
        var selectedRiskFactors = this.getSelectedRiskFactors();
        if (1 === selectedRiskFactors.length) {
            var selectedRiskFactor = selectedRiskFactors[0];
            var siblings = selectedRiskFactor.parent.children;
            var riskSourcesTreeData = view.perturbationTable.getModel().getData();
            var oldIndex = _.indexOf(siblings, selectedRiskFactor);
            if (oldIndex < siblings.length - 1) {
                this.moveRiskFactor(selectedRiskFactor, siblings, riskSourcesTreeData, oldIndex + 1, oldIndex);
                var selectedTableIndex = view.perturbationTable.getSelectedIndices()[0];
                view.perturbationTable.clearSelection();
                // SOPHIE TODO: not quite... because if we're moving elements with children and they're expanded, then this won't be accurate... we want to move up/down one risk factor of the same level... not just one row.
                view.perturbationTable.setSelectedIndex(selectedTableIndex + 1);
            }
        }
    },

    moveRiskFactor: function (riskSource, siblings, riskSourcesTreeData, newIndex, oldIndex) {
        if (newIndex !== oldIndex) {
            // remove the source element from its original location
            siblings.splice(oldIndex, 1);
            // add the source element to its new location
            siblings.splice(newIndex, 0, riskSource);
            this.setDirty(true);
            this.getView().perturbationTable.getModel().refresh();
        }
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Horizon Table Event Handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    horizonTableRowChangeHandler: function (oEvent) {
        var view = this.getView();
        if (oEvent) {
            view.deleteHorizonButton.setEnabled(oEvent.getSource().getSelectedIndices().length !== 0);
        }
    },

    onEditInputColumn: function (event) {

        var input = event.getSource();
        var view = this.getView();
        var data = view.horizonTable.getModel().getData();

        event.preventDefault();

        input.setValue(input.getValue().replace(/\D*/, ""));

        var value = parseInt(input.getValue(), 10);

        if (value && value > 0 && $.grep(data, function (e) {
                // Use jQuery grep to verify no other horizons have this value
                return e.horizon === value;
            }).length === 0) {
            input.setValueState(sap.ui.core.ValueState.None);
            input.setValue(value);
        } else {
            input.setValueState(sap.ui.core.ValueState.Error);
        }

        this.checkApplyButton();

    },

    onEditDateColumn: function (event) {
        var validDate = event.getParameter("valid");
        var datePicker = event.getSource();
        var view = this.getView();

        if (validDate) {
            datePicker.setValueState(sap.ui.core.ValueState.None);
        } else {
            datePicker.setValueState(sap.ui.core.ValueState.Error);
        }
        this.checkApplyButton();
    },

    checkApplyButton: function () {
        var view = this.getView();
        var model = view.horizonTable.getModel();
        var data = model.getData();
        var state = 0;

        var intervals = [];
        var dates = [];
        var utcDate;

        var idx = 0;
        _.each(data, function (item) {
            if (item.date) {
                utcDate = item.date.toLocaleDateString(undefined, { month: "short", day: "numeric", year: "numeric" });
            }

            // Is it a date horizon? If so, does it already exist?
            if (item.horizon === -1 && item.date && dates[utcDate]) {
                state = utcDate;
            }

            if (item.horizon > -1 && (intervals[item.horizon] || !item.horizon || item.horizon < 1)) {
                state = item.horizon;
            }

            intervals[item.horizon] = true;
            dates[utcDate] = true;
            ++idx;
        });

        this.horizonDirty = true;
        this.setDirty(this.dirty);

        view.horizonCancelButton.setEnabled(true);

        if (state !== 0) {
            view.horizonApplyButton.setEnabled(false);
            view.inlineUnique.setVisible(true);
            view.inlineUnique.setText(this.rb.getText("horizonTable.uniqueError.txt", [state]));
            view.horizonPanel.getBottom().setSize("6rem");
        } else {
            view.horizonApplyButton.setEnabled(true);
            view.inlineUnique.setVisible(false);
            view.horizonPanel.getBottom().setSize("3.5rem");
        }
    },

    isChangeHorizon: function (data) {
        if (data.length !== 1) {
            return false;
        } else if (data[0].periodType === sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE && (undefined === data[0].date || null === data[0].date)) {
            return true;
        }
        return false;
    },

    getCurrentPeriodType: function () {
        var view = this.getView();
        var selectedKey = view.typeSelect.getSelectedKey();
        if (selectedKey) {
            return selectedKey;
        } else {
            // The horizon is using a custom type
            return view.typeSelect.getValue();
        }
    },

    getMaxHorizonDate: function (horizons) {
        if (horizons.length > 0) {
            var maxDate = Math.max.apply(Math, horizons.map(function (o) {
                if (o.date) {
                    return o.date;
                } else {
                    return 0;
                }
            }));
            if (maxDate > 0) {
                return new Date(maxDate);
            }
        }
        return null;
    },

    getNewDateToAdd: function (horizons) {
        var date;
        var greatestDate = this.getMaxHorizonDate(horizons);
        if (greatestDate !== null) {
            // If we're basing it on the existing horizons, we need to increment by 1.
            greatestDate.setUTCDate(greatestDate.getUTCDate() + 1);
            date = greatestDate;
        } else {
            // If we're basing it on the run date, then we leave it as-is.
            date = new Date(this.getContext().getRunDate().replace(/-/g, "/"));
        }
        return date;
    },

    onIntervalChange: function (changeEvent) {
        var that = this;
        var view = this.getView();
        var data = view.horizonTable.getModel().getData();
        var selectedPeriodType = this.getCurrentPeriodType();
        var newTypeIsDate = (selectedPeriodType === sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE);
        var date;

        // Is the only horizon the "change" horizon? If so, we need to leave it as-is. Otherwise we must update all existing horizons.
        if (!this.isChangeHorizon(data)) {
            // It's not the "change" horizon.
            data = data.map(function (horizonObj) {
                // We've switched to a date, if one isn't already set on the horizon then we need to initialize it.
                if (newTypeIsDate) {
                    if (!horizonObj.date) {
                        if (!date) {
                            date = that.getNewDateToAdd(data);
                        } else {
                            date.setUTCDate(date.getUTCDate() + 1);
                        }
                        horizonObj.date = new Date(date);
                    }
                    // Dates do not have horizon set.
                    horizonObj.horizon = -1;
                } else if (!horizonObj.horizon === undefined || -1 === horizonObj.horizon) {
                    // There was no horizon information, so we will default to the index.
                    horizonObj.horizon = horizonObj.index;
                }
                horizonObj.periodType = selectedPeriodType;
                return horizonObj;
            });
        }

        var IntervalTypeUtil = new sas.risk.scenariomanager.utils.IntervalTypeUtil();
        var isDateType = IntervalTypeUtil.isImplicit(selectedPeriodType);

        var scenario = this.getScenario();
        var hasBaseline = (undefined !== scenario.baseline && scenario.baseline.id !== undefined && scenario.baseline.id !== null && scenario.baseline.id !== "0");
        view.horizonDateEditColumn.setVisible(isDateType && !hasBaseline);
        view.horizonDateValueColumn.setVisible(isDateType && hasBaseline);
        view.horizonInputEditColumn.setVisible(!isDateType && !hasBaseline);
        view.horizonInputValueColumn.setVisible(!isDateType && hasBaseline);

        this.setOrUpdateHorizonModelData(data);
        this.checkApplyButton();
    },

    setOrUpdateHorizonModelData: function (horizons) {
        var view = this.getView();
        var horizonModel = view.horizonTable.getModel();
        if (undefined === horizonModel) {
            horizonModel = new sas.risk.common.models.RiskJSONModel(horizons);
            view.horizonTable.setModel(horizonModel);
        } else {
            horizonModel.setData(horizons);
        }

        // Hide "fake" single horizon (aka the "change" column)
        var binding = view.horizonTable.getBinding("rows");
        if (binding !== undefined) {
            var horizonFilter = new sap.ui.model.Filter("horizon", sap.ui.model.FilterOperator.GT, 0);
            var dateFilter = new sap.ui.model.Filter("date", sap.ui.model.FilterOperator.NE, null);
            var allFilters = new sap.ui.model.Filter([horizonFilter, dateFilter]);
            binding.filter(allFilters);
        }
    },

    onDeleteHorizons: function () {
        var view = this.getView();
        var model = view.horizonTable.getModel();
        var data = model.getData();
        var newData = [];
        var itemsSelected = view.horizonTable.getSelectedIndices();

        var toDelete = [];
        var that = this;

        $.each(itemsSelected, function (idx, selectedIdx) {
            var context = view.horizonTable.getContextByIndex(selectedIdx);
            // that.deletedIndex.push(context.getObject().index);
            toDelete.push(selectedIdx);
        });

        var i = 0;
        $.each(data, function (idx, horizon) {
            if ($.inArray(idx, toDelete) === -1) {
                horizon.index = ++i;
                newData.push(horizon);
            }
        });

        view.horizonTable.setSelectedIndex(-1);

        if (newData.length === 0) {
            newData.push(sas.risk.scenariomanager.utils.ScenarioEditorUtil.getNewPeriod(1));
        }

        this.setOrUpdateHorizonModelData(newData);
        this.checkApplyButton();

    },

    onAddHorizons: function (oEvent) {
        var view = this.getView();
        var data = view.horizonTable.getModel().getData();
        var index = data.length + 1;
        var newHorizonValue = 1; // default value
        var newPeriodType = this.getCurrentPeriodType();
        var IntervalTypeUtil = new sas.risk.scenariomanager.utils.IntervalTypeUtil();
        var isDateType = IntervalTypeUtil.isImplicit(newPeriodType);
        if (isDateType) {
            newHorizonValue = -1; // datetime types don't use a horizon value
        }
        var newDate = this.getNewDateToAdd(data);

        if (this.isChangeHorizon(data)) {
            // Update the change horizon
            data[0].horizon = newHorizonValue;
            data[0].periodType = newPeriodType;
            if (isDateType) {
                // Only set date if it is needed (if going from "change" to "day", then skip date)
                data[0].date = newDate;
            }
        } else {
            // Create a new horizon
            if (!isDateType && data.length > 0) {
                // make the new horizon value one more than the current largest
                newHorizonValue = 1 + Math.max.apply(Math, data.map(function (o) {
                        return o.horizon;
                    }));
            }

            // Only set the date if it is needed for the specified period type
            if (!isDateType) {
                newDate = undefined;
            }

            // S1329435: If we've got an 'IMPLICIT' on a new horizon, check if the scenario has a guessed interval;
            // the new horizon has to use the guessed interval
            if (newPeriodType === sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE) {
                newPeriodType = this.getScenario().periodType;
            }

            var newHorizon = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getNewPeriod(index, newPeriodType, newDate, newHorizonValue);

            // Add the new horizon
            data.push(newHorizon);

            // TODO: maybe sort the horizons here, or arrange to insert rather than push
        }

        this.setOrUpdateHorizonModelData(data);
        this.checkApplyButton();

        // Ensure that the newly added row is visible
        view.horizonTable.setFirstVisibleRow(index);
    },

    onCancelHorizons: function () {
        this.horizonDirty = false;
        this.setDirty(this.dirty);

        this.setOrUpdateHorizonPanel(this.getScenario());
    },

    onApplyHorizons: function () {
        var that = this;
        var view = this.getView();
        var context = this.getContext();
        this.openBusyDialog('scenarioEditor.updatingHorizons.txt');

        var changeHorizons = sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(view.horizonTable.getModel()
                                                                                            .getData());

        var sortFunction = (changeHorizons[0].horizon === -1) ?
                           function compare(a, b) {
                               return new Date(a.date).getTime() - new Date(b.date).getTime();
                           } :
                           function compare(a, b) {
                               return a.horizon - b.horizon;
                           };

        changeHorizons.sort(sortFunction);

        var i = 0;
        changeHorizons = changeHorizons.map(function (period) {
            if (period.horizon === -1) {
                if (period.date) {
                    var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance({ pattern: "yyyy-MM-dd" });
                    period.date = dateTimeFormat.format(new Date(period.date), true);
                    /* "true" for UTC */
                    period.visible = sas.risk.scenariomanager.settings.ApplicationSettings.getShowObsoleteHorizons() || !sas.risk.scenariomanager.utils.ScenarioEditorUtil.isHorizonObsolete(period, context.getRunDate());
                }
            } else {
                // If the user made a change to the horizon, then it is currently a string (because of Input),
                // we need to convert it to a number.
                period.horizon = parseInt(period.horizon);
            }
            period.matchString = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getHorizonMatchString(period);
            period.index = ++i;
            period.columnType = "period";
            return period;
        });

        this.horizonDirty = false;
        this.setDirty(true);

        var scenario = this.getScenario();

        var currentPeriods = scenario.periods;
        var fromChangeType = (currentPeriods.length === 1 && currentPeriods[0].horizon === -1 && !currentPeriods[0].date);

        var isOrigEmpty = this._isPeriodListEmpty(currentPeriods);
        var isNewEmpty = this._isPeriodListEmpty(changeHorizons);

        if (isOrigEmpty && isNewEmpty) {
            view.horizonApplyButton.setEnabled(false);
            view.horizonCancelButton.setEnabled(false);
            that.closeBusyDialog('scenarioEditor.updatingHorizons.txt');
            return 0;
        }

        var mainStresses = this.getFlattenedStresses(true);

        var newNumPerturbations = changeHorizons.length;

        if (isOrigEmpty || isNewEmpty) {
            this._clearPerturbationLists(mainStresses, newNumPerturbations);
        } else {
            this._updatePerturbationLists(currentPeriods, changeHorizons, mainStresses, newNumPerturbations);
        }

        scenario.periods = changeHorizons;

        // Ensure the values in the perturbation table are accurate
        for (var stressIndex = 0; stressIndex < mainStresses.length; ++stressIndex) {
            var rf = mainStresses[stressIndex];
            this.scenarioTableHelper.pullNewValues(rf);
            // If we are going to/from the change horizon AND we're not dealing with a hidden stress (rowType is undefined for hidden stresses)
            if ((fromChangeType || isNewEmpty) && undefined !== rf.rowType) {
                rf.perturbTypes = sas.risk.scenariomanager.utils.ScenarioTableHelper.getValidPerturbTypesForRiskSource(this.scenarioTableHelper, rf.variable, rf.rowType, changeHorizons);
            }
        }

        this.scenarioTableHelper.setColumns(scenario.periods);

        scenario.periodType = changeHorizons[0].periodType;

        scenario.stresses = mainStresses;

        if (view.plots) {
            view.plots.onHorizonsChanged();
        }

        view.horizonApplyButton.setEnabled(false);
        view.perturbationTable.clearSelection();

        // Refresh the table columns and rows in a timeout, otherwise the busy indicator doesn't make it up.
        setTimeout(function () {
            view.perturbationTable.getModel().refresh();
        }, 0);
    },

    /*
     * returns a boolean indicating if empty. criteria is
     * expected !null && length!=0  .    Also , if the
     * list is empty, a single row is added to the list with
     * specific values.   This also checks for that entry.
     */
    _isPeriodListEmpty: function (periodList) {
        return sas.risk.scenariomanager.utils.ScenarioTableHelper.isPeriodListEmpty(periodList);
    },

    /*
     * populates the scenario with data based on the list contents.  If one
     * or the other list is empty, there is no need to do the logic of what
     * entry moved up or down.
     */
    _clearPerturbationLists: function (stresses, newNumPerturbations) {
        if (!stresses || stresses.length === 0) {
            return;
        }

        // if new is empty, we need to put a column for Changes.
        for (var stressIdx = 0; stressIdx < stresses.length; stressIdx++) {
            var rf = stresses[stressIdx];
            var zeroedPerturbations = new Array(newNumPerturbations);
            sas.risk.scenariomanager.utils.ScenarioEditorUtil.arrayFill(zeroedPerturbations, 0);
            rf.perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(zeroedPerturbations);

            if (rf.children) {
                this.clonePerturbationsToDescendants(rf);
            }
        }

    },

    /*
     * If horizons were deleted, then a negative number is returned. If horizons were added, then a positive number is returned.
     * If horizons are the same, then zero is returned.
     *
     * @param originalHorizons
     * @param newHorizons
     * @returns {int}
     */
    compareHorizons: function (originalHorizons, newHorizons) {
        // can't use diff on the horizons because api fills out
        // bogus date even if numeric is used.

        var originalStrs = _.pluck(originalHorizons, 'matchString');
        var newStrs = _.pluck(newHorizons, 'matchString');
        return this.compareHorizonMatchStrings(originalStrs, newStrs);
    },

    compareHorizonMatchStrings: function (originalStrs, newStrs) {
        var deletedRows = _.difference(originalStrs, newStrs);
        var addedRows = _.difference(newStrs, originalStrs);

        if (deletedRows.length > 0) {
            // Horizons being removed is more important than horizons being added, because it is deleting data.
            return (-1 * deletedRows.length);
        } else if (addedRows.length > 0) {
            // No horizons are being deleted, but at least one is being added.
            return addedRows.length;
        } else {
            // The horizons are the same.
            return 0;
        }
    },

    /*
     * update the perturbation lists to match the new horizon list.  Handles
     * removing entries for deleted horizons and inserting entries for new ones.
     */
    _updatePerturbationLists: function (originalHorizons, dialogHorizons, stresses, newNumPerturbations) {
        var originalHorizonMatchStrs = _.pluck(originalHorizons, 'matchString');
        var newHorizonMatchStrs = _.pluck(dialogHorizons, 'matchString');
        if (0 === this.compareHorizonMatchStrings(originalHorizonMatchStrs, newHorizonMatchStrs)) {
            // There's nothing to update, close the dialog and get out of here.
            this.closeBusyDialog('scenarioEditor.updatingHorizons.txt');
            return;
        }

        // give mapping of reordered perturbations.
        var oldToNewIdx = new Array(originalHorizons.length);
        sas.risk.scenariomanager.utils.ScenarioEditorUtil.arrayFill(oldToNewIdx, -1);
        var oldIndex;
        for (oldIndex = 0; oldIndex < originalHorizons.length; oldIndex++) {
            var key = originalHorizons[oldIndex].matchString;
            oldToNewIdx[oldIndex] = newHorizonMatchStrs.indexOf(key);
        }

        // update the perturbations
        var stressIdx;
        for (stressIdx = 0; stressIdx < stresses.length; stressIdx++) {
            var rf = stresses[stressIdx];
            var zeroedPerturbations = new Array(newNumPerturbations);
            sas.risk.scenariomanager.utils.ScenarioEditorUtil.arrayFill(zeroedPerturbations, 0);
            var newPerturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.mapPerturbations(zeroedPerturbations);

            // if a horizon was added in middle of two existing, this is logic to have the this table auto update.
            oldIndex = 0;
            for (oldIndex = 0; oldIndex < oldToNewIdx.length; oldIndex++) {
                var newIndex = oldToNewIdx[oldIndex];
                if (newIndex !== -1 && newIndex < newPerturbations.length) {
                    newPerturbations[newIndex] = rf.perturbations[oldIndex];
                }
            }

            rf.perturbations = newPerturbations;
            if (rf.children) {
                this.clonePerturbationsToDescendants(rf);
            }
        }
    },

    // This only clones the values, it doesn't handle overrides. We need to call pullNewValues for that.
    clonePerturbationsToDescendants: function (riskFactor) {
        for (var childIndex = 0; childIndex < riskFactor.children.length; ++childIndex) {
            var child = riskFactor.children[childIndex];
            child.perturbations = sas.risk.scenariomanager.utils.ScenarioEditorUtil.deepCopy(riskFactor.perturbations);
            if (child.children) {
                this.clonePerturbationsToDescendants(child);
            }
        }
    },

    /*
     * changes the horizons list into an array of strings
     *
     * this is necessary because the scenario editor provides data in
     * the value fields even if the interval type does not equal that
     * value type, while the dialog only provides data in the appropriate
     * value field.   Thus we can't do a compare but instead translate
     * to strings , dropping the invalid fields.
     */
    _horizonsAsString: function (horizonArray) {
        var newArray = [];
        if (!horizonArray || horizonArray.length === 0) {
            return newArray;

        }
        var hIdx;
        for (hIdx = 0; hIdx < horizonArray.length; hIdx++) {
            var row = horizonArray[hIdx];
            newArray[hIdx] = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getHorizonMatchString(row);
        }
        return newArray;
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Perturbation Table Event Handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    perturbationTableRowChangeHandler: function (oEvent) {
        this.togglePerturbationTableButtons();
    },

    onCellClick: function (event) {
        var that = this;
        var view = this.getView();
        var bindingContext = event.getParameter("rowBindingContext");
        var columnIndex = event.getParameter("columnIndex");
        if (undefined !== bindingContext && null !== bindingContext && columnIndex >= view.perturbationTable.getFixedColumnCount()) {
            event.preventDefault();
            var obj = bindingContext.getObject();
            // Only swap controls if the cell is editable.
            if (this.isEditable() && false === obj.readOnly) {

                var cellControl = event.getParameter("cellControl");
                // check for an input in the cell
                if ($("#" + cellControl.getId() + " input").length > 0) {
                    // we've already got an input in this cell
                    return;
                }
                that.enterCellEditMode(cellControl, obj, undefined);
            }
        }
    },

    enterCellEditMode: function (cellControl, obj, key) {
        var that = this;
        var content = cellControl.getAggregation("content");
        var text = content[0];
        var horizonIndex = text.data().horizon;
        var pert = obj.perturbations[horizonIndex];

        var input = new sas.hc.m.Input({
            type: "Number",
            textAlign: sap.ui.core.TextAlign.End,
            width: "100%",
            disableValidation: false,
            visible: true,
            value: text.getText() // Use the formatted value from Text, not the perturbation value directly!
        });

        // Are we in "quick edit" mode?
        if (undefined !== key) {
            that.quickEditPreviousValue = input.getValue();
            input.setValue(key);
            // Don't select the text when in quick edit mode, instead we want the cursor to the right of the newly entered value.
            input.setSelectTextOnPress(false);
        }

        text.setVisible(false);
        input.placeAt(cellControl, 1);

        input.addEventDelegate({
                                   onAfterRendering: $.proxy(this.perturbationInputHandler, this, input, text, obj, key, pert, horizonIndex)
                               }, that);
    },

    perturbationInputHandler: function (input, text, obj, key, pert, horizonIndex, oEvent) {
        var that = this;
        var view = this.getView();
        // Grab the actual input
        var inputSelector = jQuery.sap.byId(input.getDomRef("inner").id);
        // set the focus to the input
        input.focus();

        // Are we in "quick edit" mode?
        if (undefined !== key) {
            // multiply length by 2 as a precation to ensure we're at the end across all browsers
            var end = inputSelector.val().length * 2;
            // this moves the cursor to the end of the input
            inputSelector[0].setSelectionRange(end, end);
        }
        that._cellsInEditMode++;

        // watch for focusout from the input, when this happens we want to swap back to Text and destroy the input
        inputSelector.focusout(function () {
            // Only save the value to the model if it is good.
            if (input.getValueState() === sap.ui.core.ValueState.None) {
                // Clear the quick edit previous value, we're accepting the new one.
                that.quickEditPreviousValue = undefined;

                pert.value = that.parsePerturbationValue(input.getValue(), obj.perturbType);

                that.setDirty(true);
                that.scenarioTableHelper.pullNewValues(obj, horizonIndex);

                if (view.plots) {
                    if (_.indexOf(view.plotSplitter.getContentAreas(), view.plots) >= 0) {
                        view.plots.updatePerturbation(obj, horizonIndex);
                    }
                }
            }
            text.setVisible(true);
            // remove the focusout event handler
            inputSelector.off("focusout");
            input.setVisible(false);
            that._cellsInEditMode = 0;
            // destroy the input
            input.destroy();
            view.perturbationTable.getModel().refresh();

        });
    },

    perturbationTableKeydownHandler: function (oEvent) {
        var view = this.getView();
        var ctrl = this;

        if (ctrl._cellsInEditMode === 0) {
            var enterEditMode = true;
            var key;
            // On F2, Spacebar, Enter, minus sign, period
            if (oEvent.which === 113 || oEvent.which === 32 || oEvent.which === 13) {
                enterEditMode = true;
            } else if (oEvent.key && ((oEvent.which >= 48 && oEvent.which <= 57) || (oEvent.which >= 96 && oEvent.which <= 105))) { // TODO in Future: when we support instance variables in the pert table, this will need to change
                //($.isNumeric(oEvent.key)
                // 48 to 57 are top numbers, 96 to 105 are numberpad numbers
                key = oEvent.key;
            } else if (oEvent.which === 109 || oEvent.which === 189) { // 173 in FF?
                key = "-";
            } else if (oEvent.which === 190) {
                key = ".";
            } else if (oEvent.which === 188) {
                key = ",";
            } else if (oEvent.key && oEvent.which === 110) {
                // this is the locale-specific decimal point
                key = oEvent.key;
            } else {
                enterEditMode = false;
            }
            if (enterEditMode) {
                var focusedArea = view.perturbationTable.getFocusDomRef();
                if (focusedArea && focusedArea.id) {
                    var $areas = $("#" + focusedArea.id + " .sapUiTableCell .sasMText").control();
                    if ($areas.length > 0) {
                        var text = $areas[0];
                        if (text && text.getBindingContext) {
                            var bindingContext = text.getBindingContext();
                            if (undefined !== bindingContext && null !== bindingContext) {
                                var obj = bindingContext.getObject();
                                // Only swap controls if the cell is editable.
                                if (ctrl.isEditable() && false === obj.readOnly) {
                                    // check for an input in the cell
                                    if ($("#" + focusedArea.id + " .sapUiTableCell input").length > 0) {
                                        // we've already got an input in this cell
                                        oEvent.preventDefault();
                                        return;
                                    }
                                    var cellControl = $("#" + focusedArea.id + " .sapUiTableCell .sapUiHLayout")
                                        .control()[0];

                                    if (undefined !== cellControl) {
                                        // we're focused on a table cell, we need to fake the cellClick!
                                        ctrl.enterCellEditMode(cellControl, obj, key);
                                        oEvent.preventDefault();
                                        return;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        } else if (this._cellsInEditMode === 1 && oEvent.which === 27) {
            // The user is in cell-edit mode and hit the "esc" key, we need to revert to the previously value (NOT the value used to enter quick-edit mode)
            if (undefined !== this.quickEditPreviousValue) {
                var focusedCell = view.perturbationTable.getFocusDomRef();
                if (focusedCell && focusedCell.id) {
                    var inputs = $("#" + focusedCell.id + " .sapUiTableCell input");
                    if (inputs.length > 0) {
                        var input = inputs.control()[0];
                        // Set the input back to the previously assigned value
                        input.setValue(this.quickEditPreviousValue);
                        this.quickEditPreviousValue = undefined;
                    }
                }
            }
        }
    },

    handleScrollInCellEditMode: function () {
        if (this._cellsInEditMode > 0) {
            this.getView().perturbationTable.focus();
        }
    },

    parsePerturbationValue: function (valueToParse, perturbationType) {
        if (!this.percentType) {
            this.percentType = new sap.ui.model.type.Float({
                decimals: 4,
                groupingEnabled: true,
                percentSign: ''
            });
        }
        if (!this.floatType) {
            this.floatType = new sap.ui.model.type.Float({
                decimals: 4,
                groupingEnabled: true
            });
        }
        var vResult;
        var oType = (perturbationType === "REL" || perturbationType === "REL_TIME") ? this.percentType : this.floatType;
        var oOutputFormat = oType.oOutputFormat,
            oInputFormat = oType.oInputFormat;

        vResult = oOutputFormat.parse(valueToParse);
        if (oType === this.percentType) {
            vResult /= 100.0; //store in model as a float
        }

        if (oInputFormat) {
            vResult = oInputFormat.format(vResult);
        }

        if (!isNaN(vResult)) {
            return vResult;
        }
    },

    //*****************************************************************************************************
    //*****************************************************************************************************
    // Risk factor handlers
    //*****************************************************************************************************
    //*****************************************************************************************************

    getRiskSourceFromTable: function (index) {
        var view = this.getView();

        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            var selections = view.riskSourcesTree.getSelectedIndices();
            if (selections !== null && selections.length > 0) {
                index = selections[0];
            } else {
                index = -1;
            }
        }

        var riskSource = null;
        if (index > -1) {
            var context = view.riskSourcesTree.getContextByIndex(index);
            if (context) {
                riskSource = context.getObject();
            }
        }
        return riskSource;
    },

    buildTreeNodes: function (contextData, unassignedGroupName) {
        var that = this;
        var constants = sas.risk.scenariomanager.model.RiskVarTypeConstants;
        var rootNode = {};
        rootNode.children = [];

        if (!contextData) {
            return rootNode;
        }

        var groupNames = contextData.getObjectNames(constants.GROUP_OBJTYPE).slice().sort();
        var driverNames = contextData.getObjectNames(constants.DRIVER_OBJTYPE).slice().sort();
        groupNames[groupNames.length] = unassignedGroupName;

        this.buildNodes(driverNames, constants.DRIVER_OBJTYPE, rootNode, true, contextData);
        this.buildNodes(groupNames, constants.GROUP_OBJTYPE, rootNode, true, contextData);

        var groupNodes = rootNode.children;

        // create risk factor groups
        var groupNodeIdx = 0;

        _.each(groupNodes, function (groupNode) {
            var riskFactorNames = [];
            var arrayNames = [];
            var driverNames = [];

            if (groupNode.elementName === unassignedGroupName) {
                riskFactorNames = contextData.getUnassignedVarNames(constants.RISKFACTOR_OBJTYPE);
                arrayNames = contextData.getUnassignedVarNames(constants.RISKFACTOR_ARRAY_OBJTYPE);
                if (riskFactorNames.length > 0 || arrayNames.length > 0) {
                    that.populateGroupNode(groupNode, riskFactorNames, arrayNames, driverNames, contextData);
                    groupNodeIdx++;
                } else {
                    // remove unassigned, it has no children
                    rootNode.children.splice(rootNode.children.length - 1, 1);
                }
            } else {
                if (constants.GROUP_OBJTYPE === groupNode.elementType) {
                    var groupDetails = contextData.getObject(groupNode.elementName, groupNode.elementType);
                    riskFactorNames = groupDetails.riskFactorNames;
                    arrayNames = groupDetails.riskFactorArrayNames;
                } else if (constants.DRIVER_OBJTYPE === groupNode.elementType) {
                    var driverDetails = contextData.getObject(groupNode.elementName, groupNode.elementType);
                    riskFactorNames = driverDetails.riskFactorNames;
                    arrayNames = driverDetails.riskFactorArrayNames;
                    driverNames = driverDetails.riskFactorDriverNames;
                }
                that.populateGroupNode(groupNode, riskFactorNames, arrayNames, driverNames, contextData);
                groupNodeIdx++;
            }
        });

        return rootNode;
    },

    /*
     * Create tree nodes for the list of names, setting the type to the given
     * and, if provided, add them to the parentNode starting at the indicated position.
     */
    buildNodes: function (elementNames, elementType, parentNode, keepNameOrder, contextData) {
        if (!elementNames || elementNames.length === 0) {
            return [];
        }

        // var parentNodeDetails;
        // if (parentNode.elementName && parentNode.elementType) {
        //     parentNodeDetails = contextData.getObject(parentNode.elementName, parentNode.elementType);
        // }

        var sortedNames = (keepNameOrder ? elementNames : elementNames.slice().sort());
        _.each(sortedNames, function (nodeName) {

            var node = {
                'elementName': nodeName,
                'elementType': elementType
            };
            if (!parentNode.children) {
                parentNode.children = [];
            }
            // // If we're adding risk factors to a group, then we may need to do some pruning...
            // if (sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_OBJTYPE === elementType &&
            //     sas.risk.scenariomanager.model.RiskVarTypeConstants.GROUP_OBJTYPE === parentNode.elementType &&
            //     parentNodeDetails && parentNodeDetails.riskFactorArrayNames && parentNodeDetails.riskFactorArrayNames.length > 0) {
            //     // Iterate through the arrays until one is found that contains the risk factor (if one exists)
            //     var foundArray = _.find(parentNodeDetails.riskFactorArrayNames, function (arrayName) {
            //         var arrayDetails = contextData.getObject(arrayName, sas.risk.scenariomanager.model.RiskVarTypeConstants.RISKFACTOR_ARRAY_OBJTYPE);
            //         return _.find(arrayDetails.riskFactorNames, function (riskFactorName) {
            //             return riskFactorName === nodeName;
            //         });
            //     });
            //     // Only add the risk factor to the group if it isn't already accounted for in one of the grouped arrays
            //     if (!foundArray) {
            parentNode.children.push(node);
                //}
            // } else {
            //     parentNode.children.push(node);
            // }
        });
    },

    /*
     * Populate the group node with the risk factors and risk factor arrays
     * in the group, also populating the risk factor array with contents.
     */
    populateGroupNode: function (groupNode,
                                 riskFactorNames,
                                 arrayNames,
                                 driverNames,
                                 contextData) {
        var that = this;
        var constants = sas.risk.scenariomanager.model.RiskVarTypeConstants;

        this.buildNodes(riskFactorNames, constants.RISKFACTOR_OBJTYPE, groupNode, false, contextData);
        this.buildNodes(arrayNames, constants.RISKFACTOR_ARRAY_OBJTYPE, groupNode, false, contextData);
        this.buildNodes(driverNames, constants.DRIVER_OBJTYPE, groupNode, false, contextData);

        if (groupNode.children) {
            _.each(groupNode.children, function (childNode) {
                if (constants.RISKFACTOR_ARRAY_OBJTYPE === childNode.elementType || constants.DRIVER_OBJTYPE === childNode.elementType) {
                    var objectDetails = contextData.getObject(childNode.elementName, childNode.elementType);
                    that.buildNodes(objectDetails.riskFactorNames, constants.RISKFACTOR_OBJTYPE, childNode, false, contextData);
                    that.buildNodes(objectDetails.riskFactorArrayNames, constants.RISKFACTOR_ARRAY_OBJTYPE, childNode, false, contextData);
                    that.buildNodes(objectDetails.riskFactorDriverNames, constants.DRIVER_OBJTYPE, childNode, false, contextData);

                    if (childNode.children) {
                        _.each(childNode.children, function (grandchildNode) {
                            if (constants.RISKFACTOR_ARRAY_OBJTYPE === grandchildNode.elementType) {
                                var grandchildObjectDetails = contextData.getObject(grandchildNode.elementName, grandchildNode.elementType);
                                that.buildNodes(grandchildObjectDetails.riskFactorNames, constants.RISKFACTOR_OBJTYPE, grandchildNode, false, contextData);
                            }
                        });
                    }
                }
            });
        }
    },

    onCollapseAllInRiskSourcesTable: function (event) {
        this.getView().riskSourcesTree.collapseAll();
    },

    onExpandAllInRiskSourcesTable: function (event) {
        var view = this.getView();
        var binding = view.riskSourcesTree.getBinding("rows");
        var model = view.riskSourcesTree.getModel();
        for (var i = 0; i < binding.getLength(); ++i) {
            var context = view.riskSourcesTree.getContextByIndex(i);
            var nodePath = context.getPath();
            var node = model.getProperty(nodePath);
            if (node.children && node.children.length > 0) {
                view.riskSourcesTree.expand(i);
            }
        }
    },

    onRiskSourcesTreeFilterChanged: function (event) {
        var value = event.getSource().getValue() || "";
        var nameFilter = new sap.ui.model.Filter("elementName", sap.ui.model.FilterOperator.Contains, value);
        var allFilters = new sap.ui.model.Filter([nameFilter]);
        var binding = this.getView().riskSourcesTree.getBinding("rows");
        binding.filter(allFilters);
    },

    onRiskSourcesTreeTableRowSelectionChange: function (event) {
        var rowIndex = event.getParameter('rowIndex');
        if (rowIndex === -1) {
            return;
        }
        var selectedRiskSource = this.getRiskSourceFromTable(rowIndex);

        var propertyData = [];
        var scenarioContext = this.getContext();
        var contextObject = scenarioContext.getObject(selectedRiskSource.elementName, selectedRiskSource.elementType);
        if (undefined === contextObject || null === contextObject) {
            this.setOrUpdateMetadataTableModelData();
            return;
        }

        var metadataFields = this.getMetadataTableFields(selectedRiskSource.elementType);
        var rowCount = 0;

        for (var idx = 0; idx < metadataFields.length; idx++) {
            var fieldName = metadataFields[idx];
            var row = this.getMetadataDisplayValue(contextObject, fieldName, selectedRiskSource.elementType);
            if (undefined !== row && null !== row) {
                propertyData[rowCount++] = row;
            }
        }

        this.setOrUpdateMetadataTableModelData(propertyData);
    },

    setOrUpdateMetadataTableModelData: function (propertyData) {
        var view = this.getView();
        var metadataModel = view.metadataTable.getModel();

        if (undefined === propertyData) {
            propertyData = [];
        }

        if (undefined === metadataModel) {
            metadataModel = new sas.risk.common.models.RiskJSONModel(propertyData);
            view.metadataTable.setModel(metadataModel);
        } else {
            metadataModel.setData(propertyData);
        }
    },

    getMetadataTableFields: function (objectType) {
        var oTypeConst = sas.risk.scenariomanager.model.RiskVarTypeConstants;
        var OBJECT_TYPES = sas.risk.scenariomanager.model.RiskVarTypeConstants.OBJECT_TYPES;
        if (undefined === objectType || null === objectType ||
            undefined === OBJECT_TYPES[objectType.toUpperCase()] || null === OBJECT_TYPES[objectType.toUpperCase()]) {
            return ["name", "label"];
        }

        var fields = ["name", "label", "type"];
        switch (OBJECT_TYPES[objectType.toUpperCase()]) {
            case OBJECT_TYPES[oTypeConst.RISKFACTOR_OBJTYPE]  :
                fields = ["name",
                          "label",
                          "type",
                          "role",
                          "volatilityOfName", // base risk factor
                          "measurementLevel",
                          "fromcur", // "from currency"
                          "currency",
                          "maturity",
                          "maturityUnits"];
                break;
            case OBJECT_TYPES[oTypeConst.RISKFACTOR_ARRAY_OBJTYPE]  :
                fields = ["name", "label", "type", "role", "currency"];
        }

        return fields;
    },

    getMetadataDisplayValue: function (elementContext, fieldName, objectType) {
        if (fieldName !== "type" && !elementContext.hasOwnProperty(fieldName)) {
            return null;
        }

        var attrConst = sas.risk.scenariomanager.model.RiskVarConstants.ATTRIBUTES;
        var fieldValue = attrConst[fieldName.toUpperCase()];
        var label = this.rb.getText("riskVarMetaData." + fieldName + ".txt");

        var value = "";

        switch (fieldValue) {
            case attrConst.NAME:
                value = elementContext.name;
                break;

            case attrConst.LABEL:
                value = elementContext.label;
                if (undefined === value || null === value || "" === value) {
                    value = elementContext.name;
                }
                break;

            case attrConst.TYPE:
                value = this.getObjectTypeText(objectType);
                break;

            case attrConst.MATURITYUNITS:
                value = this.getRiskVarMaturityUnitsText(elementContext.maturityUnits);
                break;

            case attrConst.ROLE:
                value = this.getRiskVarRoleText(elementContext.role);
                break;

            case attrConst.MEASUREMENTLEVEL:
                value = this.getRiskVarMeasurementLevelText(elementContext.measurementLevel);
                break;

            case attrConst.VOLATILITYOFNAME:
                value = elementContext.volatilityOfName;
                break;

            case attrConst.CURRENCY:
                value = elementContext.currency;
                break;

            case attrConst.FROMCURRENCY:
                value = elementContext.fromcur;
                break;

            case attrConst.MATURITY:
                value = elementContext.maturity;
                break;

            default :
        }

        if (undefined === value || null === value) {
            value = "";
        }

        label += ":";

        return { id: label, value: value };
    },

    getObjectTypeText: function (objectType) {
        if (undefined === objectType || null === objectType) {
            return "";
        }
        var OBJECT_TYPES = sas.risk.scenariomanager.model.RiskVarConstants.OBJECT_TYPES;
        var oTypeConst = sas.risk.scenariomanager.model.RiskVarTypeConstants;
        switch (OBJECT_TYPES[objectType.toUpperCase()]) {
            case OBJECT_TYPES[oTypeConst.RISKFACTOR_OBJTYPE]:
                return this.rb.getText("riskVar.type.single.txt");
            case OBJECT_TYPES[oTypeConst.DRIVER_OBJTYPE]:
                return this.rb.getText("riskVar.type.driver.txt");
            case OBJECT_TYPES[oTypeConst.GROUP_OBJTYPE]:
                return this.rb.getText("riskVar.type.group.txt");
            case OBJECT_TYPES[oTypeConst.RISKFACTOR_ARRAY_OBJTYPE]:
                return this.rb.getText("riskVar.type.array.txt");
        }
        return objectType;
    },

    getRiskVarMaturityUnitsText: function (mUnits) {
        if (undefined === mUnits || null === mUnits) {
            return "";
        }

        var MUNITS = sas.risk.scenariomanager.model.RiskVarConstants.MUNIT;
        switch (MUNITS[mUnits.toUpperCase()]) {
            case MUNITS.DAY:
                return this.rb.getText("riskVar.munit.day.txt");
            case MUNITS.HOUR:
                return this.rb.getText("riskVar.munit.hour.txt");
            case MUNITS.MONTH:
                return this.rb.getText("riskVar.munit.month.txt");
            case MUNITS.WEEK:
                return this.rb.getText("riskVar.munit.week.txt");
            case MUNITS.YEAR:
                return this.rb.getText("riskVar.munit.year.txt");
        }

        return mUnits;
    },

    getRiskVarRoleText: function (vrole) {
        if (undefined === vrole || null === vrole) {
            return "";
        }

        var roleConst = sas.risk.scenariomanager.model.RiskVarConstants.ROLE_RF;
        var roleValue = roleConst[vrole.toUpperCase()];

        switch (roleValue) {
            case roleConst.FX:
                return this.rb.getText("riskVar.role.FX.txt");
            case roleConst.FX_FORWARD:
                return this.rb.getText("riskVar.role.FX_FORWARD.txt");
            case roleConst.FX_SPOT:
                return this.rb.getText("riskVar.role.FX_SPOT.txt");
            case roleConst.FXSURFACE:
                return this.rb.getText("riskVar.role.FXSURFACE.txt");
            case roleConst.IR:
                return this.rb.getText("riskVar.role.IR.txt");
            case roleConst.IRSURFACE:
                return this.rb.getText("riskVar.role.IRSURFACE.txt");
            case roleConst.SURFACE:
                return this.rb.getText("riskVar.role.SURFACE.txt");
            case roleConst.VAR:
                return this.rb.getText("riskVar.role.VAR.txt");
            case roleConst.VOLATILITY:
                return this.rb.getText("riskVar.role.VOLATILITY.txt");
        }
        return vrole;
    },

    getRiskVarMeasurementLevelText: function (mLevel) {
        if (undefined === mLevel || null === mLevel) {
            return "";
        }

        var MLEVELS = sas.risk.scenariomanager.model.RiskVarConstants.MLEVEL;

        switch (mLevel) {
            case MLEVELS.RATIO:
                return this.rb.getText("riskVar.mlevel.ratio.txt");
            case MLEVELS.INTERVAL:
                return this.rb.getText("riskVar.mlevel.interval.txt");
        }

        return this.rb.getText("riskVar.mlevel.undefined.txt");
    },

    onChangeDataSource: function () {
        var ctrl = this;
        var view = this.getView();
        var scenario = this.getScenario();
        var workGroupsForCubes = sas.risk.scenariomanager.utils.ScenarioEditorUtil.getWorkGroupsForCubes(scenario.workGroup);

        ctrl.openBusyDialog("scenarioEditor.loadCubeDetails.txt");
        sas.risk.scenariomanager.ScenarioAPI.getCubes(workGroupsForCubes).done(function (cubes) {
            ctrl.closeBusyDialog("scenarioEditor.loadCubeDetails.txt");
            ctrl.openChangeDataSourceDialog(cubes);
        }).fail(function (event) {
            ctrl.closeBusyDialog("scenarioEditor.loadCubeDetails.txt");
            ctrl.onServerFailure(event);

            // Restore the repository checkbox in the event of a failure loading cubes
            var selectedDataSource = view.dataSourceField.getInputProperty("value");
            if (!selectedDataSource || selectedDataSource == undefined) {
                view.repositoryCheckBox.setSelected(true);
                view.repositoryCheckBox.fireSelect();
            }
        });
    },

    openChangeDataSourceDialog: function (cubelist) {
        // need the list sorted by first type (stress/price) and then by name.
        // can't get the table dialog to group.
        var ctrl = this;
        var view = this.getView();

        //  the list is suppose to be sorted by  type then by name.
        var sortedData = cubelist.slice();

        var oTableSelectDialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.DataSourcesSelectDialog');
        if (!oTableSelectDialog) {
            oTableSelectDialog = sap.ui.jsfragment('sas.risk.scenariomanager.view.DataSourcesSelectDialog', ctrl);
        }

        oTableSelectDialog.attachAfterClose(function () {
            var selectedDataSource = view.dataSourceField.getInputProperty("value");
            if (!selectedDataSource || selectedDataSource == undefined) {
                view.repositoryCheckBox.setSelected(true);
                view.repositoryCheckBox.fireSelect();
            }
        });

        var scenario = this.getScenario();
        var currentContext = sas.risk.scenariomanager.ScenarioAPI.findOptionalLink(scenario.links, { rel: 'stressContext' });
        var selected = null;

        if (currentContext && currentContext.href) {
            // TODO: Replace with "up" link from stress context -> cube details
            var uri = jQuery.sap.getUriParameters(currentContext.href);
            selected = {};
            selected.application = uri.get("application");
            selected.workGroup = uri.get("workGroup");
            selected.relPath = uri.get("relPath");
            selected.name = scenario.dataSourceName;
        }

        var dialogModel = oTableSelectDialog.getModel();
        if (!dialogModel) {
            dialogModel = new sas.risk.common.models.RiskJSONModel();
            oTableSelectDialog.setModel(dialogModel);
        }

        dialogModel.setProperty("/dataSources", { folders: sortedData });
        dialogModel.setProperty("/selectedDataSource", selected);

        oTableSelectDialog.setModel(view.getModel('i18N'), 'i18N');
        oTableSelectDialog.open();
    },

    /*
     * Set the scenario to use the given datasource.
     * The data passed is that of the entry as it appears in the context listing.
     *
     * {  "id", "name", "lib", "label", links: { ... }
     */
    onChangeDataSourceOkay: function (cube) {
        // SOPHIE TODO: This function should be combined with ScenarioPropertyController's onDataSourceSelected function, they're basically clones.
        var deferred = $.Deferred();
        var ctrl = this;
        var scenario = this.getScenario();

        var getDataSourceArgs = function (def, dataSource) {
            var selfLink = sas.risk.scenariomanager.ScenarioAPI.findRequiredLink(def, dataSource.links, { rel: 'self' }),
                tokens = selfLink.href.split("?");

            return (tokens.length < 2) ? "" : tokens[1];
        };

        if (cube) {
            scenario.dataSourceId = cube.id;
            scenario.dataSourceName = cube.id; // Use id as the name, that way it doesn't have the .rskcdesc extension, which can cause problems when trying to run in HPRisk
            scenario.dataSourceArgs = getDataSourceArgs(deferred, cube);

            // MAS todo: is this right?
            // replace the stressContext link in the scenario with that of the selected cube
            var scenarioContextLinkIndex = -1;
            _.find(scenario.links, function (c, index) {
                if (c.rel === 'stressContext') {
                    scenarioContextLinkIndex = index;
                    return true;
                }
            });

            sas.risk.scenariomanager.ScenarioAPI.getCubeSelf(cube).done(function (context) {

                var cubeContextLink = _.findWhere(context.links, { rel: 'stressContext' });
                if (cubeContextLink) {
                    if (scenarioContextLinkIndex !== -1) {
                        scenario.links[scenarioContextLinkIndex] = cubeContextLink;
                    } else {
                        scenario.links.push(cubeContextLink);
                    }
                }

                ctrl.openBusyDialog();
                sas.risk.scenariomanager.ScenarioAPI.getCubeContext(context).done(function (context) {
                    ctrl.setContext(context);
                    ctrl.setupRiskSourcesModel();
                    ctrl.refresh(scenario, context);
                    ctrl.closeBusyDialog();
                    deferred.resolve();
                }).fail(function (event) {
                    ctrl.closeBusyDialog();
                    sas.hc.m.MessageBox.error(ctrl.rb.getText("changeDataSource.noContext.fmt.txt", context.name), ctrl.rb.getText("changeDataSource.error.txt"));
                    deferred.reject(event);
                });

            }).fail(function (jqxhr) {
                deferred.reject(jqxhr);
            });

        } else {
            scenario.dataSourceId = '';
            scenario.dataSourceName = '';
            scenario.dataSourceArgs = '';

            // MAS todo: is this right?
            // remove the stressContext link from the scenario since using the scenario repository
            var scenarioContextLinkIndex = -1;
            _.find(scenario.links, function (c, index) {
                if (c.rel === 'stressContext') {
                    scenarioContextLinkIndex = index;
                    return true;
                }
            });
            if (scenarioContextLinkIndex !== -1) {
                scenario.links.splice(scenarioContextLinkIndex, 1);
            }

            ctrl.openBusyDialog();
            sas.risk.scenariomanager.ScenarioAPI.getRepositoryContext().done(function (context) {
                ctrl.setContext(context);
                ctrl.setupRiskSourcesModel();
                ctrl.refresh(scenario, context);
                ctrl.closeBusyDialog();
                deferred.resolve();
            }).fail(function (event) {
                ctrl.closeBusyDialog();
                ctrl.onServerFailure(event);
                deferred.reject(event);
            });
        }

        this.setDirty(true);
        return deferred;

    },

    onFilterChanged: function (event) {
        var view = this.getView();
        var value = event.getSource().getValue() || '';
        var nameFilter = new sap.ui.model.Filter('variable', sap.ui.model.FilterOperator.Contains, value);
        var binding = view.perturbationTable.getBinding('rows');
        binding.filter(nameFilter);
    },

    onCollapseAllInPerturbationTable: function (event) {
        this.getView().perturbationTable.collapseAll();
    },

    onExpandAllInPerturbationTable: function (event) {
        var view = this.getView();
        var binding = view.perturbationTable.getBinding("rows");
        var model = view.perturbationTable.getModel();

        for (var i = 0; i < binding.getLength(); ++i) {
            var context = view.perturbationTable.getContextByIndex(i);
            var nodePath = context.getPath();
            var node = model.getProperty(nodePath);
            if (node.children && node.children.length > 0) {
                view.perturbationTable.expand(i);
            }
        }
    },

    onServerFailure: function (event) {
        var title = this.rb.getText('error.window.title');
        var response = event.responseJSON;
        if (response) {
            sas.risk.common.widgets.MessageDetailsBox.error(response, title);
        } else {
            sas.hc.m.MessageBox.error(this.rb.getText('error.message.txt'), title);
        }
    },

    openBusyDialog: function (titleKey) {
        if (!titleKey) {
            titleKey = "default.busyDialog.title";
        }
        if (!this.busyDialogs) {
            this.busyDialogs = {};
        }

        if (!this.busyDialogs[titleKey]) {
            var busyDialog = new sas.hc.m.BusyDialog({
                title: this.rb.getText(titleKey)
            });
            busyDialog.setEarcon(null);
            this.busyDialogs[titleKey] = busyDialog;
            busyDialog.open();
        }
    },

    hasBusyDialog: function (titleKey) {
        if (!titleKey) {
            titleKey = "default.busyDialog.title";
        }
        return (this.busyDialogs && this.busyDialogs[titleKey]);
    },

    closeBusyDialog: function (titleKey) {
        if (!titleKey) {
            titleKey = "default.busyDialog.title";
        }
        if (this.busyDialogs && this.busyDialogs[titleKey]) {
            this.busyDialogs[titleKey].close();
            this.busyDialogs[titleKey].destroy();
            this.busyDialogs = _.omit(this.busyDialogs, titleKey);
        }
    },

    closeAllBusyDialogs: function () {
        if (!this.busyDialogs) {
            this.busyDialogs = {};
        } else {
            var titleKeys = Object.keys(this.busyDialogs);
            for (var numKeys = titleKeys.length; 0 < numKeys; --numKeys) {
                var busyDialog = this.busyDialogs[titleKeys[0]];
                if (busyDialog) {
                    busyDialog.close();
                    busyDialog.destroy();
                }
                this.busyDialogs = _.omit(this.busyDialogs, titleKeys[0]);
            }
        }
    },

    /*
     * If the first scenario period is a date before the run date, warn the user.
     */
    _checkDateRangeMismatch: function (scenario, context) {
        if ((!context) ||
            (!context.getRunDate()) ||
            (!scenario) ||
            (!scenario.periodType) ||
            (scenario.periodType !== sas.risk.scenariomanager.model.IntervalTypeConstants.INTERVAL_TYPE_DATE) ||
            (!scenario.periods || scenario.periods.length === 0)) {
            return;
        }

        var ctrl = this;
        var serverDateFormat = sap.ui.core.format.DateFormat.getDateInstance({
                                                                                 pattern: "yyyy-MM-dd"
                                                                             });

        var runDate = ((Object.prototype.toString.call(context.getRunDate()) === '[object Date]') ?
                       context.getRunDate() : serverDateFormat.parse(context.getRunDate()));

        var firstHorizon = scenario.periods[0];
        var horizonDate = null;

        if (firstHorizon.date) {
            horizonDate = (
                (Object.prototype.toString.call(firstHorizon.date) === '[object Date]') ?
                firstHorizon.date : serverDateFormat.parse(firstHorizon.date));
        } else {
            return;
        }

        if (undefined !== horizonDate && null !== horizonDate && horizonDate > runDate) {
            return;
        }

        sas.hc.m.MessageBox.warning(
            ctrl.rb.getText('scenarioEditor.obsoleteHorizonse.warning.txt',
                            [scenario.name, runDate.toDateString()]),
            ctrl.rb.getText('scenarioEditor.obsoleteHorizonse.window.title'),
            [sas.hc.m.MessageBox.Action.OK]);

    },

    onVariablePlotted: function (event) {
        var view = this.getView();
        var selectedRiskFactors = this.getSelectedRiskFactors();
        if (!view.plots) {
            view.createPlots(this);
        }
        if (view.plots) {
            if (selectedRiskFactors.length === 0) {
                this.showHidePlotSplitter(false);
            } else {
                this.showHidePlotSplitter(true);
                view.plots.plotVariables(selectedRiskFactors);
            }
        }
        // Clear selections after plotting
        view.perturbationTable.clearSelection();
    },

    onVariableUnplotted: function (event) {
        var view = this.getView();
        var variable = event.getParameter("variable");

        if (view.plots) {
            var numVariables = view.plots.countVariables();
            if (numVariables === 0) {
                this.showHidePlotSplitter(false);
            }
        }
    },

    showHidePlotSplitter: function (visible) {
        var view = this.getView();
        if (view.plots) {
            if (visible) {
                view.plotSplitter.addContentArea(view.plots);
            } else {
                view.plotSplitter.removeContentArea(view.plots);
            }
            var areas = view.plotSplitter.getContentAreas();
            areas[0].getLayoutData().setSize(visible ? "60%" : "100%");
        }
    },

    _subscribeToSettingsChanged: function () {
        var ctrl = this;
        var view = this.getView();
        sas.applicationSwitcher.subscribe(sas.applicationSwitcher.APP_PREFENCE_CHANGED_MESSAGETYPE, function (type,
                                                                                                              data) {
            if (data && data.settings) {
                _.each(data.settings, function (setting) {
                    switch (setting.id) {
                        case sas.risk.scenariomanager.settings.ApplicationSettings.DEFAULT_SCENARIO_DATE:
                            var context = ctrl.getContext();
                            // FIXME: Avoid hardcoding this?
                            if (context.getName() === "Scenario Repository") {
                                context.setRunDate(new Date(setting.newValue));
                                ctrl.scenarioTableHelper.checkColumnVisibility();
                                view.perturbationTable.getModel().refresh();
                            }
                            // changing the date may cause change in displayed horizons
                            break;
                        case sas.risk.scenariomanager.settings.ApplicationSettings.SHOW_OBSOLETE_HORIZONS:
                            ctrl.scenarioTableHelper.checkColumnVisibility();
                            view.perturbationTable.getModel().refresh();
                            break;
                    }
                });
            }
        });
    }

});