jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sap.ui.model.type.Date");
jQuery.sap.require('sas.hc.m.BusyDialog');
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require('sas.hc.m.MessageBox');
jQuery.sap.require('sas.hc.m.MessageToast');
jQuery.sap.require('sas.risk.common.utils.LayoutService');
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require('sas.risk.scenariomanager.EventBusConstants');
jQuery.sap.require("sas.risk.scenariomanager.utils.IntervalTypeUtil");
jQuery.sap.require('sas.risk.common.widgets.MessageDetailsBox');
jQuery.sap.require('sas.risk.scenariomanager.utils.ScenarioPropertyController');
jQuery.sap.require('sas.risk.scenariomanager.ScenarioAPI');
jQuery.sap.require('sas.risk.scenariomanager.ViewManager');
jQuery.sap.require('sas.risk.common.utils.SASStudioProxy');

sap.ui.controller('sas.risk.scenariomanager.view.ScenarioManager', {

    onInit: function () {
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: 'sas.risk.scenariomanager.nls.messagebundle' });
        this.rb = i18N.getResourceBundle();
        view.setModel(i18N, 'i18N');
        sas.eventBus.subscribe(sas.risk.scenariomanager.EventBusConstants.CHANNEL_NAME, sas.risk.scenariomanager.EventBusConstants.EXISTING_SCENARIO_MANAGER_OPENED, this.onExistingScenarioManagerOpened, this);

        new sas.risk.common.utils.LayoutService(view.scenarioTable, "ScenarioManagerMainTable");

        view.setModel(new sas.risk.common.models.RiskJSONModel(sas.risk.scenariomanager.ScenarioAPI.getWorkGroups()), 'availableWorkGroups');
        view.setModel(new sas.risk.common.models.RiskJSONModel(sas.risk.scenariomanager.ScenarioAPI.getRecommendedPrimaryWorkGroup()), 'recommendedPrimaryWorkGroup');

        this.refreshModel(true);
    },

    onExit: function () {
        sas.eventBus.unsubscribe(sas.risk.scenariomanager.EventBusConstants.CHANNEL_NAME, sas.risk.scenariomanager.EventBusConstants.EXISTING_SCENARIO_MANAGER_OPENED, this.onExistingScenarioManagerOpened, this);
    },

    onExistingScenarioManagerOpened: function (channel, event, data) {
        this.refreshModel(false, data.hideBusy);
    },

    refreshModel: function (fromInit, hideBusy) {

        var scenarioIdOpenRequested = jQuery.sap.getUriParameters().get("id");

        if (!fromInit || fromInit === undefined) {
            fromInit = false;
        }

        var ctrl = this;
        var view = ctrl.getView();
        if (hideBusy !== true) ctrl.openBusyDialog("scenarioManager.refreshModel.txt");

        // TODO: preserve selection state
        sas.risk.scenariomanager.ScenarioAPI.getScenarios().done(function (scenarioData) {
            // To maintain any sort already applied on the table, if model exists, set data on the model.
            // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
            var model = view.getModel();
            if (model instanceof sas.risk.common.models.RiskJSONModel) {
                model.setProperty('/scenarios', scenarioData);
            } else {
                view.setModel(new sas.risk.common.models.RiskJSONModel({ scenarios: scenarioData }));

                // Default sort by Name asc.
                var sorters = [new sap.ui.model.Sorter("workGroup", false), // TODO: custom sorter that puts the public work group at the top?
                               new sap.ui.model.Sorter("label", false),
                               new sap.ui.model.Sorter("scenarioVersion", true)];
                var binding = view.scenarioTable.getBinding("rows");
                if (binding) {
                    binding.sort(sorters);
                }
            }

            // The work groups are loaded by the shell, so we don't need deferred
            var workGroups = sas.risk.scenariomanager.ScenarioAPI.getWorkGroups();
            var hasWorkGroups = (workGroups !== null && workGroups !== undefined && workGroups.length > 0);

            view.scenarioTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                title: "{i18N>scenarioManager.zeroState.header.txt}",
                buttons: [
                    new sas.hc.m.Button({
                        enabled: ctrl.canCreateScenario(),
                        press: $.proxy(ctrl.onCreateScenario, ctrl),
                        text: '{i18N>scenarioManager.zeroState.newButton.txt}',
                        visible: true
                    }).addStyleClass("sasPrimaryButton"),
                    new sas.hc.m.Button({
                        enabled: hasWorkGroups,
                        press: $.proxy(ctrl.onImportExcel, ctrl),
                        text: '{i18N>scenarioManager.zeroState.importButton.txt}',
                        visible: true
                    })
                ]
            }));

            ctrl._linkBaselines(view.getModel().getData().scenarios);

            setTimeout(function() {
                ctrl.closeBusyDialog("scenarioManager.refreshModel.txt");
            }, 500);

            ctrl._toggleScenarioManagerToolbarButtons();

            if (fromInit) {
                // Now that we've loaded scenarios and set the model, we can flag initialization as complete
                sas.risk.scenariomanager.ViewManager.setInitializationComplete(true);
                if (scenarioIdOpenRequested) {
                    ctrl._openScenarioById(scenarioIdOpenRequested);
                }
            }
        }).fail(function (event) {
            ctrl.closeBusyDialog("scenarioManager.refreshModel.txt");
            ctrl.onServerFailure(event);
        });
    },

    getScenarioFromTable: function (index) {
        var view = this.getView();

        // If index wasn't specified, then we are getting the currently selected index from the table.
        if (undefined === index) {
            var selections = view.scenarioTable.getSelectedIndices();
            if (selections !== null && selections.length > 0) {
                index = selections[0];
            } else {
                index = -1;
            }
        }

        var scenario = null;
        var context = view.scenarioTable.getContextByIndex(index);

        if (index > -1 && context) {
            scenario = context.getObject();
        }
        return scenario;
    },

    _openScenarioById: function (id) {
        var ctrl = this;
        var view = ctrl.getView();
        var model = view.getModel();
        var data = model.getProperty('/scenarios');
        var scenario = _.find(data, function (scenarioO) {
            return scenarioO.id === id;
        });
        if (scenario) {
            var scenarios = [scenario];
            ctrl.openScenarios(scenarios);
        } else {
            var title = ctrl.rb.getText('error.window.title');
            var msg = ctrl.rb.getText('scenarioManager.editorOpenFailedInvalidId.error.txt');
            sas.hc.m.MessageBox.error(msg, title);
        }
    },

    copyScenario: function (scenario) {

        var ctrl = this;
        var fragment = 'sas.risk.scenariomanager.view.SaveAsDialog';
        var title = ctrl.rb.getText('copyDialog.window.title');
        var subCtrl = new sas.risk.scenariomanager.utils.ScenarioPropertyController(fragment, title);
        subCtrl.setOKHandler(function () {
            ctrl.openBusyDialog("scenarioManager.copyScenario.txt");

            // TODO: make sure either label or version has changed -- if not, this will be a save of the original
            sas.risk.scenariomanager.ScenarioAPI.saveScenarios([subCtrl.getModel()]).done(function (saved) {
                sas.applicationSwitcher.broadcastMessage("ScenariosUpdated", []);
                ctrl.closeBusyDialog("scenarioManager.copyScenario.txt");
                sas.eventBus.publish(sas.risk.scenariomanager.EventBusConstants.CHANNEL_NAME,
                    sas.risk.scenariomanager.EventBusConstants.EXISTING_SCENARIO_MANAGER_OPENED,
                    { hideBusy: true });
                ctrl.openScenarios(saved.items);
            }).fail(function (event) {
                ctrl.closeBusyDialog("scenarioManager.copyScenario.txt");
                ctrl.onServerFailure(event);
            });
        });

        subCtrl.cloneScenario(ctrl, scenario);
    },

    onCreateScenario: function () {
        var ctrl = this;
        var subCtrl = new sas.risk.scenariomanager.utils.ScenarioPropertyController('sas.risk.scenariomanager.view.CreateScenarioDialog');
        subCtrl.setOKHandler(function () {
            var scenario = subCtrl.getModel();
            scenario.readOnly = false;
            ctrl.openScenarios([scenario]);
        });

        subCtrl.createScenario(ctrl, null);
    },

    createScenarioWithAppSwitchData: function (appSwitchData) {
        var ctrl = this;
        var subCtrl = new sas.risk.scenariomanager.utils.ScenarioPropertyController('sas.risk.scenariomanager.view.CreateScenarioDialog');
        subCtrl.setOKHandler(function () {
            var scenario = subCtrl.getModel();
            scenario.readOnly = false;
            ctrl.openScenarios([scenario]);
        });

        subCtrl.createScenario(ctrl, appSwitchData);
    },

    scenariosTableCellClick: function (event) {
        var scenarios = [];
        var scenario = this.getScenarioFromTable(event.getParameter("rowIndex"));
        if (null !== scenario) {
            scenarios.push(scenario);
            this.openScenarios(scenarios);
        }
    },

    onOpenScenariosButtonPress: function (event) {
        var view = this.getView();

        var indices = view.scenarioTable.getSelectedIndices();

        var scenarios = [];
        var that = this;
        _.each(indices, function (selectedIdx) {
            scenarios.push(that.getScenarioFromTable(selectedIdx));
        });
        this.openScenarios(scenarios);
    },

    openScenarios: function (scenarios) {
        sas.risk.scenariomanager.ViewManager.loadScenarioEditor(scenarios);
    },

    onCopyScenarios: function (event) {

        var ctrl = this;
        var scenario = this.getScenarioFromTable();

        if (scenario) {
            ctrl.openBusyDialog("scenarioManager.loadScenarioList.txt");
            sas.risk.scenariomanager.ScenarioAPI.getScenarioData(scenario).done(function (scenarioData) {
                ctrl.closeBusyDialog("scenarioManager.loadScenarioList.txt");
                ctrl.copyScenario(scenarioData);
            }).fail(function (event) {
                ctrl.closeBusyDialog("scenarioManager.loadScenarioList.txt");
                ctrl.onServerFailure(event);
            });
        }
    },

    getSelectedScenarios: function () {
        var view = this.getView();
        var indices = view.scenarioTable.getSelectedIndices();
        var scenarios = [];
        var that = this;
        _.each(indices, function (selectedIdx) {
            scenarios.push(that.getScenarioFromTable(selectedIdx));
        });
        return scenarios;
    },

    onDeleteScenarios: function (event) {
        var scenarios = this.getSelectedScenarios();

        if (scenarios.length > 0) {
            this._maybeDeleteScenarios(scenarios);
        }
    },

    onRefresh: function () {
        this.refreshModel();
    },

    onSensitivityAnalysis: function () {
        var ctrl = this;
        var scenarios = this.getSelectedScenarios();
        if (scenarios.length === 1) {
            var scenario = scenarios[0];
            sas.risk.scenariomanager.ViewManager.loadModelSensitivityAnalysis(scenario);
        }
    },

    onImportExcel: function () {
        var ctrl = this;

        var dialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ImportScenariosDialog');
        if (!dialog) {
            dialog = sap.ui.jsfragment('sas.risk.scenariomanager.view.ImportScenariosDialog', ctrl);
        }

        dialog.setModel(new sap.ui.model.resource.ResourceModel({
            bundleName: 'sas.risk.scenariomanager.nls.messagebundle'
        }), 'i18N');

        var availableWorkGroups = sas.risk.scenariomanager.ScenarioAPI.getWorkGroups();
        var workGroup = sas.risk.scenariomanager.ScenarioAPI.getRecommendedPrimaryWorkGroup();
        if (availableWorkGroups) {
            if (!workGroup) {
                workGroup = availableWorkGroups[0];
            }
            dialog.setModel(new sas.risk.common.models.RiskJSONModel(availableWorkGroups), 'availableWorkGroups');
            dialog.setModel(new sas.risk.common.models.RiskJSONModel(workGroup), 'workGroup');
        }

        dialog.open();

        ////Set value in date picker to user's preferred date, if set
        var datePicker = sap.ui.getCore().byId('ImportScenariosDialog.historyDate');

        var AppSettings = sas.risk.scenariomanager.settings.ApplicationSettings;
        var defaultDate = AppSettings.getDefaultScenarioDate();
        if (defaultDate) {
            datePicker.setDateValue(new Date(defaultDate));
        } else {
            datePicker.setDateValue(new Date());
        }

        ctrl.openBusyDialog("scenarioManager.loadServerFiles.txt");
        $.when(sas.risk.scenariomanager.ScenarioAPI.loadServerFiles(workGroup.name)).done(
            function (serverFiles) {
                ctrl.closeBusyDialog("scenarioManager.loadServerFiles.txt");
                dialog.setModel(new sas.risk.common.models.RiskJSONModel({ fileList: serverFiles }));
            }
        ).fail(function (jqxhr) {
            ctrl.closeBusyDialog("scenarioManager.loadServerFiles.txt");
            ctrl.onServerFailure(jqxhr);
        });

    },

    onOpenExportDialog: function() {
        var ctrl = this;
        var dialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ExportScenariosDialog');
        if (!dialog) {
            dialog = sap.ui.jsfragment('sas.risk.scenariomanager.view.ExportScenariosDialog', ctrl);
            dialog.setModel(new sap.ui.model.resource.ResourceModel({
                bundleName: 'sas.risk.scenariomanager.nls.messagebundle'
            }), 'i18N');
        }
        dialog.open();
    },

    onConfirmExport: function() {
        var ctrl = this;

        var segmentedButton = sap.ui.getCore().byId('ExportScenariosDialog.segmentedButton');
        var selectedButtonId = segmentedButton.getSelectedButton();
        var exportToServer = selectedButtonId === "ExportScenariosDialog.serverExportButton";

        if (exportToServer) {
            var workGroup;
            var scenarios = this.getSelectedScenarios();
            if (scenarios && scenarios.length > 0) {
                workGroup = scenarios[0].workGroup;
            }
            $.when( sas.risk.scenariomanager.ScenarioAPI.loadServerFiles(workGroup)).done(
                function(serverFiles){
                    // confirm if dataset name already exists on server

                    var fileNameInput = sap.ui.getCore().byId("ExportScenariosDialog.fileNameInput");
                    var outputFileName = fileNameInput.getValue();

                    var fileTypeSelect = sap.ui.getCore().byId("ExportScenariosDialog.fileTypeSelect");
                    var fileExtension = fileTypeSelect.getSelectedKey();
                    outputFileName = outputFileName + fileExtension;

                    // returns an empty array if not on server, else array of one flle
                    var existsOnServer = serverFiles.filter(function( file ) {
                        return file.fileName == outputFileName;
                    });

                    if (existsOnServer && existsOnServer.length === 1) {
                        var title = ctrl.rb.getText('export.scenarios.server.name.conflict.error.window.title');
                        var msg = ctrl.rb.getText('export.scenarios.server.name.conflict.error.message.txt');
                        var actions = [sas.hc.m.MessageBox.Action.OK, sas.hc.m.MessageBox.Action.CANCEL];
                        sas.hc.m.MessageBox.information(msg, title, actions, null, function (btn) {
                            if (btn === sas.hc.m.MessageBox.Action.OK) {
                                ctrl.onExportScenarios();
                            }
                        });
                    } else {
                        ctrl.onExportScenarios();
                    }
            });
        } else {
            ctrl.onExportScenarios();
        }
    },

    onExportScenarios: function () {
        var ctrl = this;

        var dialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ExportScenariosDialog');

        var segmentedButton = sap.ui.getCore().byId('ExportScenariosDialog.segmentedButton');
        var selectedButtonId = segmentedButton.getSelectedButton();
        var exportToServer = selectedButtonId === "ExportScenariosDialog.serverExportButton";

        var fileNameInput = sap.ui.getCore().byId("ExportScenariosDialog.fileNameInput");
        var outputFileName = fileNameInput.getValue();

        var fileTypeSelect = sap.ui.getCore().byId("ExportScenariosDialog.fileTypeSelect");
        var fileExtension = fileTypeSelect.getSelectedKey();

        // In order to export in formats other than sas7bdat, we need to add the extension to the file name, otherwise the exported file type will be sas dataset
        if (fileExtension !== '.sas7bdat') {
            outputFileName = outputFileName + fileExtension;
        }

        var openInSASStudioCheckbox = sap.ui.getCore().byId("ExportScenariosDialog.openInSASStudioCheckbox");
        var openInSASStudio = openInSASStudioCheckbox.getSelected();

        var scenarios = this.getSelectedScenarios();
        if (scenarios && scenarios.length > 0) {
            var workGroupName = scenarios[0].workGroup;
            var scenarioIds = [];
            _.each(scenarios, function (scenario) {
                scenarioIds.push(scenario.id);
            });
            sas.risk.scenariomanager.ScenarioAPI.exportScenarios(scenarioIds, outputFileName, workGroupName, exportToServer).done(function (data) {
                if (exportToServer) {
                    if (openInSASStudio && fileExtension === '.sas7bdat') {
                        ctrl.openServerDataSetInSASStudio(outputFileName, data.location);
                    }
                } else {
                    var streamLink = sas.risk.scenariomanager.ScenarioAPI.findOptionalLink(data.links, { rel: 'exportGetResults' });
                    if (streamLink) {
                        window.open(streamLink.href, "_self");
                    }
                }
                dialog.close();
            }).fail(function (event) {
                ctrl.onServerFailure(event);
            });
        }
    },
    
    openServerDataSetInSASStudio: function(fileName, filePath) {
        sas.risk.common.utils.SASStudioProxy.launchStudio();
        var libname = "libname sm_scens \'" + filePath + "\';";
        sas.risk.common.utils.SASStudioProxy.submitCode(libname);
        sas.risk.common.utils.SASStudioProxy.openTable("sm_scens." + fileName);
    },

    onServerFailure: function (event) {
        var ctrl = this;

        var title = ctrl.rb.getText('error.window.title');
        var msg = ctrl.rb.getText('error.message.txt');
        var response = event.responseJSON;
        if (response) {
            sas.risk.common.widgets.MessageDetailsBox.error(response, title);
        } else {
            sas.hc.m.MessageBox.error(msg, title);
        }
    },

    // Fills in the baselineName, baselineVersion properties in the given collection of scenarios
    _linkBaselines: function (scenarios) {
        // Build up an index of the baselines in the set (two passes removes the need to sort;
        // since sorting would involve at least one pass, it wouldn't actually save us anything)

        // NOTE: dividing by 1000 is a hack that takes advantage of knowing how the id's are assigned.
        // On current javascript engines, we're going to end up with an internal dictionary for the sparse
        // array anyway, but on older engines that may be using an actual array, dividing keeps the array
        // from being 1000x larger than necessary
        var baseIdx = [];
        _.each(scenarios, function (scenario) {
            if (scenario.baselineId === "0") {
                baseIdx[scenario.id / 1000] = scenario;
            }
        });

        return _.filter(scenarios, function (scenario) {
            if (scenario.baselineId <= 0) {
                return true;
            }

            var baseline = baseIdx[scenario.baselineId / 1000];
            if (!baseline) {
                return false;
            }

            scenario.baselineLabel = baseline.label;
            scenario.baselineVersion = baseline.scenarioVersion;
            return true;
        });
    },

    _maybeDeleteScenarios: function (scenarios) {
        var ctrl = this;

        var existingDialog = sap.ui.getCore().byId("deleteScenariosDialog");
        if (existingDialog) {
            existingDialog.open();
        } else {
            var dialog = sap.ui.jsfragment("sas.risk.scenariomanager.view.DeleteScenariosDialog", ctrl);
            dialog.setModel(new sap.ui.model.resource.ResourceModel({
                bundleName: 'sas.risk.scenariomanager.nls.messagebundle'
            }), 'i18N');
            ctrl.openBusyDialog();
            $.when.apply($, ctrl.checkIfBaselineSelected(scenarios)).done(function () {
                ctrl.closeBusyDialog();
                dialog.open();
            }).fail(function (event) {
                ctrl.closeBusyDialog();
                ctrl.onServerFailure(event);
                dialog.destroy();
            });
        }
    },

    checkIfBaselineSelected: function (scenarios) {
        var deferred = [];
        for (var scenarioIndex = 0; scenarioIndex < scenarios.length; ++scenarioIndex) {
            if (scenarios[scenarioIndex].baselineId === '0') {
                var isUsedAsBaselineCall = this.isUsedAsBaseline(scenarios[scenarioIndex])
                                               .done($.proxy(this.enableDeleteScenariosCascadeDelete, this));
                deferred.push(isUsedAsBaselineCall);
            }
        }
        return deferred;
    },

    isUsedAsBaseline: function (scenario) {
        var ctrl = this;
        var deferred = $.Deferred();
        sas.risk.scenariomanager.ScenarioAPI.getScenariosForBaseline(scenario.id).done(function (consumingScenarios) {
            deferred.resolve(consumingScenarios.length > 0);
        }).fail(function (event) {
            ctrl.onServerFailure(event);
            deferred.reject(event);
        });
        return deferred;
    },

    enableDeleteScenariosCascadeDelete: function (isUsedAsBaseline) {
        if (true === isUsedAsBaseline || undefined === isUsedAsBaseline) {
            this.getView().byId('deleteScenariosDialog.cascadeDeleteCheckbox').setVisible(true);
        }
    },

    _doDeleteScenarios: function (cascade) {
        var ctrl = this;
        var view = this.getView();
        var scenarios = this.getSelectedScenarios();
        if (scenarios) {
            sas.risk.scenariomanager.ScenarioAPI.deleteScenarios(scenarios, cascade).done(function () {
                var deletedInfo = [];
                _.each(scenarios, function (scenario) {
                    var dsType = (scenario.dataSourceType === 'S') ? 'stress' :
                        (scenario.dataSourceType === 'P') ? 'price' : '';
                    deletedInfo.push({
                        scenarioId: scenario.id,
                        scenarioWorkGroup: scenario.workGroup,
                        scenarioName: scenario.name,
                        scenarioVersion: scenario.scenarioVersion,
                        dataSourceId: scenario.dataSourceId,
                        dataSourceParms: sas.risk.scenariomanager.ScenarioAPI.encodeArgsAsObject(scenario.dataSourceArgs),
                        dataSourceType: dsType,
                        action: "DELETE"
                    });
                });
                sas.applicationSwitcher.broadcastMessage("ScenariosUpdated", deletedInfo);
                view.scenarioTable.clearSelection();
                ctrl.onRefresh();
            }).fail(function (event) {
                ctrl.onServerFailure(event);
            });
        }
    },

    onLockButton: function () {
        var ctrl = this;
        var scenario = this.getSelectedScenarios()[0];
        var view = this.getView();

        if (scenario) {
            var rel = (scenario.readOnly === true) ? "unlock" : "lock";
            var busyKey = (scenario.readOnly === true) ? 'scenarioEditor.unlockScenario.txt' :
                          'scenarioEditor.lockScenario.txt';
            ctrl.openBusyDialog(busyKey);
            view.lockButton.setSelected(scenario.readOnly);

            sas.risk.scenariomanager.ScenarioAPI.putScenario(scenario, rel).done(function (scenarioSummary) {
                ctrl.refreshModel();
                ctrl.closeBusyDialog(busyKey);
            }).fail(function (event) {
                ctrl.closeBusyDialog(busyKey);
                ctrl.onServerFailure(event);
            });
        }
    },

    // ---------------------------
    // Import Scenarios controller
    // ---------------------------

    // TODO: move this to its own class
    onImportBtnClick: function () {

        var ctrl = this;
        var deferred = $.Deferred().fail(function (event) {
            ctrl.onServerFailure(event);
        });

        var toWorkGroupName;
        var toWorkGroupControl = ctrl.getView().byId("importScenariosToWorkGroupField");
        var selectedToWorkGroup = toWorkGroupControl.getSelectedItem();
        if (selectedToWorkGroup) {
            toWorkGroupName = selectedToWorkGroup.data('workGroup').name;
        }

        var dateFormat = sap.ui.core.format.DateFormat.getDateInstance({ pattern: "yyyy-MM-dd" }),
            importBox = sap.ui.getCore().byId('ImportScenariosDialog.importHistoryBox'),
            datePicker = sap.ui.getCore().byId('ImportScenariosDialog.historyDate'),
            dateValue = datePicker.getValue(),
            hasHistoryDate = (importBox.getSelected() && dateValue),
            dateStr = hasHistoryDate ? dateFormat.format(new Date(dateValue), true) : "";

        var serverImport = sap.ui.getCore().byId('ImportScenariosDialog.filePicker').getVisible();
        if (serverImport) {
            var fromWorkGroupName;
            var selectedFromWorkGroup = this.getView().byId("importScenariosFromWorkGroupField").getSelectedItem();
            if (selectedFromWorkGroup) {
                fromWorkGroupName = selectedFromWorkGroup.data('workGroup').name;
            }
            var filePicker = sap.ui.getCore().byId('ImportScenariosDialog.filePicker');
            var dataSource = filePicker.getSelectedItem();
            ctrl.openBusyDialog("importScenariosDialog.busyDialog.title");
            sas.risk.scenariomanager.ScenarioAPI.importFile(dataSource.fileName, fromWorkGroupName, dateStr).done(function (response) {
                ctrl.onFileSelected(response);
            }).fail(function (event) {
                ctrl.closeBusyDialog("importScenariosDialog.busyDialog.title");
                ctrl.onServerFailure(event);
            });
        } else {
            var rootLinks = sas.risk.scenariomanager.ScenarioAPI.getRootLinks();
            var link = sas.risk.scenariomanager.ScenarioAPI.findRequiredLink(deferred, rootLinks, { rel: 'reportScenarios' });
            if (!link) {
                return;
            }
            var url = link.href;

            if (hasHistoryDate) {
                url += "?historyDate=" + dateStr;
            }
            if (toWorkGroupName !== undefined) {
                if (!hasHistoryDate) {
                    // No history date specified, need to add first url arg
                    url += "?";
                } else {
                    // History date exists, adding additional url arg
                    url += "&";
                }
                url += "workGroup=" + toWorkGroupName;
            }

            var fileUploader = sap.ui.getCore().byId('ImportScenariosDialog.fileUploader');
            fileUploader.setUploadUrl(url);
            if (fileUploader.getValue()) {
                ctrl.openBusyDialog("importScenariosDialog.busyDialog.title");
                fileUploader.upload();
            }
        }
    },

    onFromWorkGroupChangeHandler: function (event) {
        var selectedFromWorkGroup = this.getView().byId("importScenariosFromWorkGroupField").getSelectedItem();
        if (selectedFromWorkGroup) {
            var fromWorkGroupName = selectedFromWorkGroup.data('workGroup').name;

            $.when(sas.risk.scenariomanager.ScenarioAPI.loadServerFiles(fromWorkGroupName)).done(
                function (serverFiles) {

                    var dialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ImportScenariosDialog');

                    dialog.getModel().setData({ fileList: serverFiles });

                }
            );
        }
    },

    onFileSelected: function (response) {
        var ctrl = this;
        ctrl.closeBusyDialog("importScenariosDialog.busyDialog.title");

        var dialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ImportScenariosDialog');
        var fromWorkGroupName;
        var selectedFromWorkGroup = this.getView().byId("importScenariosFromWorkGroupField").getSelectedItem();
        if (selectedFromWorkGroup) {
            fromWorkGroupName = selectedFromWorkGroup.data('workGroup').name;
        }
        dialog.setVisible(false);

        if (response.hasOwnProperty('errorCode')) {
            dialog.setVisible(true);
            sas.risk.common.widgets.MessageDetailsBox.error(response, sas.config.appName);
        } else {
            ctrl.showUploadReport(response.items, fromWorkGroupName);
        }
    },

    onFileUploaded: function (event) {

        var dialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ImportScenariosDialog');
        var toWorkGroupName;
        var toWorkGroupControl = this.getView().byId("importScenariosToWorkGroupField");
        var selectedToWorkGroup = toWorkGroupControl.getSelectedItem();
        if (selectedToWorkGroup) {
            toWorkGroupName = selectedToWorkGroup.data('workGroup').name;
        }
        dialog.setVisible(false);

        var ctrl = this;

        ctrl.closeBusyDialog("importScenariosDialog.busyDialog.title");
        var response = event.getParameter('responseRaw');
        if (response) {
            var responseJSON = JSON.parse(response);
            if (responseJSON.hasOwnProperty('errorCode')) {
                dialog.setVisible(true);
                sas.risk.common.widgets.MessageDetailsBox.error(responseJSON, sas.config.appName);
            } else {
                ctrl.showUploadReport(responseJSON.items, toWorkGroupName);
            }
        }

    },

    showUploadReport: function (reportItems, workGroupName) {

        var ctrl = this;

        var dialog = sap.ui.getCore().byId('sas.risk.scenariomanager.view.ImportSummaryDialog');
        if (!dialog) {
            dialog = sap.ui.jsfragment('sas.risk.scenariomanager.view.ImportSummaryDialog', ctrl);
        }

        dialog.setModel(new sap.ui.model.resource.ResourceModel({
            bundleName: 'sas.risk.scenariomanager.nls.messagebundle'
        }), 'i18N');

        dialog.setModel(new sas.risk.common.models.RiskJSONModel({
            reportItems: reportItems,
            addedCount: reportItems[0].length,
            updatedCount: reportItems[1].length,
            affectedCount: reportItems[2].length,
            workGroup: workGroupName
        }));

        dialog.open();

    },

    onTypeMissmatch: function (event) {
        var ctrl = this;
        var aFileTypes = event.getSource().getFileType();
        $.each(aFileTypes, function (key, value) {
            aFileTypes[key] = '*.' + value;
        });
        var sSupportedFileTypes = aFileTypes.join(', ');
        sas.hc.m.MessageBox.error(
            ctrl.rb.getText('fileUploadTypeMismatch.msg.fmt.txt', [event.getParameter('fileType'),
                                                                   sSupportedFileTypes]),
            ctrl.rb.getText('fileUploadTypeMismatch.msg.title.txt')
        );

    },

    onFileSizeExceed: function (event) {
        var ctrl = this;
        sas.hc.m.MessageToast.show(ctrl.rb.getText('fileUploadFileSizeExceed.msg.fmt.txt', [event.getSource().getMaximumFileSize()]), { at: sap.ui.core.Popup.Dock.CenterBottom });
    },

    onUserConfirmation: function (event) {

        var reportItems = event.getSource().getModel().getProperty("/reportItems");

        var ctrl = this;
        var added = reportItems[0];
        var updated = reportItems[1];
        var allItems = added.concat(updated);

        sap.ui.getCore().byId('sas.risk.scenariomanager.view.ImportSummaryDialog').close();
        sap.ui.getCore().byId('sas.risk.scenariomanager.view.ImportScenariosDialog').close();

        if (reportItems.length > 3) {
            var history = reportItems[3];
            allItems = allItems.concat(history);
        }

        ctrl.openBusyDialog("importScenariosDialog.busyDialog.title");
        sas.risk.scenariomanager.ScenarioAPI.saveScenarios(allItems).done(function (data) {
            var updatedInfo = [];
            _.each(updated, function (scenario) {
                var dsType = (scenario.dataSourceType === 'S') ? 'stress' :
                    (scenario.dataSourceType === 'P') ? 'price' : '';
                updatedInfo.push({
                    scenarioId: scenario.id,
                    scenarioWorkGroup: scenario.workGroup,
                    scenarioName: scenario.name,
                    scenarioVersion: scenario.scenarioVersion,
                    dataSourceId: scenario.dataSourceId,
                    dataSourceParms: sas.risk.scenariomanager.ScenarioAPI.encodeArgsAsObject(scenario.dataSourceArgs),
                    dataSourceType: dsType,
                    action: "UPDATE"
                });
            });
            sas.applicationSwitcher.broadcastMessage("ScenariosUpdated", updatedInfo);
            ctrl.closeBusyDialog("importScenariosDialog.busyDialog.title");
            ctrl.onImportComplete(data.items);
        }).fail(function (event) {
            ctrl.closeBusyDialog("importScenariosDialog.busyDialog.title");
            ctrl.onServerFailure(event);
        });
    },

    onImportComplete: function (items) {
        var ctrl = this;
        var msg = ctrl.rb.getText('importScenario.scenariosWereImportedMsg.txt', items.length);
        sas.hc.m.MessageToast.show(msg, { at: sap.ui.core.Popup.Dock.CenterBottom });
        ctrl.refreshModel();
    },

    openBusyDialog: function (titleKey) {
        if (!titleKey) {
            titleKey = "default.busyDialog.title";
        }
        if (!this.busyDialogs) {
            this.busyDialogs = {};
        }

        if (!this.busyDialogs[titleKey]) {
            var busyDialog = new sas.hc.m.BusyDialog({
                title: this.rb.getText(titleKey)
            });
            busyDialog.setEarcon(null);
            this.busyDialogs[titleKey] = busyDialog;
            busyDialog.open();
        }
    },

    closeBusyDialog: function (titleKey) {
        if (!titleKey) {
            titleKey = "default.busyDialog.title";
        }
        if (!this.busyDialogs) {
            this.busyDialogs = {};
        } else if (this.busyDialogs[titleKey]) {
            this.busyDialogs[titleKey].close();
            this.busyDialogs[titleKey].destroy();
            this.busyDialogs = _.omit(this.busyDialogs, titleKey);
        }
    },

    closeAllBusyDialogs: function () {
        if (!this.busyDialogs) {
            this.busyDialogs = {};
        } else {
            var titleKeys = Object.keys(this.busyDialogs);
            for (var index = 0; index < titleKeys.length; ++index) {
                var busyDialog = this.busyDialogs[titleKeys[index]];
                if (busyDialog) {
                    busyDialog.close();
                    busyDialog.destroy();
                }
                this.busyDialogs = _.omit(this.busyDialogs, titleKeys[index]);
            }
        }
    },

    onScenarioManagerTableRowSelectionChanged: function () {
        this._toggleScenarioManagerToolbarButtons();
    },

    _toggleScenarioManagerToolbarButtons: function () {
        var view = this.getView();
        var scenarios = this.getSelectedScenarios();
        var numSelected = scenarios.length;
        var hasSelections = (numSelected > 0);
        // The work groups are loaded by the shell, so we don't need deferred
        var workGroups = sas.risk.scenariomanager.ScenarioAPI.getWorkGroups();
        var hasWorkGroups = (workGroups !== null && workGroups !== undefined && workGroups.length > 0);
        if (!hasWorkGroups) {
            view.noWorkGroupsErrorMessage.setVisible(true);
            view.scenarioPanel.getTop().setSize("3.5rem");
            view.scenarioPanel.getTop().setVisible(true);
        }

        view.openButton.setEnabled(hasWorkGroups && numSelected === 1);
        view.lockButton.setEnabled(hasWorkGroups && numSelected === 1 && this.canLockOrUnlockScenario(scenarios[0]));
        view.deleteButton.setEnabled(hasWorkGroups && hasSelections && this.canDeleteScenarios(scenarios));
        view.copyButton.setEnabled(hasWorkGroups && numSelected === 1);
        view.sensitivityAnalysisButton.setEnabled(hasWorkGroups && numSelected === 1);
        view.exportButton.setEnabled(hasWorkGroups && hasSelections && this.canExportScenarios(scenarios));
        view.importButton.setEnabled(hasWorkGroups);

        // Update lock checkmark in overflow menu
        var isLocked = numSelected === 1 && scenarios[0].readOnly;
        view.lockButton.setSelected(isLocked);

    },

    canCreateScenario: function () {
        var deferred = $.Deferred();
        var rootLinks = sas.risk.scenariomanager.ScenarioAPI.getRootLinks();
        var linkDesc = {
            rel: 'createScenario',
            type: 'application/vnd.sas.risk.scenario+json;charset=utf-8',
            responseType: 'application/vnd.sas.risk.scenario+json;charset=utf-8'
        };

        var link = sas.risk.scenariomanager.ScenarioAPI.findRequiredLink(deferred, rootLinks, linkDesc);
        // The work groups are loaded by the shell, so we don't need deferred
        var workGroups = sas.risk.scenariomanager.ScenarioAPI.getWorkGroups();
        return !!link && workGroups !== undefined && workGroups !== null && workGroups.length > 0;
    },

    // this method assumes that a scenario with baselineId === "0" means no baseline is linked
    canDeleteScenarios: function (scenarios) {
        var view = this.getView();

        // Find a selected item with no 'delete' link
        var protectedItem = _.find(scenarios, function (item, idx) {
            var deleteLink = view.findOptionalLink(item.links, { rel: 'delete' });
            return !deleteLink;
        });

        return !protectedItem;
    },

    canLockOrUnlockScenario: function (scenario) {
        var deferred = $.Deferred();
        var linkRel = (scenario.readOnly === true) ? "unlock" : "lock";
        var link = sas.risk.scenariomanager.ScenarioAPI.findRequiredLink(deferred, scenario.links, { rel: linkRel });
        return !!link;
    },

    // Confirm all scenarios are in the same workgroup
    canExportScenarios: function (scenarios) {
        if (scenarios.length === 1) {
            return true;
        } else if (scenarios.length === 0) {
            return false;
        }

        var uniqueWorkGroups = _.uniq(_.pluck(scenarios, 'workGroup'));

        return uniqueWorkGroups.length === 1;
    },

    onFilterScenarios: function (event) {
        var view = this.getView();
        var searchFieldValue = (view.byId("searchField").getValue() || "").toUpperCase();
        var item = view.byId("workGroupField").getSelectedItem();
        var selectedWorkGroup = item.data('workGroup');
        var workGroupValue = "";
        if (selectedWorkGroup && selectedWorkGroup.name) {
           workGroupValue = selectedWorkGroup.name; 
        }


        // NOTE: keep this in sync with the table's format to keep filtering on formatted value in sync
        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();

        var andFilters = [];
        if (workGroupValue !== undefined && workGroupValue !== null && workGroupValue !== "") {
            andFilters.push(new sap.ui.model.Filter('workGroup', sap.ui.model.FilterOperator.EQ, workGroupValue));
        }

        var orFilters = [];
        orFilters.push(new sap.ui.model.Filter('label', sap.ui.model.FilterOperator.Contains, searchFieldValue));
        orFilters.push(new sap.ui.model.Filter('scenarioVersion', sap.ui.model.FilterOperator.Contains, searchFieldValue));
        orFilters.push(new sap.ui.model.Filter('baselineLabel', sap.ui.model.FilterOperator.Contains, searchFieldValue));
        orFilters.push(new sap.ui.model.Filter('baselineVersion', sap.ui.model.FilterOperator.Contains, searchFieldValue));
        orFilters.push(new sap.ui.model.Filter('description', sap.ui.model.FilterOperator.Contains, searchFieldValue));
        orFilters.push(new sap.ui.model.Filter('modifiedBy', sap.ui.model.FilterOperator.Contains, searchFieldValue));
        orFilters.push(new sap.ui.model.Filter('createdBy', sap.ui.model.FilterOperator.Contains, searchFieldValue));
        orFilters.push(new sap.ui.model.Filter('periodType', function (val) {
            var label = new sas.risk.scenariomanager.utils.IntervalTypeUtil().getDisplayText(val, false, 1)
                                                                             .toUpperCase();
            return label.indexOf(searchFieldValue.toUpperCase()) >= 0;
        }));
        orFilters.push(new sap.ui.model.Filter('modifiedTimeStamp', function (dttm) {
            var label = ((dttm) ? dateTimeFormat.format(new Date(dttm)) : '').toUpperCase();
            return label.indexOf(searchFieldValue) >= 0;
        }));
        orFilters.push(new sap.ui.model.Filter('creationTimeStamp', function (dttm) {
            var label = ((dttm) ? dateTimeFormat.format(new Date(dttm)) : '').toUpperCase();
            return label.indexOf(searchFieldValue.toUpperCase()) >= 0;
        }));
        orFilters.push(new sap.ui.model.Filter('dataSourceName', function (val) {
            var label = val.replace(/\.rskcdesc$/i, '').toUpperCase();
            return label.indexOf(searchFieldValue.toUpperCase()) >= 0;
        }));

        // These two work against the ISO8601 date format
        //var modifiedFilter = new sap.ui.model.Filter('modifiedTimeStamp',sap.ui.model.FilterOperator.Contains, value);
        //var createdFilter = new sap.ui.model.Filter('creationTimeStamp',sap.ui.model.FilterOperator.Contains, value);

        var filters = [];
        if (andFilters.length > 0) {
            filters.push(new sap.ui.model.Filter({
                filters: andFilters,
                and: true
            }));
        }
        filters.push(new sap.ui.model.Filter({
            filters: orFilters,
            and: false
        }));

        var allFilters = new sap.ui.model.Filter({
            filters: filters, and: true
        });

        var binding = view.scenarioTable.getBinding('rows');
        binding.filter(allFilters);
    },

    findScenario: function(workGroupName, scenarioName, scenarioVersion){
        var view = this.getView();
        var scenarios = view.getModel().getData().scenarios;
        return _.find(scenarios, function (scen) {
            // Does the work group match?
            if ((!scen.workGroup && !workGroupName) || (workGroupName && scen.workGroup && workGroupName.toLocaleUpperCase() === scen.workGroup.toLocaleUpperCase())) {
                // Do the names both exist and do they match? (case insensitive)
                if ((scen.name && scenarioName) && scen.name.toLocaleUpperCase() === scenarioName.toLocaleUpperCase()) {
                    // Are the scenario versions either unset or the same?
                    return ((!scen.scenarioVersion && !scenarioVersion) || (scen.scenarioVersion && scenarioVersion && scen.scenarioVersion.toLocaleUpperCase() === scenarioVersion.toLocaleUpperCase()));
                }
            }
            return false;
        });
    }

});
